@@ -248,11 +248,13 @@ for element_rel in dvipng.exe \
                    libdav1d.dll \
                    rav1e.dll \
                    libde265-0.dll \
-                   libx265.dll
+                   libx265.dll \
+                   libSvtAv1Enc.dll \
+                   libyuv.dll
 do
   cp -v ${OLD_MINGW64_FOLDER}/bin/${element_rel} ${NEW_MINGW64_FOLDER}/bin/
 done
 cp -v ${OLD_MINGW64_FOLDER}/var/lib/texmf/fonts/map/dvips/updmap/ps2pk.map ${NEW_MINGW64_FOLDER}/bin/
-# fix issue with mingw64/ssl not installed properly in --root ${NEW_MSYS2_FOLDER}
-rm -rf ${NEW_MINGW64_FOLDER}/ssl
-cp -rv ${OLD_MINGW64_FOLDER}/ssl ${NEW_MINGW64_FOLDER}/
+# fix issue with mingw64/etc/ssl not installed properly in --root ${NEW_MSYS2_FOLDER}
+rm -rf ${NEW_MINGW64_FOLDER}/etc/ssl
+cp -rv ${OLD_MINGW64_FOLDER}/etc/ssl ${NEW_MINGW64_FOLDER}/etc/
