@@ -1,7 +1,7 @@
 /*
  * ct_actions.h
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -49,7 +49,7 @@ public:
 
 public:
     CtCodebox*      curr_codebox_anchor{nullptr};
-    CtTable*        curr_table_anchor{nullptr};
+    CtTableCommon*  curr_table_anchor{nullptr};
     CtImageAnchor*  curr_anchor_anchor{nullptr};
     CtImageLatex*   curr_latex_anchor{nullptr};
     CtImagePng*     curr_image_anchor{nullptr};
@@ -92,6 +92,7 @@ public: // todo: fix naming
     bool          _is_curr_node_not_read_only_or_error();
     bool          _is_curr_node_not_syntax_highlighting_or_error(bool plain_text_ok = false);
     bool          _is_there_text_selection_or_error();
+    bool          _is_there_anch_widg_selection_or_error(const char anch_widg_id);
 
 public:
     void object_set_selection(CtAnchoredWidget* widget);
@@ -140,6 +141,7 @@ private:
                          bool ascendings);
     bool _tree_sort_level_and_sublevels(const Gtk::TreeNodeChildren& children,
                                         bool ascending);
+    void _node_date(const bool from_sel_not_root);
 
 public:
     // tree actions
@@ -152,7 +154,7 @@ public:
         _node_add(true/*is_duplicate*/, false/*add_as_child*/, &treeIter, _pCtMainWin);
     }
     void node_child_add()          {
-        if (!_is_there_selected_node_or_error()) return;
+        if (not _is_there_selected_node_or_error()) return;
         _node_add(false/*is_duplicate*/, true/*add_as_child*/);
     }
     void node_subnodes_duplicate();
@@ -164,7 +166,8 @@ public:
     void node_inherit_syntax();
     void node_delete();
     void node_toggle_read_only();
-    void node_date();
+    void node_date_from_root() { _node_date(false/*from_sel_not_root*/); }
+    void node_date_from_sel() { _node_date(true/*from_sel_not_root*/); }
     void node_up();
     void node_down();
     void node_right();
@@ -256,17 +259,20 @@ private:
 public:
     // view actions
     void toggle_show_hide_tree();
+    void toggle_show_hide_vte();
     void toggle_show_hide_menubar();
     void toggle_show_hide_toolbars();
     void toggle_show_hide_statusbar();
     void toggle_show_hide_tree_lines();
     void toggle_show_hide_node_name_header();
-    void toggle_tree_text();
+    void toggle_focus_tree_text();
+    void toggle_focus_vte_text();
     void nodes_expand_all();
     void nodes_collapse_all();
     void toolbar_icons_size_increase();
     void toolbar_icons_size_decrease();
-    void fullscreen_toggle();
+    void toggle_fullscreen();
+    void toggle_always_on_top();
     void disable_menubar_in_titlebar() { _menubar_in_titlebar_set(false); }
     void enable_menubar_in_titlebar() { _menubar_in_titlebar_set(true); }
 
@@ -283,11 +289,11 @@ private:
         CtTextView*  text_view;
         std::string  syntax_highl;
         CtCodebox*   codebox;
-        CtTable*     table;
+        CtTableHeavy*     table;
     };
     text_view_n_buffer_codebox_proof _get_text_view_n_buffer_codebox_proof();
     CtCodebox* _codebox_in_use();
-    CtTable* _table_in_use();
+    CtTableCommon* _table_in_use();
     void _save_tags_at_cursor_as_latest(Glib::RefPtr<Gtk::TextBuffer> rTextBuffer,
                                         int cursorOffset);
     bool _links_entries_pre_dialog(const Glib::ustring& curr_link,
@@ -391,6 +397,7 @@ private:
                              Gtk::TextIter insert_iter,
                              Gtk::TextIter* iter_bound);
     bool _on_embfiles_sentinel_timeout();
+    void _exec_code(const bool is_all);
 
 public:
     // others actions
@@ -431,7 +438,8 @@ public:
     void codebox_delete();
     void codebox_delete_keeping_text();
     void codebox_change_properties();
-    void exec_code();
+    void exec_code_all() { _exec_code(true/*is_all*/); }
+    void exec_code_line_or_selection() { _exec_code(false/*is_all*/); }
     void codebox_load_from_file();
     void codebox_save_to_file();
     void codebox_increase_width();
@@ -476,8 +484,6 @@ public:
     void import_nodes_from_zim_directory() noexcept;
     void import_node_from_md_file() noexcept;
     void import_nodes_from_md_directory() noexcept;
-    void import_node_from_pandoc() noexcept;
-    void import_directory_from_pandoc() noexcept;
     void import_nodes_from_gnote_directory() noexcept;
     void import_nodes_from_tomboy_directory() noexcept;
     void import_nodes_from_keepnote_directory() noexcept;
@@ -520,4 +526,9 @@ public:
     void online_help();
     void folder_cfg_open();
     void dialog_about();
+
+public:
+    void terminal_copy();
+    void terminal_paste();
+    void terminal_reset();
 };
