@@ -1,7 +1,7 @@
 /*
  * ct_actions_find.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -103,12 +103,16 @@ void CtActions::image_insert()
 
 void CtActions::table_insert()
 {
-    if (!_node_sel_and_rich_text()) return;
-    if (!_is_curr_node_not_read_only_or_error()) return;
-    CtDialogs::TableHandleResp res = CtDialogs::table_handle_dialog(_pCtMainWin, _("Insert Table"), true/*is_insert*/);
+    if (not _node_sel_and_rich_text()) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
+    bool is_light{_pCtConfig->tableColumns*_pCtConfig->tableRows > _pCtConfig->tableCellsGoLight};
+    CtDialogs::TableHandleResp res = CtDialogs::table_handle_dialog(
+        _pCtMainWin,
+        _("Insert Table"),
+        true/*is_insert*/,
+        is_light);
     if (res == CtDialogs::TableHandleResp::Cancel) return;
 
-    const int col_width = _pCtConfig->tableColWidthDefault;
     std::list<std::vector<std::string>> rows;
     if (res == CtDialogs::TableHandleResp::Ok) {
         std::vector<std::string> empty_row(_pCtConfig->tableColumns, "");
@@ -116,34 +120,40 @@ void CtActions::table_insert()
             rows.push_back(empty_row);
         }
     }
-    CtTable* pCtTable{nullptr};
+    CtTableMatrix tbl_matrix;
+    const auto charOffset = _curr_buffer()->get_insert()->get_iter().get_offset();
+    int col_width = _pCtConfig->tableColWidthDefault;
     if (res == CtDialogs::TableHandleResp::OkFromFile) {
         CtDialogs::FileSelectArgs args{_pCtMainWin};
         args.curr_folder = _pCtConfig->pickDirCsv;
-        args.curr_file_name = "";
+        args.curr_file_name.clear();
         args.filter_name = _("CSV File");
         args.filter_pattern = {"*.csv"};
 
         std::string filepath = CtDialogs::file_select_dialog(args);
         if (filepath.empty()) return;
         _pCtConfig->pickDirCsv = Glib::path_get_dirname(filepath);
-        pCtTable = CtTable::from_csv(filepath, _pCtMainWin, _curr_buffer()->get_insert()->get_iter().get_offset(), "").release();
+        CtTableCommon::populate_table_matrix_from_csv(filepath, _pCtMainWin, is_light, tbl_matrix);
+        col_width = 60;
     }
-
-    if (!pCtTable) {
-        CtTableMatrix tableMatrix;
+    else {
         for (auto& row : rows) {
-            tableMatrix.push_back(CtTableRow{});
+            tbl_matrix.push_back(CtTableRow{});
             for (auto& cell : row) {
-                tableMatrix.back().push_back(new CtTextCell{_pCtMainWin, cell, CtConst::TABLE_CELL_TEXT_ID});
+                if (is_light) {
+                    tbl_matrix.back().push_back(new Glib::ustring{});
+                } else {
+                    tbl_matrix.back().push_back(new CtTextCell{_pCtMainWin, cell, CtConst::TABLE_CELL_TEXT_ID});    
+                }
             }
         }
-        pCtTable = new CtTable{_pCtMainWin,
-                               tableMatrix,
-                               col_width,
-                               _curr_buffer()->get_insert()->get_iter().get_offset(),
-                               "",
-                               CtTableColWidths{}};
+    }
+    CtTableCommon* pCtTable{nullptr};
+    if (is_light) {
+        pCtTable = new CtTableLight{_pCtMainWin, tbl_matrix, col_width, charOffset, "", CtTableColWidths{}};
+    }
+    else {
+        pCtTable = new CtTableHeavy{_pCtMainWin, tbl_matrix, col_width, charOffset, "", CtTableColWidths{}};
     }
     Glib::RefPtr<Gsv::Buffer> gsv_buffer = Glib::RefPtr<Gsv::Buffer>::cast_dynamic(_curr_buffer());
     pCtTable->insertInTextBuffer(gsv_buffer);
@@ -198,7 +208,7 @@ void CtActions::embfile_insert_path(const std::string& filepath)
 
     if (fs::file_size(filepath) > static_cast<uintmax_t>(_pCtConfig->embfileMaxSize * 1024 * 1024)) {
         bool is_sqlite = fs::get_doc_type(_pCtMainWin->get_ct_storage()->get_file_path()) == CtDocType::SQLite;
-        auto message = str::format(_("The Maximum Size for Embedded Files is %s MB"), _pCtConfig->embfileMaxSize);
+        auto message = str::format(_("The Maximum Size for Embedded Files is %s MB."), _pCtConfig->embfileMaxSize);
         if (is_sqlite) {
             if (!CtDialogs::question_dialog(message + "\n" + _("Do you want to Continue?"), *_pCtMainWin))
                 return;
@@ -554,6 +564,13 @@ void CtActions::cut_as_plain_text()
 // Copy as Plain Text
 void CtActions::copy_as_plain_text()
 {
+#if defined(HAVE_VTE)
+    Gtk::Widget* pVte = _pCtMainWin->get_vte();
+    if (pVte and pVte->has_focus()) {
+        terminal_copy();
+        return;
+    }
+#endif // HAVE_VTE
     CtClipboard::force_plain_text();
     auto proof = _get_text_view_n_buffer_codebox_proof();
     g_signal_emit_by_name(G_OBJECT(proof.text_view->gobj()), "copy-clipboard");
@@ -562,6 +579,13 @@ void CtActions::copy_as_plain_text()
 // Paste as Plain Text
 void CtActions::paste_as_plain_text()
 {
+#if defined(HAVE_VTE)
+    Gtk::Widget* pVte = _pCtMainWin->get_vte();
+    if (pVte and pVte->has_focus()) {
+        terminal_paste();
+        return;
+    }
+#endif // HAVE_VTE
     auto proof = _get_text_view_n_buffer_codebox_proof();
     CtClipboard::force_plain_text();
     g_signal_emit_by_name(G_OBJECT(proof.text_view->gobj()), "paste-clipboard");
@@ -574,7 +598,7 @@ void CtActions::text_row_cut()
     if (not proof.text_view->get_buffer()) return;
     if (not _is_curr_node_not_read_only_or_error()) return;
 
-    CtTextRange range = CtList{_pCtMainWin, proof.text_view->get_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, proof.text_view->get_buffer()}.get_paragraph_iters();
     if (not range.iter_end.forward_char() and !range.iter_start.backward_char()) return;
     proof.text_view->get_buffer()->select_range(range.iter_start, range.iter_end);
     g_signal_emit_by_name(G_OBJECT(proof.text_view->gobj()), "cut-clipboard");
@@ -586,7 +610,7 @@ void CtActions::text_row_copy()
     auto proof = _get_text_view_n_buffer_codebox_proof();
     if (not proof.text_view->get_buffer()) return;
 
-    CtTextRange range = CtList{_pCtMainWin, proof.text_view->get_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, proof.text_view->get_buffer()}.get_paragraph_iters();
     if (not range.iter_end.forward_char() and !range.iter_start.backward_char()) return;
     proof.text_view->get_buffer()->select_range(range.iter_start, range.iter_end);
     g_signal_emit_by_name(G_OBJECT(proof.text_view->gobj()), "copy-clipboard");
@@ -599,7 +623,7 @@ void CtActions::text_row_delete()
     if (not proof.text_view->get_buffer()) return;
     if (not _is_curr_node_not_read_only_or_error()) return;
 
-    CtTextRange range = CtList{_pCtMainWin, proof.text_view->get_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, proof.text_view->get_buffer()}.get_paragraph_iters();
     if (not range.iter_end.forward_char() and !range.iter_start.backward_char()) return;
     proof.text_view->get_buffer()->erase(range.iter_start, range.iter_end);
     _pCtMainWin->get_state_machine().update_state();
@@ -641,7 +665,7 @@ void CtActions::text_row_selection_duplicate()
     }
     else {
         int cursor_offset = text_buffer->get_iter_at_mark(text_buffer->get_insert()).get_offset();
-        CtTextRange range = CtList{_pCtMainWin, proof.text_view->get_buffer()}.get_paragraph_iters();
+        CtTextRange range = CtList{_pCtConfig, proof.text_view->get_buffer()}.get_paragraph_iters();
         if (range.iter_start.get_offset() == range.iter_end.get_offset()) {
             Gtk::TextIter iter_start = text_buffer->get_iter_at_mark(text_buffer->get_insert());
             text_buffer->insert(iter_start, CtConst::CHAR_NEWLINE);
@@ -677,7 +701,7 @@ void CtActions::text_row_up()
     if (not _is_curr_node_not_read_only_or_error()) return;
 
     auto text_buffer = proof.text_view->get_buffer();
-    CtTextRange range = CtList{_pCtMainWin, text_buffer}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, text_buffer}.get_paragraph_iters();
     range.iter_end.forward_char();
     bool missing_leading_newline = false;
     Gtk::TextIter destination_iter = range.iter_start;
@@ -760,7 +784,7 @@ void CtActions::text_row_down()
     if (not _is_curr_node_not_read_only_or_error()) return;
 
     auto text_buffer = proof.text_view->get_buffer();
-    CtTextRange range = CtList{_pCtMainWin, text_buffer}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, text_buffer}.get_paragraph_iters();
     if (not range.iter_end.forward_char()) return;
     int missing_leading_newline = false;
     Gtk::TextIter destination_iter = range.iter_end;
@@ -1007,7 +1031,7 @@ void CtActions::_text_selection_change_case(gchar change_type)
     if (not text_buffer->get_has_selection() and
         not _pCtMainWin->apply_tag_try_automatic_bounds(text_buffer, text_buffer->get_insert()->get_iter()))
     {
-        CtDialogs::warning_dialog(_("No Text is Selected"), *_pCtMainWin);
+        CtDialogs::warning_dialog(_("No Text is Selected."), *_pCtMainWin);
         return;
     }
 
