@@ -1,7 +1,7 @@
 /*
  * ct_actions_file.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -55,34 +55,30 @@ void CtActions::file_vacuum()
 // Save the file providing a new name
 void CtActions::file_save_as()
 {
-    if (not _is_tree_not_empty_or_error())
-    {
+    if (not _is_tree_not_empty_or_error()) {
         return;
     }
-    CtDialogs::storage_select_args storageSelArgs(_pCtMainWin);
+    CtDialogs::storage_select_args storageSelArgs{_pCtMainWin};
+    storageSelArgs.showAutosaveOptions = true;
     fs::path currDocFilepath = _pCtMainWin->get_ct_storage()->get_file_path();
-    if (not currDocFilepath.empty())
-    {
+    if (not currDocFilepath.empty()) {
         storageSelArgs.ctDocType = fs::get_doc_type(currDocFilepath);
         storageSelArgs.ctDocEncrypt = fs::get_doc_encrypt(currDocFilepath);
     }
-    if (not CtDialogs::choose_data_storage_dialog(storageSelArgs))
-    {
+    if (not CtDialogs::choose_data_storage_dialog(storageSelArgs)) {
         return;
     }
     CtDialogs::FileSelectArgs fileSelArgs{_pCtMainWin};
-    if (not currDocFilepath.empty())
-    {
+    if (not currDocFilepath.empty()) {
         fileSelArgs.curr_folder = currDocFilepath.parent_path();
         fs::path suggested_basename = currDocFilepath.filename();
-        fileSelArgs.curr_file_name = suggested_basename.stem().string() + CtMiscUtil::get_doc_extension(storageSelArgs.ctDocType, storageSelArgs.ctDocEncrypt);
+        fileSelArgs.curr_file_name = suggested_basename.stem() + CtMiscUtil::get_doc_extension(storageSelArgs.ctDocType, storageSelArgs.ctDocEncrypt);
     }
     fileSelArgs.filter_name = _("CherryTree Document");
     std::string fileExtension = CtMiscUtil::get_doc_extension(storageSelArgs.ctDocType, storageSelArgs.ctDocEncrypt);
     fileSelArgs.filter_pattern.push_back(std::string{CtConst::CHAR_STAR}+fileExtension);
     std::string filepath = CtDialogs::file_save_as_dialog(fileSelArgs);
-    if (filepath.empty())
-    {
+    if (filepath.empty()) {
         return;
     }
 
@@ -139,7 +135,9 @@ void CtActions::preferences_import()
     _pCtConfig->statusbarVisible = ctConfigImported.statusbarVisible;
     _pCtConfig->treeLinesVisible = ctConfigImported.treeLinesVisible;
     _pCtConfig->hpanedPos = ctConfigImported.hpanedPos;
+    _pCtConfig->vpanedPos = ctConfigImported.vpanedPos;
     _pCtConfig->treeVisible = ctConfigImported.treeVisible;
+    _pCtConfig->vteVisible = ctConfigImported.vteVisible;
     _pCtConfig->menubarVisible = ctConfigImported.menubarVisible;
     _pCtConfig->linkType = ctConfigImported.linkType;
     _pCtConfig->menubarInTitlebar = ctConfigImported.menubarInTitlebar;
@@ -185,6 +183,8 @@ void CtActions::preferences_import()
     _pCtConfig->wrappingIndent = ctConfigImported.wrappingIndent;
     _pCtConfig->autoIndent = ctConfigImported.autoIndent;
     _pCtConfig->codeExecConfirm = ctConfigImported.codeExecConfirm;
+    _pCtConfig->codeExecVte = ctConfigImported.codeExecVte;
+    _pCtConfig->vteShell = ctConfigImported.vteShell;
     _pCtConfig->rtShowWhiteSpaces = ctConfigImported.rtShowWhiteSpaces;
     _pCtConfig->ptShowWhiteSpaces = ctConfigImported.ptShowWhiteSpaces;
     _pCtConfig->rtHighlCurrLine = ctConfigImported.rtHighlCurrLine;
@@ -227,6 +227,7 @@ void CtActions::preferences_import()
     _pCtConfig->ptFont = ctConfigImported.ptFont;
     _pCtConfig->treeFont = ctConfigImported.treeFont;
     _pCtConfig->codeFont = ctConfigImported.codeFont;
+    _pCtConfig->vteFont = ctConfigImported.vteFont;
     _pCtConfig->ttDefFg = ctConfigImported.ttDefFg;
     _pCtConfig->ttDefBg = ctConfigImported.ttDefBg;
     _pCtConfig->ttSelFg = ctConfigImported.ttSelFg;
@@ -288,7 +289,7 @@ void CtActions::preferences_import()
         _pCtConfig->customCodexecExt[currPair.first] = currPair.second;
     }
 
-    CtDialogs::info_dialog(_("This Change will have Effect Only After Restarting CherryTree"), *_pCtMainWin);
+    CtDialogs::info_dialog(_("This Change will have Effect Only After Restarting CherryTree."), *_pCtMainWin);
 }
 
 void CtActions::preferences_export()
