@@ -1,7 +1,7 @@
 /*
  * ct_actions_find.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -56,7 +56,7 @@ void CtActions::find_in_selected_node()
         if (entry_predefined_text.length()) {
             _s_options.str_find = entry_predefined_text;
         }
-        std::string title = _s_state.replace_active ? _("Replace in Current Node...") : _("Search in Current Node...");
+        std::string title = _s_state.replace_active ? _("Replace in Current Node") : _("Search in Current Node");
         pattern = CtDialogs::dialog_search(_pCtMainWin, title, _s_options, _s_state.replace_active, false/*multiple_nodes*/);
         if (entry_predefined_text.length()) {
             curr_buffer->move_mark(curr_buffer->get_insert(), iter_insert);
@@ -105,7 +105,7 @@ void CtActions::find_in_selected_node()
         CtDialogs::info_dialog(str::format(_("The pattern '%s' was not found"), str::xml_escape(pattern)), *_pCtMainWin);
     }
     else if (all_matches) {
-        _s_state.match_dialog_title = std::to_string(_s_state.matches_num) + CtConst::CHAR_SPACE + _("Matches");
+        _s_state.match_dialog_title = "'" + _s_options.str_find + "'  -  " + std::to_string(_s_state.matches_num) + CtConst::CHAR_SPACE + _("Matches");
         CtDialogs::match_dialog(_s_state.match_dialog_title, _pCtMainWin, _s_state.match_store);
     }
     else if (_s_options.iterative_dialog) {
@@ -245,7 +245,7 @@ void CtActions::find_in_multiple_nodes()
     }
     else {
         if (all_matches) {
-            _s_state.match_dialog_title = std::to_string(_s_state.matches_num) + CtConst::CHAR_SPACE + _("Matches");
+            _s_state.match_dialog_title = "'" + _s_options.str_find + "'  -  " + std::to_string(_s_state.matches_num) + CtConst::CHAR_SPACE + _("Matches");
             CtDialogs::match_dialog(_s_state.match_dialog_title, _pCtMainWin, _s_state.match_store);
         }
         else {
@@ -278,7 +278,7 @@ void CtActions::find_again_iter(const bool fromIterativeDialog)
             find_in_multiple_nodes();
         } break;
         default:
-            CtDialogs::warning_dialog(_("No Previous Search Was Performed During This Session"), *_pCtMainWin);
+            CtDialogs::warning_dialog(_("No Previous Search Was Performed During This Session."), *_pCtMainWin);
     }
     _s_state.from_find_iterated = false;
     _s_options.iterative_dialog = restore_iterative_dialog;
@@ -345,7 +345,7 @@ bool CtActions::_parse_given_node_content(CtTreeIter node_iter,
         // not first_fromsel or first_fromsel with first_node already parsed
         optFirstNode = false;
     }
-    if (optFirstNode.has_value() and not node_iter.get_node_is_excluded_from_search()) {
+    if (optFirstNode.has_value() and (not node_iter.get_node_is_excluded_from_search() or _s_options.override_exclusions)) {
         if (_s_options.node_content) {
             if (_parse_node_content_iter(node_iter,
                                          node_iter.get_node_text_buffer(),
@@ -366,7 +366,7 @@ bool CtActions::_parse_given_node_content(CtTreeIter node_iter,
     }
 
     CtTreeStore& ctTreeStore = _pCtMainWin->get_tree_store();
-    if (not node_iter.get_node_children_are_excluded_from_search()) {
+    if (not node_iter.get_node_children_are_excluded_from_search() or _s_options.override_exclusions) {
         // check for children
         if (not node_iter->children().empty()) {
             Gtk::TreeIter child_iter = forward ? node_iter->children().begin() : --node_iter->children().end();
@@ -417,7 +417,11 @@ bool CtActions::_parse_node_name_n_tags_iter(CtTreeIter& node_iter,
             gint64 node_id = node_iter.get_node_id();
             Glib::ustring node_hier_name = CtMiscUtil::get_node_hierarchical_name(node_iter, " << ", false, false);
             Glib::ustring line_content = _get_first_line_content(node_iter.get_node_text_buffer());
-            _s_state.match_store->add_row(node_id, node_name, str::xml_escape(node_hier_name), 0, 0, 1, line_content);
+            const Glib::ustring text_tags = node_iter.get_node_tags();
+            _s_state.match_store->add_row(node_id,
+                                          text_tags.empty() ? node_name : node_name + "\n [" +  _("Tags") + _(": ") + text_tags + "]",
+                                          str::xml_escape(node_hier_name),
+                                          0, 0, 1, line_content);
         }
         if (_s_state.replace_active and not node_iter.get_node_read_only()) {
             std::string replacer_text = _s_options.str_replace;
@@ -627,13 +631,14 @@ bool CtActions::_find_pattern(CtTreeIter tree_iter,
     Gtk::TreeIter iterAllMatchesRow;
     if (all_matches) {
         const gint64 node_id = tree_iter.get_node_id();
-        const std::string node_name = tree_iter.get_node_name();
+        const Glib::ustring node_name = tree_iter.get_node_name();
         const std::string node_hier_name = CtMiscUtil::get_node_hierarchical_name(tree_iter, " << ", false, false);
         const std::string line_content = obj_match_offsets.first != -1 ? obj_content : _get_line_content(text_buffer, iter_insert);
         int line_num = text_buffer->get_iter_at_offset(_s_state.latest_match_offsets.first).get_line();
         if (not _s_state.newline_trick) { line_num += 1; }
+        const Glib::ustring text_tags = tree_iter.get_node_tags();
         iterAllMatchesRow = _s_state.match_store->add_row(node_id,
-                                                          node_name,
+                                                          text_tags.empty() ? node_name : node_name + "\n [" +  _("Tags") + _(": ") + text_tags + "]",
                                                           str::xml_escape(node_hier_name),
                                                           _s_state.latest_match_offsets.first,
                                                           _s_state.latest_match_offsets.second,
@@ -688,14 +693,15 @@ Glib::ustring CtActions::_check_pattern_in_object(Glib::RefPtr<Glib::Regex> patt
         }
         if (pattern->match(text)) return text;
     }
-    else if (CtTable* table = dynamic_cast<CtTable*>(obj)) {
-        for (auto& row : table->get_table_matrix()) {
-            for (auto& col : row) {
-                Glib::ustring text = col->get_text_content();
+    else if (auto table = dynamic_cast<CtTableCommon*>(obj)) {
+        std::vector<std::vector<Glib::ustring>> rows;
+        table->write_strings_matrix(rows);
+        for (auto& row : rows) {
+            for (Glib::ustring& col : row) {
                 if (_s_options.accent_insensitive) {
-                    text = str::diacritical_to_ascii(text);
+                    col = str::diacritical_to_ascii(col);
                 }
-                if (pattern->match(text)) {
+                if (pattern->match(col)) {
                     return "<table>";
                 }
             }
