@@ -1,7 +1,7 @@
 /*
  * ct_actions_format.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -62,8 +62,8 @@ void CtActions::_save_tags_at_cursor_as_latest(Glib::RefPtr<Gtk::TextBuffer> rTe
         
     }
     if (not tagProperties.empty()) {
-        _pCtMainWin->get_ct_config()->latestTagProp = str::join(tagProperties, ",");
-        _pCtMainWin->get_ct_config()->latestTagVal = str::join(tagValues, ",");
+        _pCtConfig->latestTagProp = str::join(tagProperties, ",");
+        _pCtConfig->latestTagVal = str::join(tagValues, ",");
     }
 }
 
@@ -78,12 +78,12 @@ void CtActions::apply_tags_latest()
     if (not _is_there_selected_node_or_error()) return;
     if (not _is_curr_node_not_syntax_highlighting_or_error()) return;
     if (not _is_curr_node_not_read_only_or_error()) return;
-    if (_pCtMainWin->get_ct_config()->latestTagProp.empty())
-        CtDialogs::warning_dialog(_("No Previous Text Format Was Performed During This Session"), *_pCtMainWin);
+    if (_pCtConfig->latestTagProp.empty())
+        CtDialogs::warning_dialog(_("No Previous Text Format Was Performed During This Session."), *_pCtMainWin);
     else {
         remove_text_formatting();
-        std::vector<std::string> tagProperties = str::split(_pCtMainWin->get_ct_config()->latestTagProp, ",");
-        std::vector<std::string> tagValues = str::split(_pCtMainWin->get_ct_config()->latestTagVal, ",");
+        std::vector<std::string> tagProperties = str::split(_pCtConfig->latestTagProp, ",");
+        std::vector<std::string> tagValues = str::split(_pCtConfig->latestTagVal, ",");
         for (size_t i = 0; i < tagProperties.size(); ++i) {
             apply_tag(tagProperties.at(i), tagValues.at(i));
         }
@@ -98,7 +98,7 @@ void CtActions::remove_text_formatting()
     if (not _is_curr_node_not_read_only_or_error()) return;
     auto curr_buffer = _pCtMainWin->get_text_view().get_buffer();
     if (not curr_buffer->get_has_selection() and !_pCtMainWin->apply_tag_try_automatic_bounds(curr_buffer, curr_buffer->get_insert()->get_iter())) {
-        CtDialogs::warning_dialog(_("No Text is Selected"), *_pCtMainWin);
+        CtDialogs::warning_dialog(_("No Text is Selected."), *_pCtMainWin);
         return;
     }
     Gtk::TextIter iter_sel_start, iter_sel_end;
@@ -154,7 +154,7 @@ void CtActions::apply_tag_strikethrough()
 void CtActions::apply_tag_indent()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     if (not range.iter_start) return;
 
     //Each time we increase indent, we'll add this much margin to the text
@@ -166,7 +166,7 @@ void CtActions::apply_tag_indent()
 void CtActions::reduce_tag_indent()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     if (not range.iter_start) return;
 
     int newMargin = _find_previous_indent_margin() -1;
@@ -183,7 +183,7 @@ void CtActions::reduce_tag_indent()
 //If not, return the default "zero margin" (i.e. the margin shown in the UI when there's no indentation)
 int CtActions::_find_previous_indent_margin()
 {
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     std::vector<Glib::RefPtr<Gtk::TextTag>> curr_tags = range.iter_start.get_tags();
     for (auto& curr_tag : curr_tags) {
         Glib::ustring curr_tag_name = curr_tag->property_name();
@@ -197,7 +197,7 @@ int CtActions::_find_previous_indent_margin()
 void CtActions::_apply_tag_hN(const char* tagPropScaleVal)
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     if (not range.iter_start) return;
     apply_tag(CtConst::TAG_SCALE, tagPropScaleVal, range.iter_start, range.iter_end);
 }
@@ -236,7 +236,7 @@ void CtActions::list_bulleted_handler()
     if (not _is_curr_node_not_read_only_or_error()) return;
     text_view_n_buffer_codebox_proof proof = _get_text_view_n_buffer_codebox_proof();
     if (not proof.text_view->get_buffer()) return;
-    CtList{_pCtMainWin, proof.text_view->get_buffer()}.list_handler(CtListType::Bullet);
+    CtList{_pCtConfig, proof.text_view->get_buffer()}.list_handler(CtListType::Bullet);
 }
 
 // Handler of the Numbered List
@@ -245,7 +245,7 @@ void CtActions::list_numbered_handler()
     if (not _is_curr_node_not_read_only_or_error()) return;
     text_view_n_buffer_codebox_proof proof = _get_text_view_n_buffer_codebox_proof();
     if (not proof.text_view->get_buffer()) return;
-    CtList{_pCtMainWin, proof.text_view->get_buffer()}.list_handler(CtListType::Number);
+    CtList{_pCtConfig, proof.text_view->get_buffer()}.list_handler(CtListType::Number);
 }
 
 // Handler of the ToDo List
@@ -254,14 +254,14 @@ void CtActions::list_todo_handler()
     if (not _is_curr_node_not_read_only_or_error()) return;
     text_view_n_buffer_codebox_proof proof = _get_text_view_n_buffer_codebox_proof();
     if (not proof.text_view->get_buffer()) return;
-    CtList{_pCtMainWin, proof.text_view->get_buffer()}.list_handler(CtListType::Todo);
+    CtList{_pCtConfig, proof.text_view->get_buffer()}.list_handler(CtListType::Todo);
 }
 
 // The Justify Left Button was Pressed
 void CtActions::apply_tag_justify_left()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     if (not range.iter_start) return;
     apply_tag(CtConst::TAG_JUSTIFICATION, CtConst::TAG_PROP_VAL_LEFT, range.iter_start, range.iter_end);
 }
@@ -270,7 +270,7 @@ void CtActions::apply_tag_justify_left()
 void CtActions::apply_tag_justify_center()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     if (not range.iter_start) return;
     apply_tag(CtConst::TAG_JUSTIFICATION, CtConst::TAG_PROP_VAL_CENTER, range.iter_start, range.iter_end);
 }
@@ -279,7 +279,7 @@ void CtActions::apply_tag_justify_center()
 void CtActions::apply_tag_justify_right()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     if (not range.iter_start) return;
     apply_tag(CtConst::TAG_JUSTIFICATION, CtConst::TAG_PROP_VAL_RIGHT, range.iter_start, range.iter_end);
 }
@@ -288,7 +288,7 @@ void CtActions::apply_tag_justify_right()
 void CtActions::apply_tag_justify_fill()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
-    CtTextRange range = CtList{_pCtMainWin, _curr_buffer()}.get_paragraph_iters();
+    CtTextRange range = CtList{_pCtConfig, _curr_buffer()}.get_paragraph_iters();
     if (not range.iter_start) return;
     apply_tag(CtConst::TAG_JUSTIFICATION, CtConst::TAG_PROP_VAL_FILL, range.iter_start, range.iter_end);
 }
@@ -310,7 +310,7 @@ void CtActions::apply_tag(const Glib::ustring& tag_property,
                 if (tag_property != CtConst::TAG_LINK) {
                     restore_cursor_offset = text_buffer->get_insert()->get_iter().get_offset();
                     if (not _pCtMainWin->apply_tag_try_automatic_bounds(text_buffer, text_buffer->get_insert()->get_iter())) {
-                        CtDialogs::warning_dialog(_("No Text is Selected"), *_pCtMainWin);
+                        CtDialogs::warning_dialog(_("No Text is Selected."), *_pCtMainWin);
                         return;
                     }
                 }
@@ -339,7 +339,7 @@ void CtActions::apply_tag(const Glib::ustring& tag_property,
             iter_sel_end = it_sel_end;
         }
         else {
-            CtDialogs::warning_dialog(_("The Cursor is Not into a Paragraph"), *_pCtMainWin);
+            CtDialogs::warning_dialog(_("The Cursor is Not into a Paragraph."), *_pCtMainWin);
             return;
         }
     }
@@ -366,7 +366,7 @@ void CtActions::apply_tag(const Glib::ustring& tag_property,
         }
         else {
             gchar color_for = tag_property[0] == 'f' ? 'f' : 'b';
-            Gdk::RGBA ret_color = Gdk::RGBA(_pCtMainWin->get_ct_config()->currColors.at(color_for));
+            Gdk::RGBA ret_color = Gdk::RGBA(_pCtConfig->currColors.at(color_for));
             Glib::ustring title = tag_property[0] == 'f' ? _("Pick a Foreground Color") : _("Pick a Background Color");
             auto res = CtDialogs::color_pick_dialog(_pCtMainWin, title, ret_color, true);
             if (res == CtDialogs::CtPickDlgState::CANCEL) {
@@ -376,7 +376,7 @@ void CtActions::apply_tag(const Glib::ustring& tag_property,
                 property_value = "-"; // don't use empty because `apply prev tag` command brings a color dialog again
             }
             else {
-                _pCtMainWin->get_ct_config()->currColors[color_for] = CtRgbUtil::rgb_to_string(ret_color);
+                _pCtConfig->currColors[color_for] = CtRgbUtil::rgb_to_string(ret_color);
                 property_value = CtRgbUtil::rgb_to_string(ret_color);
             }
         }
@@ -444,16 +444,16 @@ void CtActions::apply_tag(const Glib::ustring& tag_property,
 
 CtActions::text_view_n_buffer_codebox_proof CtActions::_get_text_view_n_buffer_codebox_proof()
 {
-    if (CtCodebox* pCodebox = _codebox_in_use()) {
+    if (auto pCodebox = _codebox_in_use()) {
         return text_view_n_buffer_codebox_proof{
             &pCodebox->get_text_view(),
             pCodebox->get_syntax_highlighting(),
             pCodebox,
             nullptr};
     }
-    if (CtTable* pTable = _table_in_use()) {
+    if (auto pTable = dynamic_cast<CtTableHeavy*>(_table_in_use())) {
         return text_view_n_buffer_codebox_proof{
-            &pTable->get_table_matrix().at(pTable->current_row()).at(pTable->current_column())->get_text_view(),
+            &pTable->curr_cell_text_view(),
             CtConst::PLAIN_TEXT_ID,
             nullptr,
             pTable};
@@ -478,14 +478,14 @@ CtCodebox* CtActions::_codebox_in_use()
     return nullptr;
 }
 
-CtTable* CtActions::_table_in_use()
+CtTableCommon* CtActions::_table_in_use()
 {
     if (not curr_table_anchor) return nullptr;
     if (not _curr_buffer()) return nullptr;
     Gtk::TextIter iter_sel_start = _curr_buffer()->get_insert()->get_iter();
     auto widgets = _pCtMainWin->curr_tree_iter().get_anchored_widgets(iter_sel_start.get_offset(), iter_sel_start.get_offset());
     if (widgets.empty()) return nullptr;
-    if (auto pCtTable = dynamic_cast<CtTable*>(widgets.front())) {
+    if (auto pCtTable = dynamic_cast<CtTableCommon*>(widgets.front())) {
         return pCtTable;
     }
     return nullptr;
@@ -496,7 +496,7 @@ bool CtActions::_links_entries_pre_dialog(const Glib::ustring& curr_link, CtLink
 {
     const CtLinkEntry new_entry = CtMiscUtil::get_link_entry(curr_link);
     if (new_entry.type.empty()) {
-        CtDialogs::error_dialog(str::format("Tag Name Not Recognized! (%s)", std::string(curr_link)), *_pCtMainWin);
+        CtDialogs::error_dialog(str::format("Tag Name Not Recognized! (%s)", str::xml_escape(curr_link)), *_pCtMainWin);
         link_entry.type = CtConst::LINK_TYPE_WEBS;
         return false;
     }
