@@ -1,7 +1,7 @@
 /*
  * ct_actions_others.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -30,20 +30,22 @@
 #include <gtkmm/stock.h>
 #include <cstdlib>
 #include "ct_logging.h"
+#include "ct_list.h"
 #ifndef _WIN32
 #include <sys/wait.h> // WEXITSTATUS __FreeBSD__ (#1550)
 #endif // !_WIN32
+#if defined(HAVE_VTE)
+#include <vte/vte.h>
+#endif // HAVE_VTE
 
-// Cut Link
 void CtActions::link_cut()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     if (!_link_check_around_cursor().empty()) {
         g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "cut-clipboard");
     }
 }
 
-// Copy Link
 void CtActions::link_copy()
 {
     if (!_link_check_around_cursor().empty()) {
@@ -51,40 +53,35 @@ void CtActions::link_copy()
     }
 }
 
-//Dismiss Link
 void CtActions::link_dismiss()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     if (!_link_check_around_cursor().empty()) {
         remove_text_formatting();
     }
 }
 
-// Delete Link
 void CtActions::link_delete()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     if (!_link_check_around_cursor().empty()) {
         _curr_buffer()->erase_selection(true, _pCtMainWin->get_text_view().get_editable());
         _pCtMainWin->get_text_view().grab_focus();
     }
 }
 
-// Cut Anchor
 void CtActions::anchor_cut()
 {
     object_set_selection(curr_anchor_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "cut-clipboard");
 }
 
-// Copy Anchor
 void CtActions::anchor_copy()
 {
     object_set_selection(curr_anchor_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "copy-clipboard");
 }
 
-// Delete Anchor
 void CtActions::anchor_delete()
 {
     object_set_selection(curr_anchor_anchor);
@@ -93,7 +90,6 @@ void CtActions::anchor_delete()
     _pCtMainWin->get_text_view().grab_focus();
 }
 
-// Edit an Anchor
 void CtActions::anchor_edit()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
@@ -108,21 +104,18 @@ void CtActions::anchor_link_to_clipboard()
     CtClipboard{_pCtMainWin}.anchor_link_to_clipboard(_pCtMainWin->curr_tree_iter(), curr_anchor_anchor->get_anchor_name());
 }
 
-// Cut Embedded File
 void CtActions::embfile_cut()
 {
     object_set_selection(curr_file_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "cut-clipboard");
 }
 
-// Copy Embedded File
 void CtActions::embfile_copy()
 {
     object_set_selection(curr_file_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "copy-clipboard");
 }
 
-// Delete Embedded File
 void CtActions::embfile_delete()
 {
     object_set_selection(curr_file_anchor);
@@ -131,7 +124,6 @@ void CtActions::embfile_delete()
     _pCtMainWin->get_text_view().grab_focus();
 }
 
-// Embedded File Save Dialog
 void CtActions::embfile_save()
 {
     CtDialogs::FileSelectArgs args{_pCtMainWin};
@@ -145,7 +137,6 @@ void CtActions::embfile_save()
     g_file_set_contents(filepath.c_str(), curr_file_anchor->get_raw_blob().c_str(), (gssize)curr_file_anchor->get_raw_blob().size(), nullptr);
 }
 
-// Embedded File Open
 void CtActions::embfile_open()
 {
     const size_t open_id = curr_file_anchor->get_unique_id();
@@ -176,7 +167,6 @@ void CtActions::embfile_open()
     }
 }
 
-// Embedded File Rename
 void CtActions::embfile_rename()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
@@ -190,7 +180,6 @@ void CtActions::embfile_rename()
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
-// Save to Disk the selected LatexBox Image
 void CtActions::latex_save()
 {
     CtDialogs::FileSelectArgs args{_pCtMainWin};
@@ -212,7 +201,6 @@ void CtActions::latex_save()
     }
 }
 
-// Edit the selected LatexBox
 void CtActions::latex_edit()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
@@ -222,21 +210,18 @@ void CtActions::latex_edit()
     _latex_edit_dialog(curr_latex_anchor->get_latex_text(), iter_insert, &iter_bound);
 }
 
-// Cut LatexBox
 void CtActions::latex_cut()
 {
     object_set_selection(curr_latex_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "cut-clipboard");
 }
 
-// Copy LatexBox
 void CtActions::latex_copy()
 {
     object_set_selection(curr_latex_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "copy-clipboard");
 }
 
-// Delete LatexBox
 void CtActions::latex_delete()
 {
     object_set_selection(curr_latex_anchor);
@@ -245,7 +230,6 @@ void CtActions::latex_delete()
     _pCtMainWin->get_text_view().grab_focus();
 }
 
-// Save to Disk the selected Image
 void CtActions::image_save()
 {
     CtDialogs::FileSelectArgs args{_pCtMainWin};
@@ -267,7 +251,6 @@ void CtActions::image_save()
     }
 }
 
-// Edit the selected Image
 void CtActions::image_edit()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
@@ -277,21 +260,18 @@ void CtActions::image_edit()
     _image_edit_dialog(curr_image_anchor->get_pixbuf(), iter_insert, &iter_bound);
 }
 
-// Cut Image
 void CtActions::image_cut()
 {
     object_set_selection(curr_image_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "cut-clipboard");
 }
 
-// Copy Image
 void CtActions::image_copy()
 {
     object_set_selection(curr_image_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "copy-clipboard");
 }
 
-// Delete Image
 void CtActions::image_delete()
 {
     object_set_selection(curr_image_anchor);
@@ -300,7 +280,6 @@ void CtActions::image_delete()
     _pCtMainWin->get_text_view().grab_focus();
 }
 
-// Edit the Link Associated to the Image
 void CtActions::image_link_edit()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
@@ -323,7 +302,6 @@ void CtActions::image_link_edit()
     }
 }
 
-// Dismiss the Link Associated to the Image
 void CtActions::image_link_dismiss()
 {
     if (not _is_curr_node_not_read_only_or_error()) return;
@@ -337,7 +315,6 @@ void CtActions::toggle_show_hide_main_window()
     _pCtMainWin->signal_app_show_hide_main_win();
 }
 
-// Function Called at Every Link Click
 void CtActions::link_clicked(const Glib::ustring& tag_property_value, bool from_wheel)
 {
     CtLinkEntry link_entry = CtMiscUtil::get_link_entry(tag_property_value);
@@ -362,14 +339,14 @@ void CtActions::link_clicked(const Glib::ustring& tag_property_value, bool from_
         if (from_wheel) {
             filepath = filepath.parent_path();
             if (not fs::is_directory(filepath)) {
-                CtDialogs::error_dialog(str::format(_("The Folder Link '%s' is Not Valid"), str::xml_escape(filepath.string())), *_pCtMainWin);
+                CtDialogs::error_dialog(str::format(_("The Folder Link '%s' is Not Valid."), str::xml_escape(filepath.string())), *_pCtMainWin);
                 return;
             }
             fs::open_folderpath(filepath, _pCtConfig);
         }
         else {
             if (not Glib::file_test(filepath.string(), Glib::FILE_TEST_IS_REGULAR)) {
-                CtDialogs::error_dialog(str::format(_("The File Link '%s' is Not Valid"), str::xml_escape(filepath.string())), *_pCtMainWin);
+                CtDialogs::error_dialog(str::format(_("The File Link '%s' is Not Valid."), str::xml_escape(filepath.string())), *_pCtMainWin);
                 return;
             }
             fs::open_filepath(filepath, true, _pCtConfig);
@@ -431,32 +408,32 @@ void CtActions::current_node_scroll_to_anchor(Glib::ustring anchor_name)
     }
 }
 
-// Cut CodeBox
 void CtActions::codebox_cut()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     object_set_selection(curr_codebox_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "cut-clipboard");
 }
 
-// Copy CodeBox
 void CtActions::codebox_copy()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     object_set_selection(curr_codebox_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "copy-clipboard");
 }
 
-// Delete CodeBox
 void CtActions::codebox_delete()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     object_set_selection(curr_codebox_anchor);
     _curr_buffer()->erase_selection(true, _pCtMainWin->get_text_view().get_editable());
     curr_codebox_anchor = nullptr;
    _pCtMainWin->get_text_view().grab_focus();
 }
 
-// Delete CodeBox but keep the Text
 void CtActions::codebox_delete_keeping_text()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     if (not _is_curr_node_not_read_only_or_error()) return;
     Glib::ustring content = curr_codebox_anchor->get_text_content();
     object_set_selection(curr_codebox_anchor);
@@ -466,9 +443,9 @@ void CtActions::codebox_delete_keeping_text()
     _curr_buffer()->insert_at_cursor(content);
 }
 
-// Change CodeBox Properties
 void CtActions::codebox_change_properties()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     if (not _is_curr_node_not_read_only_or_error()) return;
     _pCtConfig->codeboxWidth = curr_codebox_anchor->get_frame_width();
     _pCtConfig->codeboxWidthPixels = curr_codebox_anchor->get_width_in_pixels();
@@ -488,28 +465,40 @@ void CtActions::codebox_change_properties()
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
-// Exec Code
-void CtActions::exec_code()
+void CtActions::_exec_code(const bool is_all)
 {
-    if (!_is_there_selected_node_or_error()) return;
+    if (not _is_there_selected_node_or_error()) return;
 
     std::string code_type;
     Glib::ustring code_val;
     auto proof = _get_text_view_n_buffer_codebox_proof();
-    if (proof.codebox) {
+    auto text_buffer = proof.text_view->get_buffer();
+    if (CtConst::PLAIN_TEXT_ID == proof.syntax_highl ||
+        CtConst::RICH_TEXT_ID == proof.syntax_highl ||
+        CtConst::TABLE_CELL_TEXT_ID == proof.syntax_highl)
+    {
+        code_type = "sh";
+    }
+    else {
         code_type = proof.syntax_highl;
-        code_val = proof.codebox->get_text_content();
-    } else {
-        if (_pCtMainWin->curr_tree_iter().get_node_is_rich_text()) {
-            CtDialogs::warning_dialog(_("No CodeBox is Selected"), *_pCtMainWin);
-            return;
+    }
+    if (is_all) {
+        code_val = text_buffer->begin().get_text(text_buffer->end());
+    }
+    else {
+        if (text_buffer->get_has_selection()) {
+            Gtk::TextIter iter_start, iter_end;
+            text_buffer->get_selection_bounds(iter_start, iter_end);
+            code_val = text_buffer->get_text(iter_start, iter_end);
+        }
+        else {
+            CtTextRange range = CtList{_pCtConfig, text_buffer}.get_paragraph_iters();
+            code_val = text_buffer->get_text(range.iter_start, range.iter_end);
         }
-        code_type = _pCtMainWin->curr_tree_iter().get_node_syntax_highlighting();
-        code_val = _curr_buffer()->begin().get_text(_curr_buffer()->end());
     }
     std::string binary_cmd = CtPrefDlg::get_code_exec_type_cmd(_pCtMainWin, code_type);
     if (binary_cmd.empty()) {
-        CtDialogs::warning_dialog(str::format(_("You must associate a command to '%s'.\nDo so in the Preferences Dialog"), str::xml_escape(code_type)), *_pCtMainWin);
+        CtDialogs::warning_dialog(str::format(_("You must associate a command to '%s'.\nDo so in the Preferences Dialog."), str::xml_escape(code_type)), *_pCtMainWin);
         return;
     }
     std::string code_type_ext = CtPrefDlg::get_code_exec_ext(_pCtMainWin, code_type);
@@ -517,33 +506,60 @@ void CtActions::exec_code()
 
     fs::path filepath_src_tmp = _pCtMainWin->get_ct_tmp()->getHiddenFilePath("exec_code." + code_type_ext);
     fs::path filepath_bin_tmp = _pCtMainWin->get_ct_tmp()->getHiddenFilePath("exec_code.exe");
-    binary_cmd = str::replace(binary_cmd, CtConst::CODE_EXEC_TMP_SRC, filepath_src_tmp.string());
+    const bool using_tmp_src{std::string::npos != binary_cmd.find(CtConst::CODE_EXEC_TMP_SRC)};
+    if (using_tmp_src) {
+        binary_cmd = str::replace(binary_cmd, CtConst::CODE_EXEC_TMP_SRC, filepath_src_tmp.string());
+    }
+    else {
+        binary_cmd = str::replace(binary_cmd, CtConst::CODE_EXEC_CODE_TXT, code_val.raw());
+    }
     binary_cmd = str::replace(binary_cmd, CtConst::CODE_EXEC_TMP_BIN, filepath_bin_tmp.string());
-    Glib::ustring terminal_cmd = str::replace(code_exec_term, CtConst::CODE_EXEC_COMMAND, binary_cmd);
 
-    if (_pCtConfig->codeExecConfirm and not CtDialogs::exec_code_confirm_dialog(*_pCtMainWin)) {
+    if (_pCtConfig->codeExecConfirm and not CtDialogs::exec_code_confirm_dialog(*_pCtMainWin, code_type, code_val)) {
         return;
     }
 
-#ifndef _WIN32
-    static bool xterm_verified{false};
-    if (not xterm_verified and str::startswith(terminal_cmd, "xterm ")) {
-        const int status = std::system("xterm -version");
-        if (WEXITSTATUS(status) != 0) {
-            CtDialogs::error_dialog(_("Install the package 'xterm' or configure a different terminal in the Preferences Dialog"), *_pCtMainWin);
-            return;
+#if !defined(HAVE_VTE)
+    if (_pCtConfig->codeExecVte) {
+        _pCtConfig->codeExecVte = false;
+    }
+#endif // !HAVE_VTE
+
+    Glib::ustring terminal_cmd;
+    if (not _pCtConfig->codeExecVte) {
+        terminal_cmd = str::replace(code_exec_term, CtConst::CODE_EXEC_COMMAND, binary_cmd);
+    }
+
+#if !defined(_WIN32)
+    if (not _pCtConfig->codeExecVte) {
+        static bool xterm_verified{false};
+        if (not xterm_verified and str::startswith(terminal_cmd, "xterm ")) {
+            const int status = std::system("xterm -version");
+            if (WEXITSTATUS(status) != 0) {
+                CtDialogs::error_dialog(_("Install the package 'xterm' or configure a different terminal in the Preferences Dialog."), *_pCtMainWin);
+                return;
+            }
+            xterm_verified = true;
         }
-        xterm_verified = true;
     }
 #endif // !_WIN32
 
-    g_file_set_contents(filepath_src_tmp.c_str(), code_val.c_str(), (gssize)code_val.bytes(), nullptr);
-    (void)std::system(terminal_cmd.c_str());
+    if (using_tmp_src) {
+        g_file_set_contents(filepath_src_tmp.c_str(), code_val.c_str(), (gssize)code_val.bytes(), nullptr);
+    }
+
+    if (_pCtConfig->codeExecVte) {
+        binary_cmd += "\n";
+        _pCtMainWin->exec_in_vte(binary_cmd);
+    }
+    else {
+        (void)std::system(terminal_cmd.c_str());
+    }
 }
 
-// Load the CodeBox Content From a Text Fil
 void CtActions::codebox_load_from_file()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     if (not _is_curr_node_not_read_only_or_error()) return;
     CtDialogs::FileSelectArgs args{_pCtMainWin};
     args.curr_folder = _pCtConfig->pickDirCbox;
@@ -556,9 +572,9 @@ void CtActions::codebox_load_from_file()
     curr_codebox_anchor->get_buffer()->set_text(buffer);
 }
 
-// Save the CodeBox Content To a Text File
 void CtActions::codebox_save_to_file()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     CtDialogs::FileSelectArgs args{_pCtMainWin};
     args.curr_folder=_pCtConfig->pickDirCbox;
 
@@ -572,6 +588,7 @@ void CtActions::codebox_save_to_file()
 
 void CtActions::codebox_increase_width()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
     int prevFrameWidth = curr_codebox_anchor->get_frame_width();
     if (curr_codebox_anchor->get_width_in_pixels()) {
@@ -591,6 +608,7 @@ void CtActions::codebox_increase_width()
 
 void CtActions::codebox_decrease_width()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
     if (curr_codebox_anchor->get_width_in_pixels()) {
         if (curr_codebox_anchor->get_frame_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX >= CtCodebox::CB_WIDTH_LIMIT_MIN) {
@@ -608,6 +626,7 @@ void CtActions::codebox_decrease_width()
 
 void CtActions::codebox_increase_height()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
     int prevFrameHeight = curr_codebox_anchor->get_frame_height();
     if (_pCtConfig->codeboxAutoResize and prevFrameHeight < curr_codebox_anchor->get_text_view().get_allocated_height() ) {
@@ -619,6 +638,7 @@ void CtActions::codebox_increase_height()
 
 void CtActions::codebox_decrease_height()
 {
+    if (not _is_there_anch_widg_selection_or_error('c')) return;
     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
     if (curr_codebox_anchor->get_frame_height() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX >= CtCodebox::CB_HEIGHT_LIMIT_MIN) {
         curr_codebox_anchor->set_width_height(0, curr_codebox_anchor->get_frame_height() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX);
@@ -628,54 +648,62 @@ void CtActions::codebox_decrease_height()
 
 void CtActions::table_cut()
 {
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
     object_set_selection(curr_table_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "cut-clipboard");
 }
 
 void CtActions::table_copy()
 {
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
     object_set_selection(curr_table_anchor);
     g_signal_emit_by_name(G_OBJECT(_pCtMainWin->get_text_view().gobj()), "copy-clipboard");
 }
 
 void CtActions::table_delete()
 {
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
     object_set_selection(curr_table_anchor);
-    _curr_buffer()->erase_selection(true, _pCtMainWin->get_text_view().get_editable());
+    _curr_buffer()->erase_selection(true/*interactive*/, _pCtMainWin->get_text_view().get_editable());
     curr_table_anchor = nullptr;
-   _pCtMainWin->get_text_view().grab_focus();
+    _pCtMainWin->get_text_view().grab_focus();
 }
 
 void CtActions::table_column_add()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     curr_table_anchor->column_add(curr_table_anchor->current_column());
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_column_delete()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     curr_table_anchor->column_delete(curr_table_anchor->current_column());
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_column_left()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
-    curr_table_anchor->column_move_left(curr_table_anchor->current_column());
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
+    curr_table_anchor->column_move_left(curr_table_anchor->current_column(), false/*from_move_right*/);
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_column_right()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     curr_table_anchor->column_move_right(curr_table_anchor->current_column());
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_column_increase_width()
 {
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
     curr_table_anchor->set_col_width(curr_table_anchor->get_col_width() + CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX);
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
@@ -683,6 +711,7 @@ void CtActions::table_column_increase_width()
 
 void CtActions::table_column_decrease_width()
 {
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
     if (_pCtMainWin->curr_tree_iter().get_node_read_only()) return;
     if (curr_table_anchor->get_col_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX >= CtCodebox::CB_WIDTH_LIMIT_MIN) {
         curr_table_anchor->set_col_width(curr_table_anchor->get_col_width() - CtCodebox::CB_WIDTH_HEIGHT_STEP_PIX);
@@ -692,62 +721,72 @@ void CtActions::table_column_decrease_width()
 
 void CtActions::table_row_add()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     curr_table_anchor->row_add(curr_table_anchor->current_row());
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_row_cut()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     table_row_copy();
     table_row_delete();
 }
 
 void CtActions::table_row_copy()
 {
-    auto table_state = std::dynamic_pointer_cast<CtAnchoredWidgetState_Table>(curr_table_anchor->get_state());
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    auto table_state = std::dynamic_pointer_cast<CtAnchoredWidgetState_TableCommon>(curr_table_anchor->get_state());
     // remove rows after current
     while (table_state->rows.size() > curr_table_anchor->current_row() + 1)
         table_state->rows.pop_back();
     // remove rows between current and header
     while (table_state->rows.size() > 2)
         table_state->rows.erase(table_state->rows.begin() + 1);
-    auto new_table = dynamic_cast<CtTable*>(table_state->to_widget(_pCtMainWin));
+    auto new_table = dynamic_cast<CtTableCommon*>(table_state->to_widget(_pCtMainWin));
     CtClipboard{_pCtMainWin}.table_row_to_clipboard(new_table);
     delete new_table;
 }
 
 void CtActions::table_row_paste()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
+    curr_table_anchor->exit_cell_edit();
     CtClipboard{_pCtMainWin}.table_row_paste(curr_table_anchor);
+    curr_table_anchor->grab_focus();
 }
 
 void CtActions::table_row_delete()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     curr_table_anchor->row_delete(curr_table_anchor->current_row());
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_row_up()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
-    curr_table_anchor->row_move_up(curr_table_anchor->current_row());
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
+    curr_table_anchor->row_move_up(curr_table_anchor->current_row(), false/*from_move_down*/);
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_row_down()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     curr_table_anchor->row_move_down(curr_table_anchor->current_row());
     _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
 }
 
 void CtActions::table_rows_sort_descending()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     if (curr_table_anchor->row_sort_desc()) {
         _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
     }
@@ -755,7 +794,8 @@ void CtActions::table_rows_sort_descending()
 
 void CtActions::table_rows_sort_ascending()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     if (curr_table_anchor->row_sort_asc()) {
         _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
     }
@@ -763,16 +803,36 @@ void CtActions::table_rows_sort_ascending()
 
 void CtActions::table_edit_properties()
 {
-    if (!_is_curr_node_not_read_only_or_error()) return;
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
+    if (not _is_curr_node_not_read_only_or_error()) return;
     _pCtConfig->tableColWidthDefault = curr_table_anchor->get_col_width_default();
-    if (CtDialogs::TableHandleResp::Cancel == CtDialogs::table_handle_dialog(_pCtMainWin, _("Edit Table Properties"), false/*is_insert*/))
+    const bool was_light = curr_table_anchor->get_is_light();
+    bool is_light{was_light};
+    if (CtDialogs::TableHandleResp::Cancel == CtDialogs::table_handle_dialog(
+        _pCtMainWin, _("Edit Table Properties"), false/*is_insert*/, is_light))
+    {
         return;
+    }
     curr_table_anchor->set_col_width_default(_pCtConfig->tableColWidthDefault);
-    _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
+    if (was_light != is_light) {
+        std::shared_ptr<CtAnchoredWidgetState_TableCommon> pStateCommon = curr_table_anchor->get_state_common();
+        table_delete();
+        auto pCtTable = is_light ? static_cast<CtTableCommon*>(pStateCommon->to_widget_light(_pCtMainWin)) :
+            static_cast<CtTableCommon*>(pStateCommon->to_widget_heavy(_pCtMainWin));
+        Glib::RefPtr<Gsv::Buffer> gsv_buffer = Glib::RefPtr<Gsv::Buffer>::cast_dynamic(_curr_buffer());
+        pCtTable->insertInTextBuffer(gsv_buffer);
+        _pCtMainWin->get_tree_store().addAnchoredWidgets(_pCtMainWin->curr_tree_iter(),
+            {pCtTable}, &_pCtMainWin->get_text_view());
+        curr_table_anchor = pCtTable;
+    }
+    else {
+        _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
+    }
 }
 
 void CtActions::table_export()
 {
+    if (not _is_there_anch_widg_selection_or_error('t')) return;
     CtDialogs::FileSelectArgs args{_pCtMainWin};
     args.curr_folder = _pCtConfig->pickDirCsv;
     args.curr_file_name = "";
@@ -785,20 +845,15 @@ void CtActions::table_export()
     _pCtConfig->pickDirCsv = Glib::path_get_dirname(filename);
 
     try {
-        std::stringstream buffer;
-        curr_table_anchor->to_csv(buffer);
-        std::string result = buffer.str();
-
-        g_file_set_contents(filename.c_str(), result.c_str(), (gssize)result.size(), nullptr);
-
+        std::string result = curr_table_anchor->to_csv();
+        Glib::file_set_contents(filename, result);
     }
-    catch(std::exception& e) {
+    catch (std::exception& e) {
         spdlog::error("Exception caught while exporting table: {}", e.what());
         CtDialogs::error_dialog("Exception occured while exporting table, see log for details", *_pCtMainWin);
     }
 }
 
-// Anchor Edit Dialog
 void CtActions::_anchor_edit_dialog(CtImageAnchor* anchor, Gtk::TextIter insert_iter, Gtk::TextIter* iter_bound)
 {
     Glib::ustring dialog_title = anchor == nullptr ? _("Insert Anchor") :  _("Edit Anchor");
@@ -816,7 +871,6 @@ void CtActions::_anchor_edit_dialog(CtImageAnchor* anchor, Gtk::TextIter insert_
     image_insert_anchor(insert_iter, ret_anchor_name, image_justification);
 }
 
-// Iteration of the Modification Time Sentinel
 bool CtActions::_on_embfiles_sentinel_timeout()
 {
     for (auto& item : _embfiles_opened) {
@@ -837,7 +891,7 @@ bool CtActions::_on_embfiles_sentinel_timeout()
                 continue;
             }
             if (tree_iter.get_node_read_only()) {
-                CtDialogs::warning_dialog(_("Cannot Edit Embedded File in Read Only Node"), *_pCtMainWin);
+                CtDialogs::warning_dialog(_("Cannot Edit Embedded File in Read Only Node."), *_pCtMainWin);
                 continue;
             }
             _pCtMainWin->get_tree_view().set_cursor_safe(tree_iter);
@@ -859,3 +913,35 @@ bool CtActions::_on_embfiles_sentinel_timeout()
     }
     return true; // this way we keep the timer alive
 }
+
+void CtActions::terminal_copy()
+{
+#if defined(HAVE_VTE)
+    Gtk::Widget* pVte = _pCtMainWin->get_vte();
+    if (not pVte) {
+        return;
+    }
+    GtkWidget* pTermWidget = pVte->gobj();
+    if (not vte_terminal_get_has_selection(VTE_TERMINAL(pTermWidget))) {
+        vte_terminal_select_all(VTE_TERMINAL(pTermWidget));
+    }
+    vte_terminal_copy_clipboard_format(VTE_TERMINAL(pTermWidget), VTE_FORMAT_TEXT);
+#endif // HAVE_VTE
+}
+
+void CtActions::terminal_paste()
+{
+#if defined(HAVE_VTE)
+    Gtk::Widget* pVte = _pCtMainWin->get_vte();
+    if (not pVte) {
+        return;
+    }
+    GtkWidget* pTermWidget = pVte->gobj();
+    vte_terminal_paste_clipboard(VTE_TERMINAL(pTermWidget));
+#endif // HAVE_VTE
+}
+
+void CtActions::terminal_reset()
+{
+    _pCtMainWin->restart_vte(nullptr/*first_cmd_passed*/);
+}
