@@ -1,7 +1,7 @@
 /*
  * ct_clipboard.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -193,30 +193,31 @@ void CtClipboard::_paste_clipboard(Gtk::TextView* pTextView, CtCodebox* /*pCodeb
     Gtk::Clipboard::get()->request_contents(target, receive_fun);
 }
 
-void CtClipboard::table_row_to_clipboard(CtTable* pTable)
+void CtClipboard::table_row_to_clipboard(CtTableCommon* pTable)
 {
-    CtClipboardData* clip_data = new CtClipboardData();
+    CtClipboardData* clip_data = new CtClipboardData;
     pTable->to_xml(clip_data->xml_doc.create_root_node("root"), 0, nullptr);
-    clip_data->html_text = CtExport2Html(_pCtMainWin).table_export_to_html(pTable);
+    clip_data->html_text = CtExport2Html{_pCtMainWin}.table_export_to_html(pTable);
 
     _set_clipboard_data({CtConst::TARGET_CTD_TABLE, CtConst::TARGETS_HTML[0]}, clip_data);
 }
 
-void CtClipboard::table_row_paste(CtTable* pTable)
+void CtClipboard::table_row_paste(CtTableCommon* pTable)
 {
     std::vector<Glib::ustring> targets = Gtk::Clipboard::get()->wait_for_targets();
-    if (vec::exists(targets, CtConst::TARGET_CTD_TABLE))
-    {
+    if (vec::exists(targets, CtConst::TARGET_CTD_TABLE)) {
         auto win = _pCtMainWin;
         Gtk::TextView* view = &_pCtMainWin->get_text_view();
-        auto received_table = [win, view, pTable](const Gtk::SelectionData& s) { CtClipboard{win}.on_received_to_table(s, view, false, pTable);};
+        auto received_table = [win, view, pTable](const Gtk::SelectionData& s) {
+            CtClipboard{win}.on_received_to_table(s, view, false, pTable);
+        };
         Gtk::Clipboard::get()->request_contents(CtConst::TARGET_CTD_TABLE, received_table);
     }
 }
 
 void CtClipboard::node_link_to_clipboard(CtTreeIter node)
 {
-    CtClipboardData* clip_data = new CtClipboardData();
+    CtClipboardData* clip_data = new CtClipboardData;
     std::string tml = R"XML(<?xml version="1.0" encoding="UTF-8"?><root><slot><rich_text link="node {}">{}</rich_text></slot></root>)XML";
     clip_data->rich_text = fmt::format(tml, node.get_node_id(), str::xml_escape(node.get_node_name()));
     clip_data->plain_text = "node: " + node.get_node_name();
@@ -226,7 +227,7 @@ void CtClipboard::node_link_to_clipboard(CtTreeIter node)
 
 void CtClipboard::anchor_link_to_clipboard(CtTreeIter node, const Glib::ustring& anchor_name)
 {
-    CtClipboardData* clip_data = new CtClipboardData();
+    CtClipboardData* clip_data = new CtClipboardData;
     std::string tml = R"XML(<?xml version="1.0" encoding="UTF-8"?><root><slot><rich_text link="node {} {}">{}</rich_text></slot></root>)XML";
     clip_data->rich_text = fmt::format(tml, node.get_node_id(), str::xml_escape(anchor_name), str::xml_escape(anchor_name));
     clip_data->plain_text = "anchor: " + anchor_name;
@@ -262,7 +263,7 @@ void CtClipboard::_rich_text_process_slot(xmlpp::Element* root, int start_offset
                                           CtAnchoredWidget* obj_element, gchar change_case /*="n"*/)
 {
     xmlpp::Element* dom_iter = root->add_child("slot");
-    CtStorageXmlHelper::save_buffer_no_widgets_to_xml(dom_iter, text_buffer, start_offset, end_offset, change_case);
+    CtStorageXmlHelper{_pCtMainWin}.save_buffer_no_widgets_to_xml(dom_iter, text_buffer, start_offset, end_offset, change_case);
 
     if (obj_element != nullptr)
     {
@@ -290,15 +291,18 @@ void CtClipboard::from_xml_string_to_buffer(Glib::RefPtr<Gtk::TextBuffer> text_b
     _pCtMainWin->get_state_machine().not_undoable_timeslot_set(true);
 
     std::list<CtAnchoredWidget*> widgets;
-    for (xmlpp::Node* slot_node: parser.get_document()->get_root_node()->get_children())
-    {
+    for (xmlpp::Node* slot_node: parser.get_document()->get_root_node()->get_children()) {
         if (slot_node->get_name() != "slot")
             continue;
-        for (xmlpp::Node* child_node: slot_node->get_children())
-        {
+        for (xmlpp::Node* child_node : slot_node->get_children()) {
             Glib::RefPtr<Gsv::Buffer> gsv_buffer = Glib::RefPtr<Gsv::Buffer>::cast_dynamic(text_buffer);
             Gtk::TextIter insert_iter = text_buffer->get_insert()->get_iter();
-            CtStorageXmlHelper(_pCtMainWin).get_text_buffer_one_slot_from_xml(gsv_buffer, child_node, widgets, &insert_iter, insert_iter.get_offset());
+            CtStorageXmlHelper{_pCtMainWin}.get_text_buffer_one_slot_from_xml(
+                gsv_buffer,
+                child_node,
+                widgets,
+                &insert_iter,
+                insert_iter.get_offset());
         }
     }
     if (not widgets.empty()) {
@@ -316,17 +320,19 @@ void CtClipboard::from_xml_string_to_buffer(Glib::RefPtr<Gtk::TextBuffer> text_b
 }
 
 // Write the Selected Content to the Clipboard
-void CtClipboard::_selection_to_clipboard(Glib::RefPtr<Gtk::TextBuffer> text_buffer, Gtk::TextView* /*sourceview*/, Gtk::TextIter iter_sel_start, Gtk::TextIter iter_sel_end, int num_chars, CtCodebox* pCodebox)
+void CtClipboard::_selection_to_clipboard(Glib::RefPtr<Gtk::TextBuffer> text_buffer,
+                                          Gtk::TextView* /*sourceview*/,
+                                          Gtk::TextIter iter_sel_start,
+                                          Gtk::TextIter iter_sel_end,
+                                          int num_chars,
+                                          CtCodebox* pCodebox)
 {
     Glib::ustring node_syntax_high = _pCtMainWin->curr_tree_iter().get_node_syntax_highlighting();
     CtImage* pixbuf_target = nullptr;
-    if (not pCodebox and node_syntax_high == CtConst::RICH_TEXT_ID and num_chars == 1)
-    {
+    if (not pCodebox and node_syntax_high == CtConst::RICH_TEXT_ID and num_chars == 1) {
         std::list<CtAnchoredWidget*> widget_vector = _pCtMainWin->curr_tree_iter().get_anchored_widgets(iter_sel_start.get_offset(), iter_sel_end.get_offset());
-        if (widget_vector.size() > 0)
-        {
-            if (CtImage* image = dynamic_cast<CtImage*>(widget_vector.front()))
-            {
+        if (widget_vector.size() > 0) {
+            if (auto image = dynamic_cast<CtImage*>(widget_vector.front())) {
                 pixbuf_target = image;
 #ifdef _WIN32
                 // image target doesn't work on Win32 with other targets, so have to set it directly
@@ -337,19 +343,17 @@ void CtClipboard::_selection_to_clipboard(Glib::RefPtr<Gtk::TextBuffer> text_buf
                 }
 #endif
             }
-            else if (CtTable* table = dynamic_cast<CtTable*>(widget_vector.front()))
-            {
-                CtClipboardData* clip_data = new CtClipboardData();
+            else if (auto table = dynamic_cast<CtTableCommon*>(widget_vector.front())) {
+                CtClipboardData* clip_data = new CtClipboardData;
                 table->to_xml(clip_data->xml_doc.create_root_node("root"), 0, nullptr);
-                clip_data->html_text = CtExport2Html(_pCtMainWin).table_export_to_html(table);
-                clip_data->plain_text = CtExport2Txt(_pCtMainWin).get_table_plain(table);
+                clip_data->html_text = CtExport2Html{_pCtMainWin}.table_export_to_html(table);
+                clip_data->plain_text = CtExport2Txt{_pCtMainWin}.get_table_plain(table);
 
                 _set_clipboard_data({CtConst::TARGET_CTD_TABLE, CtConst::TARGETS_HTML[0], CtConst::TARGET_CTD_PLAIN_TEXT}, clip_data);
                 return;
             }
-            else if (CtCodebox* codebox = dynamic_cast<CtCodebox*>(widget_vector.front()))
-            {
-                CtClipboardData* clip_data = new CtClipboardData();
+            else if (auto codebox = dynamic_cast<CtCodebox*>(widget_vector.front())) {
+                CtClipboardData* clip_data = new CtClipboardData;
                 codebox->to_xml(clip_data->xml_doc.create_root_node("root"), 0, nullptr);
                 clip_data->html_text = CtExport2Html(_pCtMainWin).codebox_export_to_html(codebox);
                 if (num_chars == 1) // just copy one codebox
@@ -363,7 +367,7 @@ void CtClipboard::_selection_to_clipboard(Glib::RefPtr<Gtk::TextBuffer> text_buf
         }
     }
 
-    CtClipboardData* clip_data = new CtClipboardData();
+    CtClipboardData* clip_data = new CtClipboardData;
     clip_data->html_text = CtExport2Html(_pCtMainWin).selection_export_to_html(text_buffer, iter_sel_start, iter_sel_end, !pCodebox ? node_syntax_high : CtConst::PLAIN_TEXT_ID);
     if (not pCodebox and node_syntax_high == CtConst::RICH_TEXT_ID)
     {
@@ -557,15 +561,17 @@ void CtClipboard::on_received_to_codebox(const Gtk::SelectionData& selection_dat
 }
 
 // From Clipboard to Table
-void CtClipboard::on_received_to_table(const Gtk::SelectionData& selection_data, Gtk::TextView* pTextView, bool, CtTable* parentTable)
+void CtClipboard::on_received_to_table(const Gtk::SelectionData& selection_data,
+                                       Gtk::TextView* pTextView,
+                                       bool,
+                                       CtTableCommon* parentTable)
 {
 #ifdef __APPLE__
     Glib::ustring xml_text = selection_data.get_data_as_string();
 #else
     Glib::ustring xml_text = selection_data.get_text();
 #endif
-    if (xml_text.empty())
-    {
+    if (xml_text.empty()) {
         spdlog::error("? no clipboard xml text");
         return;
     }
@@ -579,42 +585,57 @@ void CtClipboard::on_received_to_table(const Gtk::SelectionData& selection_data,
         return;
     }
 
-    if (parentTable)
-    {
-        CtTableMatrix tableMatrix;
+    if (parentTable) {
+        CtTableMatrix tableFromClipboardMatrix;
         CtTableColWidths tableColWidths;
-        CtStorageXmlHelper(_pCtMainWin).populate_table_matrix(
-            tableMatrix,
+        bool is_light{parentTable->get_is_light()};
+        CtStorageXmlHelper{_pCtMainWin}.populate_table_matrix(
+            tableFromClipboardMatrix,
             static_cast<xmlpp::Element*>(parser.get_document()->get_root_node()->get_first_child("table")),
-            tableColWidths);
+            tableColWidths,
+            is_light);
 
-        int col_num = (int)parentTable->get_table_matrix()[0].size();
+        int col_num = (int)parentTable->get_num_columns();
         int insert_after = parentTable->current_row() - 1;
         if (insert_after < 0) insert_after = 0;
-        for (int row = 1 /*skip header*/; row < (int)tableMatrix.size(); ++row)
-        {
+        for (int row = 1/*skip header*/; row < (int)tableFromClipboardMatrix.size(); ++row) {
             std::vector<Glib::ustring> new_row;
-            std::transform(tableMatrix[row].begin(), tableMatrix[row].end(), std::back_inserter(new_row), [](CtTextCell* cell) { return cell->get_text_content(); });
+            std::transform(tableFromClipboardMatrix[row].begin(),
+                           tableFromClipboardMatrix[row].end(),
+                           std::back_inserter(new_row),
+                           [is_light](void* cell) {
+                                if (is_light) {
+                                    return *static_cast<Glib::ustring*>(cell);
+                                }
+                                return static_cast<CtTextCell*>(cell)->get_text_content();
+                           });
             while ((int)new_row.size() > col_num) new_row.pop_back();
             while ((int)new_row.size() < col_num) new_row.push_back("");
             parentTable->row_add(insert_after + (row-1), &new_row);
         }
-        for (auto row: tableMatrix)
-            for (auto cell: row)
-                delete cell;
-        _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true /*new_machine_state*/);
+        for (auto& row : tableFromClipboardMatrix) {
+            for (void* cell : row) {
+                if (is_light) delete static_cast<Glib::ustring*>(cell);
+                else delete static_cast<CtTextCell*>(cell);
+            }
+        }
+        _pCtMainWin->update_window_save_needed(CtSaveNeededUpdType::nbuf, true/*new_machine_state*/);
     }
-    else
-    {
+    else {
         std::list<CtAnchoredWidget*> widgets;
         Glib::RefPtr<Gsv::Buffer> gsv_buffer = Glib::RefPtr<Gsv::Buffer>::cast_dynamic(pTextView->get_buffer());
         Gtk::TextIter insert_iter = pTextView->get_buffer()->get_insert()->get_iter();
-        CtStorageXmlHelper(_pCtMainWin).get_text_buffer_one_slot_from_xml(gsv_buffer, parser.get_document()->get_root_node()->get_first_child("table"), widgets, &insert_iter, insert_iter.get_offset());
-        if (not widgets.empty())
-        {
+        CtStorageXmlHelper{_pCtMainWin}.get_text_buffer_one_slot_from_xml(
+            gsv_buffer,
+            parser.get_document()->get_root_node()->get_first_child("table"),
+            widgets,
+            &insert_iter,
+            insert_iter.get_offset());
+        if (not widgets.empty()) {
             _pCtMainWin->get_tree_store().addAnchoredWidgets(
-                        _pCtMainWin->curr_tree_iter(),
-                        widgets, &_pCtMainWin->get_text_view());
+                _pCtMainWin->curr_tree_iter(),
+                widgets,
+                &_pCtMainWin->get_text_view());
             _pCtMainWin->get_state_machine().update_state();
         }
 
@@ -812,37 +833,37 @@ void CtClipboard::_yaml_to_codebox(const Glib::ustring& yaml_text, Gtk::TextView
         enum class ParseState {OUT_BLOCK, IN_BLOCK};
         ParseState state = ParseState::OUT_BLOCK;
         std::string block_indent = "      ";
-        std::vector<std::string> block_lines;
-        std::istringstream iss(yaml_text);
-        for (std::string line; std::getline(iss, line);)
-        {
+        std::vector<Glib::ustring> block_lines;
+        std::vector<Glib::ustring> lines = str::split(yaml_text, CtConst::CHAR_NEWLINE);
+        for (const auto& curr_line : lines) {
             if (state == ParseState::OUT_BLOCK) {
-                std::string trim_line = str::trim(line);
-                if (trim_line == "source: |-")
+                Glib::ustring trim_line = str::trim(curr_line);
+                if (trim_line == "source: |-") {
                     state = ParseState::IN_BLOCK;
+                }
                 else if (str::startswith(trim_line, "syntax:"))             p_codebox_node->set_attribute("syntax_highlighting", get_value(trim_line));
                 else if (str::startswith(trim_line, "width:"))              p_codebox_node->set_attribute("frame_width", get_value(trim_line));
                 else if (str::startswith(trim_line, "height:"))             p_codebox_node->set_attribute("frame_height", get_value(trim_line));
                 else if (str::startswith(trim_line, "width_in_pixels:"))    p_codebox_node->set_attribute("width_in_pixels", get_value(trim_line));
                 else if (str::startswith(trim_line, "highlight_brackets:")) p_codebox_node->set_attribute("highlight_brackets", get_value(trim_line));
-            } else {
-                if (!str::startswith(line, block_indent))
+            }
+            else {
+                if (!str::startswith(curr_line, block_indent)) {
                     state = ParseState::OUT_BLOCK;
+                }
                 else {
-                    block_lines.push_back(line.substr(6)); // don't add \n at the end of the block
+                    block_lines.push_back(curr_line.substr(6)); // don't add \n at the end of the block
                 }
             }
         }
         p_codebox_node->add_child_text(str::join(block_lines, "\n"));
         _xml_to_codebox(xml_doc.write_to_string(), pTextView);
     }
-    catch (std::exception& e)
-    {
-        spdlog::error("_yaml_to_codebox is failed, exception: {}\n{}", e.what(), yaml_text);
+    catch (std::exception& e) {
+        spdlog::error("_yaml_to_codebox exception: {}\n{}", e.what(), yaml_text);
     }
-    catch (...)
-    {
-        spdlog::error("_yaml_to_codebox is failed, unknown exception\n{}", yaml_text);
+    catch (...) {
+        spdlog::error("_yaml_to_codebox unknown exception\n{}", yaml_text);
     }
 }
 
