@@ -1,7 +1,7 @@
 /*
  * ct_config.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -205,7 +205,9 @@ void CtConfig::_populate_keyfile_from_data()
     _uKeyFile->set_integer(_currentGroup, "win_size_w", winRect[2]);
     _uKeyFile->set_integer(_currentGroup, "win_size_h", winRect[3]);
     _uKeyFile->set_integer(_currentGroup, "hpaned_pos", hpanedPos);
+    _uKeyFile->set_integer(_currentGroup, "vpaned_pos", vpanedPos);
     _uKeyFile->set_boolean(_currentGroup, "tree_visible", treeVisible);
+    _uKeyFile->set_boolean(_currentGroup, "vte_visible", vteVisible);
     _uKeyFile->set_boolean(_currentGroup, "menubar_visible", menubarVisible);
     _uKeyFile->set_string(_currentGroup, "pick_dir_import", pickDirImport);
     _uKeyFile->set_string(_currentGroup, "pick_dir_export", pickDirExport);
@@ -266,6 +268,7 @@ void CtConfig::_populate_keyfile_from_data()
     _uKeyFile->set_integer(_currentGroup, "wrapping_indent", wrappingIndent);
     _uKeyFile->set_boolean(_currentGroup, "auto_indent", autoIndent);
     _uKeyFile->set_boolean(_currentGroup, "codexec_confirm", codeExecConfirm);
+    _uKeyFile->set_boolean(_currentGroup, "codexec_vte", codeExecVte);
     _uKeyFile->set_boolean(_currentGroup, "rt_show_white_spaces", rtShowWhiteSpaces);
     _uKeyFile->set_boolean(_currentGroup, "pt_show_white_spaces", ptShowWhiteSpaces);
     _uKeyFile->set_boolean(_currentGroup, "rt_highl_curr_line", rtHighlCurrLine);
@@ -275,6 +278,7 @@ void CtConfig::_populate_keyfile_from_data()
     _uKeyFile->set_integer(_currentGroup, "space_around_lines", spaceAroundLines);
     _uKeyFile->set_integer(_currentGroup, "relative_wrapped_space", relativeWrappedSpace);
     _uKeyFile->set_string(_currentGroup, "h_rule", hRule);
+    _uKeyFile->set_string(_currentGroup, "vte_shell", vteShell);
     _uKeyFile->set_string(_currentGroup, "special_chars", specialChars.item());
     _uKeyFile->set_string(_currentGroup, "last_special_char", lastSpecialChar);
     _uKeyFile->set_string(_currentGroup, "selword_chars", selwordChars.item());
@@ -310,6 +314,7 @@ void CtConfig::_populate_keyfile_from_data()
     _uKeyFile->set_integer(_currentGroup, "table_rows", tableRows);
     _uKeyFile->set_integer(_currentGroup, "table_columns", tableColumns);
     _uKeyFile->set_integer(_currentGroup, "table_col_width", tableColWidthDefault);
+    _uKeyFile->set_integer(_currentGroup, "table_cells_go_light", tableCellsGoLight);
 
     // [fonts]
     _currentGroup = "fonts";
@@ -317,6 +322,7 @@ void CtConfig::_populate_keyfile_from_data()
     _uKeyFile->set_string(_currentGroup, "pt_font", ptFont);
     _uKeyFile->set_string(_currentGroup, "tree_font", treeFont);
     _uKeyFile->set_string(_currentGroup, "code_font", codeFont);
+    _uKeyFile->set_string(_currentGroup, "vte_font", vteFont);
 
     // [colors]
     _currentGroup = "colors";
@@ -440,7 +446,9 @@ void CtConfig::_populate_data_from_keyfile()
     _populate_int_from_keyfile("win_size_w", &winRect[2]);
     _populate_int_from_keyfile("win_size_h", &winRect[3]);
     _populate_int_from_keyfile("hpaned_pos", &hpanedPos);
+    _populate_int_from_keyfile("vpaned_pos", &vpanedPos);
     _populate_bool_from_keyfile("tree_visible", &treeVisible);
+    _populate_bool_from_keyfile("vte_visible", &vteVisible);
     _populate_bool_from_keyfile("menubar_visible", &menubarVisible);
     if (savedFromPyGtk) {
         CtRecentDocRestore recentDocRestore;
@@ -549,6 +557,7 @@ void CtConfig::_populate_data_from_keyfile()
     _populate_int_from_keyfile("wrapping_indent", &wrappingIndent);
     _populate_bool_from_keyfile("auto_indent", &autoIndent);
     _populate_bool_from_keyfile("codexec_confirm", &codeExecConfirm);
+    _populate_bool_from_keyfile("codexec_vte", &codeExecVte);
     _populate_bool_from_keyfile("rt_show_white_spaces", &rtShowWhiteSpaces);
     _populate_bool_from_keyfile("pt_show_white_spaces", &ptShowWhiteSpaces);
     _populate_bool_from_keyfile("rt_highl_curr_line", &rtHighlCurrLine);
@@ -558,6 +567,7 @@ void CtConfig::_populate_data_from_keyfile()
     _populate_int_from_keyfile("space_around_lines", &spaceAroundLines);
     _populate_int_from_keyfile("relative_wrapped_space", &relativeWrappedSpace);
     _populate_string_from_keyfile("h_rule", &hRule);
+    _populate_string_from_keyfile("vte_shell", &vteShell);
     _populate_string_from_keyfile("special_chars", &specialChars);
     _populate_string_from_keyfile("last_special_char", &lastSpecialChar);
     _populate_string_from_keyfile("selword_chars", &selwordChars);
@@ -608,6 +618,7 @@ void CtConfig::_populate_data_from_keyfile()
     _populate_int_from_keyfile("table_rows", &tableRows);
     _populate_int_from_keyfile("table_columns", &tableColumns);
     _populate_int_from_keyfile("table_col_width", &tableColWidthDefault);
+    _populate_int_from_keyfile("table_cells_go_light", &tableCellsGoLight);
 
     // [fonts]
     _currentGroup = "fonts";
@@ -615,6 +626,7 @@ void CtConfig::_populate_data_from_keyfile()
     _populate_string_from_keyfile("pt_font", &ptFont);
     _populate_string_from_keyfile("tree_font", &treeFont);
     _populate_string_from_keyfile("code_font", &codeFont);
+    _populate_string_from_keyfile("vte_font", &vteFont);
 
     // [colors]
     _currentGroup = "colors";
