@@ -1,7 +1,7 @@
 /*
  * ct_const.h
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -60,7 +60,12 @@ const inline static gchar* RICH_TEXT_ID             {"custom-colors"};
 const inline static gchar* TABLE_CELL_TEXT_ID       {"table-cell-text"};
 const inline static gchar* PLAIN_TEXT_ID            {"plain-text"};
 const inline static gchar* STYLE_APPLIED_ID         {"<style-applied>"};
-const inline static gchar* SYN_HIGHL_BASH           {"sh"};
+const inline static gchar* SYN_HIGHL_SHELL          {"sh"};
+#if defined(__APPLE__)
+const inline static gchar* VTE_SHELL_DEFAULT        {"/bin/zsh"};
+#else
+const inline static gchar* VTE_SHELL_DEFAULT        {"/bin/bash"};
+#endif
 const inline static gchar* STYLE_SCHEME_LIGHT       {"classic"};
 const inline static gchar* STYLE_SCHEME_DARK        {"cobalt"};
 const inline static gchar* TIMESTAMP_FORMAT_DEFAULT {"%Y/%m/%d - %H:%M"};
@@ -68,6 +73,7 @@ const inline static gchar* FONT_RT_DEFAULT          {"Sans 11"};
 const inline static gchar* FONT_PT_DEFAULT          {"Sans 11"};
 const inline static gchar* FONT_TREE_DEFAULT        {"Sans 10"};
 const inline static gchar* FONT_CODE_DEFAULT        {"Monospace 11"};
+const inline static gchar* FONT_VTE_DEFAULT         {"Monospace 10"};
 const inline static gchar* FONT_MS_DEFAULT          {"Monospace 11"};
 
 const inline static Glib::ustring SPECIAL_CHARS_DEFAULT {"“”„‘’•◇▪▸☐☑☒★…‰€©®™°↓↑→←↔↵⇓⇑⇒⇐⇔»«▼▲►◄≤≥≠≈±¹²³½¼⅛×÷∞ø∑σ√∫ΔδΠπΣΦΩωαβγεηλμ☺☻☼♥♣♦✔♀♂♪♫✝"};
@@ -249,7 +255,7 @@ const inline static gchar* LANG_DEFAULT{"default"};
 
 const inline static std::vector<const gchar*> AVAILABLE_LANGS {
     LANG_DEFAULT, "ar", "bg", "cs", "de", "el", "en", "es", "fi", "fr", "hi_IN", "hr", "hu", "hy", "it",
-    "ja", "kk_KZ", "ko", "lt", "nl", "pl", "pt", "pt_BR", "ro", "ru", "sl", "sv", "tr", "uk", "zh_CN", "zh_TW"
+    "ja", "kk_KZ", "kk_LA", "ko", "lt", "nl", "pl", "pt", "pt_BR", "ro", "ru", "sl", "sv", "tr", "uk", "zh_CN", "zh_TW"
 };
 
 const inline static int NODE_ICON_CODE_ID          {38};
@@ -257,6 +263,7 @@ const inline static int NODE_ICON_BULLET_ID        {25};
 const inline static int NODE_ICON_NO_ICON_ID       {26};
 const inline static int NODE_ICON_SIZE             {16};
 const inline static int MAX_TOOLTIP_LINK_CHARS     {150};
+const inline static int ADVISED_TABLE_LIGHT_HEAVY  {25};
 
 const inline static std::vector<const gchar*> NODE_CUSTOM_ICONS {
     nullptr,            // NEVER USED
@@ -399,11 +406,12 @@ const inline static std::vector<const gchar*> NODE_CUSTOM_ICONS {
     "ct_docker",       // 137
     "ct_gcp",          // 138
     "ct_kubernetes",   // 139
+    "ct_csharp",       // 140
 };
 
 const inline static std::vector<int> NODE_CUSTOM_ICONS_ORDERED {
     132, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 20, 133,
-    /*coding start*/17, 21, 22, 23, 24, 38, 46, 47, 48, 49/*coding end*/,
+    /*coding start*/17, 21, 22, 23, 24, 38, 46, 47, 48, 49, 140/*coding end*/,
     114, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 39, 40, 41, 42, 43, 44, 45, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108, 109, 110, 111, 112, 126, 113,
     /*os start*/115, 116, 117, 118, 119, 120, 121/*os end*/,
     /*devops start*/134, 135, 136, 137, 138, 139/*devops end*/,
@@ -425,7 +433,7 @@ const inline static std::array<const gchar*, 11> NODE_CHERRY_ICONS {
     CHERRY_GRAY         // 10
 };
 
-const inline static std::array<std::pair<const gchar*, const gchar*>, 12> NODE_CODE_ICONS {
+const inline static std::vector<std::pair<const gchar*, const gchar*>> NODE_CODE_ICONS {
     std::make_pair("python", "ct_python"),
     std::make_pair("python3", "ct_python"),
     std::make_pair("perl", "ct_perl"),
@@ -437,14 +445,16 @@ const inline static std::array<std::pair<const gchar*, const gchar*>, 12> NODE_C
     std::make_pair("xml", "ct_xml"),
     std::make_pair("c", "ct_c"),
     std::make_pair("cpp", "ct_cpp"),
+    std::make_pair("c-sharp", "ct_csharp"),
     std::make_pair("ruby", "ct_ruby")
 };
 
 const inline static gchar* CODE_EXEC_TMP_SRC  {"<tmp_src_path>"};
 const inline static gchar* CODE_EXEC_TMP_BIN  {"<tmp_bin_path>"};
+const inline static gchar* CODE_EXEC_CODE_TXT {"<code_txt>"};
 const inline static gchar* CODE_EXEC_COMMAND  {"<command>"};
 
-const inline static std::array<std::pair<const std::string, const std::string>, 8> CODE_EXEC_TYPE_CMD_DEFAULT {
+const inline static std::vector<std::pair<const std::string, const std::string>> CODE_EXEC_TYPE_CMD_DEFAULT {
     std::make_pair("c",          std::string("gcc -o ") + CODE_EXEC_TMP_BIN + " " + CODE_EXEC_TMP_SRC + " && " + CODE_EXEC_TMP_BIN),
     std::make_pair("cpp",        std::string("g++ -o ") + CODE_EXEC_TMP_BIN + " " + CODE_EXEC_TMP_SRC + " && " + CODE_EXEC_TMP_BIN),
     std::make_pair("dosbatch",   std::string("call ") + CODE_EXEC_TMP_SRC),
@@ -452,10 +462,11 @@ const inline static std::array<std::pair<const std::string, const std::string>,
     std::make_pair("powershell", std::string("powershell -File ") + CODE_EXEC_TMP_SRC),
     std::make_pair("python",     std::string("python2 ") + CODE_EXEC_TMP_SRC),
     std::make_pair("python3",    std::string("python3 ") + CODE_EXEC_TMP_SRC),
-    std::make_pair("sh",         std::string("sh ") + CODE_EXEC_TMP_SRC)
+    std::make_pair("sh",         CODE_EXEC_CODE_TXT),
+    std::make_pair("c-sharp",    std::string("csc /out:") + CODE_EXEC_TMP_BIN + " " + CODE_EXEC_TMP_SRC + " && " + CODE_EXEC_TMP_BIN)
 };
 
-const inline static std::array<std::pair<const std::string, const std::string>, 8> CODE_EXEC_TYPE_EXT_DEFAULT {
+const inline static std::vector<std::pair<const std::string, const std::string>> CODE_EXEC_TYPE_EXT_DEFAULT {
     std::make_pair("c",          "c"),
     std::make_pair("cpp",        "cpp"),
     std::make_pair("dosbatch",   "bat"),
@@ -463,7 +474,8 @@ const inline static std::array<std::pair<const std::string, const std::string>,
     std::make_pair("powershell", "ps1"),
     std::make_pair("python",     "py"),
     std::make_pair("python3",    "py"),
-    std::make_pair("sh",         "sh")
+    std::make_pair("sh",         "sh"),
+    std::make_pair("c-sharp",    "cs")
 };
 
 const inline static std::unordered_map<std::string, std::string> CODE_EXEC_TERM_RUN_DEFAULT {
@@ -488,8 +500,14 @@ const inline static Glib::ustring TARGET_WIN_HTML = "HTML Format";
 const inline static std::vector<Glib::ustring> TARGETS_HTML = {"text/html", TARGET_WIN_HTML};
 const inline static Glib::ustring TARGET_URI_LIST = "text/uri-list";
 const inline static Glib::ustring TARGET_GTK_TEXT_BUFFER_CONTENTS = "GTK_TEXT_BUFFER_CONTENTS";
-const inline static std::vector<Glib::ustring> TARGETS_PLAIN_TEXT = {"UTF8_STRING", "COMPOUND_TEXT", "STRING", "TEXT"};
+const inline static std::vector<Glib::ustring> TARGETS_PLAIN_TEXT = {"UTF8_STRING", "COMPOUND_TEXT", "STRING", "TEXT", "text/plain;charset=utf-8", "text/plain"};
 const inline static std::vector<Glib::ustring> TARGETS_IMAGES = {"image/png", "image/jpeg", "image/bmp", "image/tiff", "image/x-MS-bmp", "image/x-bmp"};
 const inline static Glib::ustring TARGET_WINDOWS_FILE_NAME = "FileName";
+const inline static Glib::ustring TAG_UL_START = "<ul>";
+const inline static Glib::ustring TAG_UL_END = "</ul>";
+const inline static Glib::ustring TAG_OL_START = "<ol>";
+const inline static Glib::ustring TAG_OL_END = "</ol>";
+const inline static Glib::ustring TAG_LI_START = "<li>";
+const inline static Glib::ustring TAG_LI_END = "</li>";
 
 }; // struct CtConst
