@@ -1,7 +1,7 @@
 /*
  * ct_dialogs.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -151,7 +151,7 @@ void CtDialogs::bookmarks_handle_dialog(CtMainWin* pCtMainWin)
             int cmp = l->get_value(rModel->columns.desc).compare(r->get_value(rModel->columns.desc));
             return (cmp > 0);
         };
-        CtMiscUtil::node_siblings_sort_iteration(rModel, rModel->children(), need_swap);
+        CtMiscUtil::node_siblings_sort(rModel, rModel->children(), need_swap);
     });
     button_sort_desc.signal_clicked().connect([&rModel]()
     {
@@ -160,7 +160,7 @@ void CtDialogs::bookmarks_handle_dialog(CtMainWin* pCtMainWin)
             int cmp = l->get_value(rModel->columns.desc).compare(r->get_value(rModel->columns.desc));
             return (cmp < 0);
         };
-        CtMiscUtil::node_siblings_sort_iteration(rModel, rModel->children(), need_swap);
+        CtMiscUtil::node_siblings_sort(rModel, rModel->children(), need_swap);
     });
 
     if (dialog.run() != Gtk::RESPONSE_ACCEPT)
@@ -210,11 +210,11 @@ void CtDialogs::bookmarks_handle_dialog(CtMainWin* pCtMainWin)
 // Choose the CherryTree data storage type (xml or db) and protection
 bool CtDialogs::choose_data_storage_dialog(storage_select_args& args)
 {
-    Gtk::Dialog dialog(_("Choose Storage Type"),
-                       *args.pParentWin,
-                       Gtk::DialogFlags::DIALOG_MODAL | Gtk::DialogFlags::DIALOG_DESTROY_WITH_PARENT);
-    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_REJECT);
-    dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_ACCEPT);
+    Gtk::Dialog dialog{_("Choose Storage Type"),
+                       *args.pCtMainWin,
+                       Gtk::DialogFlags::DIALOG_MODAL | Gtk::DialogFlags::DIALOG_DESTROY_WITH_PARENT};
+    Gtk::Button* pButtonCancel = dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_REJECT);
+    Gtk::Button* pButtonOk = dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_ACCEPT);
     dialog.set_default_size(350, -1);
     dialog.set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
 
@@ -254,27 +254,21 @@ bool CtDialogs::choose_data_storage_dialog(storage_select_args& args)
     passw_frame.set_shadow_type(Gtk::SHADOW_NONE);
     passw_frame.add(vbox_passw);
 
-    if (args.ctDocEncrypt == CtDocEncrypt::False)
-    {
+    if (args.ctDocEncrypt == CtDocEncrypt::False) {
         passw_frame.set_sensitive(false);
-        if (args.ctDocType == CtDocType::SQLite)
-        {
+        if (args.ctDocType == CtDocType::SQLite) {
             radiobutton_sqlite_not_protected.set_active(true);
         }
-        else if (args.ctDocType == CtDocType::XML)
-        {
+        else if (args.ctDocType == CtDocType::XML) {
             radiobutton_xml_not_protected.set_active(true);
         }
     }
-    else if (args.ctDocEncrypt == CtDocEncrypt::True)
-    {
+    else if (args.ctDocEncrypt == CtDocEncrypt::True) {
         passw_frame.set_sensitive(true);
-        if (args.ctDocType == CtDocType::SQLite)
-        {
+        if (args.ctDocType == CtDocType::SQLite) {
             radiobutton_sqlite_pass_protected.set_active(true);
         }
-        else
-        {
+        else {
             radiobutton_xml_pass_protected.set_active(true);
         }
     }
@@ -283,29 +277,59 @@ bool CtDialogs::choose_data_storage_dialog(storage_select_args& args)
         passw_frame.set_sensitive(false);
     }
 
+    auto pCtMainWin = args.pCtMainWin;
+    auto pCtConfig = pCtMainWin->get_ct_config();
+    auto hbox_autosave = Gtk::manage(new Gtk::Box{Gtk::ORIENTATION_HORIZONTAL, 4/*spacing*/});
+    auto checkbutton_autosave = Gtk::manage(new Gtk::CheckButton{_("Autosave Every")});
+    Glib::RefPtr<Gtk::Adjustment> adjustment_autosave = Gtk::Adjustment::create(pCtConfig->autosaveVal, 1, 1000, 1);
+    auto spinbutton_autosave = Gtk::manage(new Gtk::SpinButton(adjustment_autosave));
+    auto label_autosave = Gtk::manage(new Gtk::Label{_("Minutes")});
+    checkbutton_autosave->set_active(pCtConfig->autosaveOn);
+    spinbutton_autosave->set_value(pCtConfig->autosaveVal);
+    spinbutton_autosave->set_sensitive(pCtConfig->autosaveOn);
+    hbox_autosave->pack_start(*checkbutton_autosave, false, false);
+    hbox_autosave->pack_start(*spinbutton_autosave, false, false);
+    hbox_autosave->pack_start(*label_autosave, false, false);
+
+    checkbutton_autosave->signal_toggled().connect([pCtConfig, pCtMainWin, checkbutton_autosave, spinbutton_autosave](){
+        pCtConfig->autosaveOn = checkbutton_autosave->get_active();
+        pCtMainWin->file_autosave_restart();
+        spinbutton_autosave->set_sensitive(pCtConfig->autosaveOn);
+    });
+    spinbutton_autosave->signal_value_changed().connect([pCtConfig, pCtMainWin, spinbutton_autosave](){
+        pCtConfig->autosaveVal = spinbutton_autosave->get_value_as_int();
+        pCtMainWin->file_autosave_restart();
+    });
+
     Gtk::Box* pContentArea = dialog.get_content_area();
     pContentArea->set_spacing(5);
     pContentArea->pack_start(type_frame);
     pContentArea->pack_start(passw_frame);
+    if (args.showAutosaveOptions) {
+        pContentArea->pack_start(*hbox_autosave);
+    }
     pContentArea->show_all();
 
-    auto on_radiobutton_savetype_toggled = [&]()
-    {
+    auto on_radiobutton_savetype_toggled = [&](){
         if ( radiobutton_sqlite_pass_protected.get_active() ||
              radiobutton_xml_pass_protected.get_active() )
         {
             passw_frame.set_sensitive(true);
             entry_passw_1.grab_focus();
         }
-        else
-        {
+        else {
             passw_frame.set_sensitive(false);
         }
     };
     auto on_key_press_edit_data_storage_type_dialog = [&](GdkEventKey* pEventKey)->bool{
         if (GDK_KEY_Return == pEventKey->keyval or GDK_KEY_KP_Enter == pEventKey->keyval) {
-            Gtk::Button* pButton = static_cast<Gtk::Button*>(dialog.get_widget_for_response(Gtk::RESPONSE_ACCEPT));
-            pButton->clicked();
+            pButtonOk->grab_focus();
+            pButtonOk->clicked();
+            return true;
+        }
+        if (GDK_KEY_Escape == pEventKey->keyval) {
+            pButtonCancel->grab_focus();
+            pButtonCancel->clicked();
             return true;
         }
         return false;
@@ -319,8 +343,7 @@ bool CtDialogs::choose_data_storage_dialog(storage_select_args& args)
     dialog.hide();
 
     bool retVal{Gtk::RESPONSE_ACCEPT == response};
-    if (retVal)
-    {
+    if (retVal) {
         args.ctDocType = (radiobutton_xml_not_protected.get_active() || radiobutton_xml_pass_protected.get_active() ?
                          CtDocType::XML : CtDocType::SQLite);
         args.ctDocEncrypt = (radiobutton_sqlite_pass_protected.get_active() || radiobutton_xml_pass_protected.get_active() ?
@@ -328,14 +351,12 @@ bool CtDialogs::choose_data_storage_dialog(storage_select_args& args)
         if (CtDocEncrypt::True == args.ctDocEncrypt)
         {
             args.password = entry_passw_1.get_text();
-            if (args.password.empty())
-            {
-                error_dialog(_("The Password Fields Must be Filled"), *args.pParentWin);
+            if (args.password.empty()) {
+                error_dialog(_("The Password Fields Must be Filled."), *args.pCtMainWin);
                 retVal = false;
             }
-            else if (args.password != entry_passw_2.get_text())
-            {
-                error_dialog(_("The Two Inserted Passwords Do Not Match"), *args.pParentWin);
+            else if (args.password != entry_passw_2.get_text()) {
+                error_dialog(_("The Two Inserted Passwords Do Not Match."), *args.pCtMainWin);
                 retVal = false;
             }
         }
@@ -343,14 +364,15 @@ bool CtDialogs::choose_data_storage_dialog(storage_select_args& args)
     return retVal;
 }
 
-CtYesNoCancel CtDialogs::exit_save_dialog(Gtk::Window& parent)
+CtYesNoCancel CtDialogs::exit_save_dialog(CtMainWin& ct_main_win)
 {
     Gtk::Dialog dialog = Gtk::Dialog(_("Warning"),
-                                     parent,
+                                     ct_main_win,
                                      Gtk::DialogFlags::DIALOG_MODAL | Gtk::DialogFlags::DIALOG_DESTROY_WITH_PARENT);
-    dialog.add_button(Gtk::Stock::DISCARD, Gtk::RESPONSE_NO);
-    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-    dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_YES);
+    Gtk::Button* pButtonDiscard = dialog.add_button(Gtk::Stock::DISCARD, Gtk::RESPONSE_NO);
+    pButtonDiscard->set_image(*ct_main_win.new_managed_image_from_stock("ct_clear", Gtk::ICON_SIZE_BUTTON));
+    Gtk::Button* pButtonCancel = dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+    Gtk::Button* pButtonSave = dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_YES);
     dialog.set_default_response(Gtk::RESPONSE_YES);
     dialog.set_default_size(350, 150);
     dialog.set_position(Gtk::WindowPosition::WIN_POS_CENTER_ON_PARENT);
@@ -366,15 +388,13 @@ CtYesNoCancel CtDialogs::exit_save_dialog(Gtk::Window& parent)
     pContentArea->pack_start(hbox);
     auto on_key_press_dialog = [&](GdkEventKey* pEventKey)->bool{
         if (GDK_KEY_Return == pEventKey->keyval or GDK_KEY_KP_Enter == pEventKey->keyval) {
-            Gtk::Button* pButton = static_cast<Gtk::Button*>(dialog.get_widget_for_response(Gtk::RESPONSE_YES));
-            pButton->grab_focus();
-            pButton->clicked();
+            pButtonSave->grab_focus();
+            pButtonSave->clicked();
             return true;
         }
         if (GDK_KEY_Escape == pEventKey->keyval) {
-            Gtk::Button* pButton = static_cast<Gtk::Button*>(dialog.get_widget_for_response(Gtk::RESPONSE_CANCEL));
-            pButton->grab_focus();
-            pButton->clicked();
+            pButtonCancel->grab_focus();
+            pButtonCancel->clicked();
             return true;
         }
         return false;
@@ -392,41 +412,60 @@ CtYesNoCancel CtDialogs::exit_save_dialog(Gtk::Window& parent)
     return CtYesNoCancel::Cancel;
 }
 
-bool CtDialogs::exec_code_confirm_dialog(CtMainWin& ct_main_win)
+bool CtDialogs::exec_code_confirm_dialog(CtMainWin& ct_main_win,
+                                         const std::string& syntax_highl,
+                                         const Glib::ustring& code_txt)
 {
     Gtk::Dialog dialog = Gtk::Dialog(_("Warning"),
                                      ct_main_win,
                                      Gtk::DialogFlags::DIALOG_MODAL | Gtk::DialogFlags::DIALOG_DESTROY_WITH_PARENT);
-    dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
-    dialog.add_button(Gtk::Stock::EXECUTE, Gtk::RESPONSE_YES);
+    Gtk::Button* pButtonCancel = dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
+    Gtk::Button* pButtonExecute = dialog.add_button(Gtk::Stock::EXECUTE, Gtk::RESPONSE_YES);
     dialog.set_default_response(Gtk::RESPONSE_YES);
     dialog.set_default_size(350, 150);
     dialog.set_position(Gtk::WindowPosition::WIN_POS_CENTER_ON_PARENT);
     Gtk::Image image;
-    image.set_from_icon_name("ct_warning", Gtk::ICON_SIZE_DIALOG);
-    Gtk::Label label(Glib::ustring{"<b>"}+_("Do you want to Execute the Code?")+"</b>");
+    image.set_from_icon_name(ct_main_win.get_code_icon_name(syntax_highl), Gtk::ICON_SIZE_DIALOG);
+    image.set_tooltip_text(syntax_highl);
+    Gtk::Label label{Glib::ustring{"<b>"}+_("Do you want to Execute the Code?")+"</b>"};
     label.set_use_markup(true);
+    Gtk::Label label_separator{" "};
     Gtk::HBox hbox;
     hbox.pack_start(image);
     hbox.pack_start(label);
     hbox.set_spacing(5);
+    Gtk::Label code_preview;
+    if (code_txt.size() < 200) {
+        code_preview.set_markup(Glib::ustring{"<tt>"} + code_txt + "</tt>");
+    }
+    else {
+        code_preview.set_markup(Glib::ustring{"<tt>"} + code_txt.substr(0, 200) + " ...</tt>");
+    }
     Gtk::CheckButton checkbutton_code_exec_confirm{_("Ask Confirmation Before Executing the Code")};
     checkbutton_code_exec_confirm.set_active(ct_main_win.get_ct_config()->codeExecConfirm);
     checkbutton_code_exec_confirm.set_can_focus(false);
+#if defined(HAVE_VTE)
+    Gtk::CheckButton checkbutton_code_exec_vte{_("Use Internal Terminal")};
+    checkbutton_code_exec_vte.set_active(ct_main_win.get_ct_config()->codeExecVte);
+    checkbutton_code_exec_vte.set_can_focus(false);
+#endif // HAVE_VTE
     Gtk::Box* pContentArea = dialog.get_content_area();
     pContentArea->pack_start(hbox);
+    pContentArea->pack_start(code_preview);
+    pContentArea->pack_start(label_separator);
     pContentArea->pack_start(checkbutton_code_exec_confirm);
+#if defined(HAVE_VTE)
+    pContentArea->pack_start(checkbutton_code_exec_vte);
+#endif // HAVE_VTE
     auto on_key_press_dialog = [&](GdkEventKey* pEventKey)->bool{
         if (GDK_KEY_Return == pEventKey->keyval or GDK_KEY_KP_Enter == pEventKey->keyval) {
-            Gtk::Button* pButton = static_cast<Gtk::Button*>(dialog.get_widget_for_response(Gtk::RESPONSE_YES));
-            pButton->grab_focus();
-            pButton->clicked();
+            pButtonExecute->grab_focus();
+            pButtonExecute->clicked();
             return true;
         }
         if (GDK_KEY_Escape == pEventKey->keyval) {
-            Gtk::Button* pButton = static_cast<Gtk::Button*>(dialog.get_widget_for_response(Gtk::RESPONSE_CANCEL));
-            pButton->grab_focus();
-            pButton->clicked();
+            pButtonCancel->grab_focus();
+            pButtonCancel->clicked();
             return true;
         }
         return false;
@@ -435,6 +474,11 @@ bool CtDialogs::exec_code_confirm_dialog(CtMainWin& ct_main_win)
     checkbutton_code_exec_confirm.signal_toggled().connect([&](){
         ct_main_win.get_ct_config()->codeExecConfirm = checkbutton_code_exec_confirm.get_active();
     });
+#if defined(HAVE_VTE)
+    checkbutton_code_exec_vte.signal_toggled().connect([&](){
+        ct_main_win.get_ct_config()->codeExecVte = checkbutton_code_exec_vte.get_active();
+    });
+#endif // HAVE_VTE
     pContentArea->show_all();
     const int response = dialog.run();
     dialog.hide();
@@ -450,7 +494,7 @@ void CtDialogs::dialog_about(Gtk::Window& parent, Glib::RefPtr<Gdk::Pixbuf> icon
     auto dialog = Gtk::AboutDialog();
     dialog.set_program_name("CherryTree");
     dialog.set_version(CtConst::CT_VERSION);
-    dialog.set_copyright("Copyright © 2009-2022\n"
+    dialog.set_copyright("Copyright © 2009-2023\n"
                          "Giuseppe Penone <giuspen@gmail.com>\n"
                          "Evgenii Gurianov <https://github.com/txe>");
     dialog.set_comments(_("A Hierarchical Note Taking Application, featuring Rich Text and Syntax Highlighting"));
@@ -491,6 +535,7 @@ MA 02110-1301, USA.
  _("Italian")+" (it) Vincenzo Reale <smart2128vr@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Japanese")+" (ja) Piyo <py2@live.jp>"+CtConst::CHAR_NEWLINE+
  _("Kazakh")+" (kk_KZ) Viktor Polyanskiy <camilot55@yandex.ru>"+CtConst::CHAR_NEWLINE+
+ _("Kazakh (Latin)")+" (kk_LA) Viktor Polyanskiy <camilot55@yandex.ru>"+CtConst::CHAR_NEWLINE+
  _("Korean")+" (ko) Sean Lee <icarusean@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Lithuanian")+" (lt) Zygis <zygimantus@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Polish")+" (pl) Daniel Napora <napcok@gmail.com>"+CtConst::CHAR_NEWLINE+
@@ -502,7 +547,7 @@ MA 02110-1301, USA.
  _("Spanish")+" (es) UserFav <userfav.post@gmail.com>"+CtConst::CHAR_NEWLINE+
  _("Swedish")+" (sv) Åke Engelbrektson <eson@svenskasprakfiler.se>"+CtConst::CHAR_NEWLINE+
  _("Turkish")+" (tr) Ferhat Aydin <ferhataydin44@gmail.com>"+CtConst::CHAR_NEWLINE+
- _("Ukrainian")+" (uk) Andriy Kovtun <kovtunos@yandex.ru>");
+ _("Ukrainian")+" (uk) Giuseppe Penone <giuspen@gmail.com>");
     dialog.set_logo(icon);
     dialog.set_title(_("About CherryTree"));
 
@@ -513,6 +558,7 @@ MA 02110-1301, USA.
 
     dialog.set_transient_for(parent);
     dialog.set_position(Gtk::WindowPosition::WIN_POS_CENTER_ON_PARENT);
+    dialog.property_destroy_with_parent() = true;
     dialog.set_modal(true);
     dialog.run();
 }
@@ -563,7 +609,7 @@ void CtDialogs::summary_info_dialog(CtMainWin* pCtMainWin, const CtSummaryInfo&
     Gtk::Label label_ta_key;
     label_ta_key.set_markup(Glib::ustring{"<b>"} + _("Number of Tables") + "</b>");
     grid.attach(label_ta_key, 0, 6, 1, 1);
-    Gtk::Label label_ta_val{std::to_string(summaryInfo.tables_num)};
+    Gtk::Label label_ta_val{str::format("{} + {}", summaryInfo.heavytables_num, summaryInfo.lighttables_num)};
     grid.attach(label_ta_val, 1, 6, 1, 1);
     Gtk::Label label_cb_key;
     label_cb_key.set_markup(Glib::ustring{"<b>"} + _("Number of CodeBoxes") + "</b>");
