@@ -1,7 +1,7 @@
 /*
  * ct_dialogs_find.cc
  *
- * Copyright 2009-2021
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -25,16 +25,16 @@
 #include "ct_main_win.h"
 #include "ct_actions.h"
 
-std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
+std::string CtDialogs::dialog_search(CtMainWin* pCtMainWin,
                                      const std::string& title,
                                      CtSearchOptions& s_options,
                                      bool replace_on,
                                      bool multiple_nodes)
 {
     Gtk::Dialog dialog{title,
-                       *pParentWin,
+                       *pCtMainWin,
                        Gtk::DialogFlags::DIALOG_MODAL | Gtk::DialogFlags::DIALOG_DESTROY_WITH_PARENT};
-    dialog.set_transient_for(*pParentWin);
+    dialog.set_transient_for(*pCtMainWin);
     dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_REJECT);
     dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_ACCEPT);
     dialog.set_default_response(Gtk::RESPONSE_ACCEPT);
@@ -65,10 +65,13 @@ std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
         replace_frame->add(*replace_entry);
     }
     Gtk::Box opt_vbox{Gtk::ORIENTATION_VERTICAL, 1/*spacing*/};
+    Gtk::Box reg_exp_hbox{Gtk::ORIENTATION_HORIZONTAL, 1/*spacing*/};
     Gtk::Box four_1_hbox{Gtk::ORIENTATION_HORIZONTAL};
     four_1_hbox.set_homogeneous(true);
     Gtk::Box four_2_hbox{Gtk::ORIENTATION_HORIZONTAL};
     four_2_hbox.set_homogeneous(true);
+    Gtk::Box four_3_hbox{Gtk::ORIENTATION_HORIZONTAL};
+    four_3_hbox.set_homogeneous(true);
     Gtk::Box bw_fw_hbox{Gtk::ORIENTATION_HORIZONTAL};
     bw_fw_hbox.set_homogeneous(true);
     Gtk::Box three_hbox{Gtk::ORIENTATION_HORIZONTAL};
@@ -78,8 +81,15 @@ std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
     match_case_checkbutton.set_active(s_options.match_case);
     Gtk::CheckButton reg_exp_checkbutton{_("Regular Expression")};
     reg_exp_checkbutton.set_active(s_options.reg_exp);
+    Gtk::Button reg_exp_help_button;
+    reg_exp_help_button.set_image(*pCtMainWin->new_managed_image_from_stock("ct_help", Gtk::ICON_SIZE_BUTTON));
+    reg_exp_help_button.set_tooltip_text(_("Online Manual"));
+    reg_exp_hbox.pack_start(reg_exp_checkbutton);
+    reg_exp_hbox.pack_start(reg_exp_help_button);
     Gtk::CheckButton accent_insensitive_checkbutton{_("Accent Insensitive")};
     accent_insensitive_checkbutton.set_active(s_options.accent_insensitive);
+    Gtk::CheckButton override_exclusions_checkbutton{_("Override Exclusions")};
+    override_exclusions_checkbutton.set_active(s_options.override_exclusions);
     Gtk::CheckButton whole_word_checkbutton{_("Whole Word")};
     whole_word_checkbutton.set_active(s_options.whole_word);
     Gtk::CheckButton start_word_checkbutton{_("Start Word")};
@@ -188,9 +198,11 @@ std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
     Gtk::CheckButton iter_dialog_checkbutton{_("Show Iterated Find/Replace Dialog")};
     iter_dialog_checkbutton.set_active(s_options.iterative_dialog);
     four_1_hbox.pack_start(match_case_checkbutton);
-    four_1_hbox.pack_start(reg_exp_checkbutton);
+    four_1_hbox.pack_start(reg_exp_hbox);
     four_2_hbox.pack_start(whole_word_checkbutton);
     four_2_hbox.pack_start(start_word_checkbutton);
+    four_3_hbox.pack_start(accent_insensitive_checkbutton);
+    four_3_hbox.pack_start(override_exclusions_checkbutton);
     bw_fw_hbox.pack_start(fw_radiobutton);
     bw_fw_hbox.pack_start(bw_radiobutton);
     three_hbox.pack_start(all_radiobutton);
@@ -199,7 +211,7 @@ std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
     three_hbox.pack_start(three_vbox);
     opt_vbox.pack_start(four_1_hbox);
     opt_vbox.pack_start(four_2_hbox);
-    opt_vbox.pack_start(accent_insensitive_checkbutton);
+    opt_vbox.pack_start(four_3_hbox);
     opt_vbox.pack_start(*Gtk::manage(new Gtk::HSeparator{}));
     opt_vbox.pack_start(bw_fw_hbox);
     opt_vbox.pack_start(*Gtk::manage(new Gtk::HSeparator{}));
@@ -224,6 +236,10 @@ std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
     content_area->show_all();
     search_entry.grab_focus();
 
+    reg_exp_help_button.signal_clicked().connect([](){
+        fs::open_weblink("https://developer-old.gnome.org/glib/stable/glib-regex-syntax.html");
+    });
+
     auto press_enter = [&dialog, &button_ok](GdkEventKey* pEventKey){
         if (GDK_KEY_Return == pEventKey->keyval or GDK_KEY_KP_Enter == pEventKey->keyval) {
             if (button_ok && button_ok->get_sensitive()) {
@@ -248,6 +264,7 @@ std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
     s_options.match_case = match_case_checkbutton.get_active();
     s_options.reg_exp = reg_exp_checkbutton.get_active();
     s_options.accent_insensitive = accent_insensitive_checkbutton.get_active();
+    s_options.override_exclusions = override_exclusions_checkbutton.get_active();
     s_options.whole_word = whole_word_checkbutton.get_active();
     s_options.start_word = start_word_checkbutton.get_active();
     s_options.direction_fw = fw_radiobutton.get_active();
@@ -262,6 +279,11 @@ std::string CtDialogs::dialog_search(Gtk::Window* pParentWin,
     s_options.node_name_n_tags = node_name_n_tags_checkbutton.get_active();
     s_options.only_sel_n_subnodes = only_sel_n_subnodes_checkbutton.get_active();
     s_options.iterative_dialog = iter_dialog_checkbutton.get_active();
+    // special cases (#2190)
+    if (s_options.reg_exp and s_options.str_find == ".*" ) {
+        s_options.node_content = false;
+        s_options.node_name_n_tags = true;
+    }
     return s_options.str_find;
 }
 
