@@ -1,7 +1,7 @@
 /*
  * ct_export2html.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -27,11 +27,12 @@
 #include "ct_dialogs.h"
 #include "ct_storage_control.h"
 #include "ct_logging.h"
-#include "ct_process.h"
 #include "ct_filesystem.h"
+#include "ct_list.h"
 
 CtExport2Html::CtExport2Html(CtMainWin* pCtMainWin)
- : _pCtMainWin(pCtMainWin)
+ : _pCtMainWin{pCtMainWin}
+ , _pCtConfig{pCtMainWin->get_ct_config()}
 {
 }
 
@@ -40,7 +41,7 @@ bool CtExport2Html::prepare_html_folder(fs::path dir_place, fs::path new_folder,
 {
     if (dir_place.empty())
     {
-        dir_place = CtDialogs::folder_select_dialog(_pCtMainWin->get_ct_config()->pickDirExport, _pCtMainWin);
+        dir_place = CtDialogs::folder_select_dialog(_pCtConfig->pickDirExport, _pCtMainWin);
         if (dir_place.empty())
             return false;
     }
@@ -106,13 +107,13 @@ void CtExport2Html::node_export_to_html(CtTreeIter tree_iter, const CtExportOpti
             node_html_text += html_slots[i];
             if (i < widgets.size()) {
                 try {
-                    if (CtImageEmbFile* embfile = dynamic_cast<CtImageEmbFile*>(widgets[i]))
+                    if (auto embfile = dynamic_cast<CtImageEmbFile*>(widgets[i]))
                         node_html_text += _get_embfile_html(embfile, tree_iter, _embed_dir);
-                    else if (CtImage* image = dynamic_cast<CtImage*>(widgets[i]))
+                    else if (auto image = dynamic_cast<CtImage*>(widgets[i]))
                         node_html_text += _get_image_html(image, _images_dir, images_count, &tree_iter);
-                    else if (CtTable* table = dynamic_cast<CtTable*>(widgets[i]))
+                    else if (auto table = dynamic_cast<CtTableCommon*>(widgets[i]))
                         node_html_text += _get_table_html(table);
-                    else if (CtCodebox* codebox = dynamic_cast<CtCodebox*>(widgets[i]))
+                    else if (auto codebox = dynamic_cast<CtCodebox*>(widgets[i]))
                         node_html_text += _get_codebox_html(codebox);
                 }
                 catch (std::exception& ex) {
@@ -234,13 +235,13 @@ void CtExport2Html::nodes_all_export_to_single_html(bool all_tree, const CtExpor
                 node_html_text += html_slots[i];
                 if (i < widgets.size()) {
                     try {
-                        if (CtImageEmbFile* embfile = dynamic_cast<CtImageEmbFile*>(widgets[i]))
+                        if (auto embfile = dynamic_cast<CtImageEmbFile*>(widgets[i]))
                             node_html_text += _get_embfile_html(embfile, tree_iter, _embed_dir);
-                        else if (CtImage* image = dynamic_cast<CtImage*>(widgets[i]))
+                        else if (auto image = dynamic_cast<CtImage*>(widgets[i]))
                             node_html_text += _get_image_html(image, _images_dir, images_count, &tree_iter);
-                        else if (CtTable* table = dynamic_cast<CtTable*>(widgets[i]))
+                        else if (auto table = dynamic_cast<CtTableCommon*>(widgets[i]))
                             node_html_text += _get_table_html(table);
-                        else if (CtCodebox* codebox = dynamic_cast<CtCodebox*>(widgets[i]))
+                        else if (auto codebox = dynamic_cast<CtCodebox*>(widgets[i]))
                             node_html_text += _get_codebox_html(codebox);
                     }
                     catch (std::exception& ex) {
@@ -331,13 +332,13 @@ Glib::ustring CtExport2Html::selection_export_to_html(Glib::RefPtr<Gtk::TextBuff
         std::list<CtAnchoredWidget*> widgets = _pCtMainWin->curr_tree_iter().get_anchored_widgets(start_iter.get_offset(), end_iter.get_offset());
         for (CtAnchoredWidget* widget : widgets) {
             int end_offset = widget->getOffset();
-            node_html_text +=_html_process_slot(start_offset, end_offset, text_buffer);
+            node_html_text += html_process_slot(_pCtConfig, _pCtMainWin, start_offset, end_offset, text_buffer);
             if (CtImage* image = dynamic_cast<CtImage*>(widget)) node_html_text += _get_image_html(image, tempFolder, images_count, nullptr);
-            else if (CtTable* table = dynamic_cast<CtTable*>(widget)) node_html_text += _get_table_html(table);
-            else if (CtCodebox* codebox = dynamic_cast<CtCodebox*>(widget)) node_html_text += _get_codebox_html(codebox);
+            else if (auto table = dynamic_cast<CtTableCommon*>(widget)) node_html_text += _get_table_html(table);
+            else if (auto codebox = dynamic_cast<CtCodebox*>(widget)) node_html_text += _get_codebox_html(codebox);
             start_offset = end_offset;
         }
-        node_html_text += _html_process_slot(start_offset, end_iter.get_offset(), text_buffer);
+        node_html_text += html_process_slot(_pCtConfig, _pCtMainWin, start_offset, end_iter.get_offset(), text_buffer);
 
         std::vector<Glib::ustring> node_lines = str::split(node_html_text, CtConst::CHAR_NEWLINE);
         if (node_lines.size() > 0) {
@@ -361,7 +362,7 @@ Glib::ustring CtExport2Html::selection_export_to_html(Glib::RefPtr<Gtk::TextBuff
 }
 
 // Returns the HTML given the table
-Glib::ustring CtExport2Html::table_export_to_html(CtTable* table)
+Glib::ustring CtExport2Html::table_export_to_html(CtTableCommon* table)
 {
     Glib::ustring html_text = str::format(HTML_HEADER, "");
     html_text += _get_table_html(table);
@@ -412,7 +413,7 @@ Glib::ustring CtExport2Html::_get_image_html(CtImage* image, const fs::path& ima
     Glib::ustring image_html = "<img src=\"" + image_rel_path + "\" alt=\"" + image_rel_path + "\" />";
     CtImagePng* png = dynamic_cast<CtImagePng*>(image);
     if (png and not png->get_link().empty()) {
-        Glib::ustring href = _get_href_from_link_prop_val(png->get_link());
+        Glib::ustring href = _get_href_from_link_prop_val(_pCtMainWin, png->get_link());
         image_html = "<a href=\"" + href + "\">" + image_html + "</a>";
     }
 
@@ -430,25 +431,28 @@ Glib::ustring CtExport2Html::_get_codebox_html(CtCodebox* codebox)
 }
 
 // Returns the HTML Table
-Glib::ustring CtExport2Html::_get_table_html(CtTable* table)
+Glib::ustring CtExport2Html::_get_table_html(CtTableCommon* table)
 {
+    std::vector<std::vector<Glib::ustring>> rows;
+    table->write_strings_matrix(rows);
     Glib::ustring table_html = "<table class=\"table\">";
-    bool first = true;
-    for (const auto& row: table->get_table_matrix())
-    {
+    bool first{true};
+    for (const auto& row : rows) {
         table_html += "<tr>";
-        for (auto cell: row) {
-            Glib::ustring content = str::xml_escape(cell->get_text_content());
-            if (content.empty()) content = " "; // Otherwise the table will render with squashed cells
-
+        for (const Glib::ustring& cell : row) {
+            Glib::ustring content = str::xml_escape(cell);
+            if (content.empty()) {
+                content = " "; // Otherwise the table will render with squashed cells
+            }
             if (first) {
                 table_html += "<th>" + content + "</th>";
             } else {
                 table_html += "<td>" + content + "</td>";
             }
         }
-
-        if (first) first = false;
+        if (first) {
+            first = false;
+        }
         table_html += "</tr>";
     }
     table_html += "</table>";
@@ -461,19 +465,6 @@ Glib::ustring CtExport2Html::_html_get_from_code_buffer(const Glib::RefPtr<Gsv::
     Gtk::TextIter curr_iter = sel_start >= 0 ? code_buffer->get_iter_at_offset(sel_start) : code_buffer->begin();
     Gtk::TextIter end_iter = sel_end >= 0 ? code_buffer->get_iter_at_offset(sel_end) : code_buffer->end();
 
-    if (_pCtMainWin->get_ct_config()->usePandoc && CtPandoc::supports_syntax(syntax_highlighting)) {
-        if (CtPandoc::has_pandoc()) {
-            Glib::ustring      input_txt(curr_iter, end_iter);
-            std::istringstream ibuff(input_txt);
-            std::ostringstream html_out;
-            CtPandoc::to_html(ibuff, html_out, syntax_highlighting);
-            return html_out.str();
-        }
-        else {
-            spdlog::warn("Exported content is eligible for processing by Pandoc but Pandoc executable could not be found");
-        }
-    }
-
     _pCtMainWin->apply_syntax_highlighting(code_buffer, syntax_highlighting, false/*forceReApply*/);
     code_buffer->ensure_highlight(curr_iter, end_iter);
 
@@ -539,38 +530,209 @@ void CtExport2Html::_html_get_from_treestore_node(CtTreeIter node_iter,
     int start_offset = sel_start == -1 ? 0 : sel_start;
     for (auto widget : out_widgets) {
         int end_offset = widget->getOffset();
-        out_slots.push_back(_html_process_slot(start_offset, end_offset, curr_buffer));
+        out_slots.push_back(html_process_slot(_pCtConfig, _pCtMainWin, start_offset, end_offset, curr_buffer));
         start_offset = end_offset;
     }
-    if (sel_end == -1)
-        out_slots.push_back(_html_process_slot(start_offset, -1, curr_buffer));
-    else
-        out_slots.push_back(_html_process_slot(start_offset, sel_end, curr_buffer));
+    if (sel_end == -1) {
+        out_slots.push_back(html_process_slot(_pCtConfig, _pCtMainWin, start_offset, -1, curr_buffer));
+    }
+    else {
+        out_slots.push_back(html_process_slot(_pCtConfig, _pCtMainWin, start_offset, sel_end, curr_buffer));
+    }
+}
+
+/*static*/int CtExport2Html::_html_process_list_info_change(Glib::ustring& curr_html_text,
+                                                            std::list<CtListType>& nested_list_types,
+                                                            CtListInfo* pListInfoFrom,
+                                                            const CtListInfo* pListInfoTo)
+{
+    int ret_forward_start{0};
+    if (*pListInfoFrom == *pListInfoTo) {
+        return ret_forward_start;
+    }
+    //spdlog::debug("list_info t={} s={} l={} c={} n={}", static_cast<int>(pListInfoTo->type),
+    //    pListInfoTo->startoffs, pListInfoTo->level, pListInfoTo->count_nl, pListInfoTo->num_seq);
+    auto f_increase_level_ol = [&](){
+        curr_html_text += (CtConst::TAG_OL_START + CtConst::TAG_LI_START);
+        nested_list_types.push_back(pListInfoTo->type);
+        ret_forward_start = 3*pListInfoTo->level + CtList::get_leading_chars_num(pListInfoTo->type, pListInfoTo->num_seq);
+    };
+    auto f_increase_level_ul = [&](){
+        curr_html_text += (CtConst::TAG_UL_START + CtConst::TAG_LI_START);
+        nested_list_types.push_back(pListInfoTo->type);
+        if (CtListType::Bullet == pListInfoTo->type) {
+            // we don't want to remove the TODO status character, only if bullet
+            ret_forward_start = 3*pListInfoTo->level + CtList::get_leading_chars_num(pListInfoTo->type, pListInfoTo->num_seq);
+        }
+    };
+    auto f_new_li = [&](){
+        curr_html_text += (CtConst::TAG_LI_END + CtConst::TAG_LI_START);
+        ret_forward_start = 3*pListInfoTo->level + CtList::get_leading_chars_num(pListInfoTo->type, pListInfoTo->num_seq);
+    };
+    auto f_same_li_new_line = [&](){
+        ret_forward_start = 3*(pListInfoTo->level + 1) - 1;
+    };
+    auto f_decrease_level_to_current = [&](){
+        int delta_levels = pListInfoFrom->level - pListInfoTo->level;
+        while (delta_levels-- > 0 and nested_list_types.size() > 0) {
+            const CtListType leaf_type = nested_list_types.back();
+            nested_list_types.pop_back();
+            curr_html_text += (CtConst::TAG_LI_END +
+                (CtListType::Number == leaf_type ? CtConst::TAG_OL_END : CtConst::TAG_UL_END));
+        }
+    };
+    if (/*FROM*/CtListType::None == pListInfoFrom->type) {
+        if (/*TO*/CtListType::None == pListInfoTo->type) {
+            spdlog::debug("?? unexp None to None");
+        }
+        else if (/*TO*/CtListType::Number == pListInfoTo->type) {
+            f_increase_level_ol();
+        }
+        else {
+            // TO CtListType::Bullet, CtListType::Todo
+            f_increase_level_ul();
+        }
+    }
+    else if (/*FROM*/CtListType::Number == pListInfoFrom->type) {
+        if (/*TO*/CtListType::None == pListInfoTo->type) {
+            f_decrease_level_to_current();
+        }
+        else if (/*TO*/CtListType::Number == pListInfoTo->type) {
+            if (pListInfoTo->level == pListInfoFrom->level) {
+                if (pListInfoTo->startoffs == pListInfoFrom->startoffs and
+                    pListInfoTo->count_nl > pListInfoFrom->count_nl)
+                {
+                    // continuation of the list item on a new line
+                    f_same_li_new_line();
+                }
+                else {
+                    // a new list item
+                    f_new_li();
+                }
+            }
+            else if (pListInfoTo->level > pListInfoFrom->level) {
+                for (int i = 0; i < (pListInfoTo->level - pListInfoFrom->level); ++i) {
+                    f_increase_level_ol();
+                }
+            }
+            else {
+                f_decrease_level_to_current();
+                f_new_li();
+            }
+        }
+        else {
+            // TO != TYPE CtListType::Bullet, CtListType::Todo
+            if (pListInfoTo->level == pListInfoFrom->level) {
+                spdlog::debug("?? unexp same level != type");
+                f_new_li();
+            }
+            else if (pListInfoTo->level > pListInfoFrom->level) {
+                for (int i = 0; i < (pListInfoTo->level - pListInfoFrom->level); ++i) {
+                    f_increase_level_ul();
+                }
+            }
+            else {
+                f_decrease_level_to_current();
+                f_new_li();
+            }
+        }
+    }
+    else {
+        // FROM CtListType::Bullet, CtListType::Todo
+        if (/*TO*/CtListType::None == pListInfoTo->type) {
+            f_decrease_level_to_current();
+        }
+        else if (/*TO != type*/CtListType::Number == pListInfoTo->type) {
+            if (pListInfoTo->level == pListInfoFrom->level) {
+                spdlog::debug("?? unexp same level != type");
+                f_new_li();
+            }
+            else if (pListInfoTo->level > pListInfoFrom->level) {
+                for (int i = 0; i < (pListInfoTo->level - pListInfoFrom->level); ++i) {
+                    f_increase_level_ol();
+                }
+            }
+            else {
+                f_decrease_level_to_current();
+                f_new_li();
+            }
+        }
+        else {
+            // TO CtListType::Bullet, CtListType::Todo
+            if (pListInfoTo->level == pListInfoFrom->level) {
+                if (pListInfoTo->startoffs == pListInfoFrom->startoffs and
+                    pListInfoTo->count_nl > pListInfoFrom->count_nl)
+                {
+                    // continuation of the list item on a new line
+                    f_same_li_new_line();
+                }
+                else {
+                    // a new list item
+                    f_new_li();
+                }
+            }
+            else if (pListInfoTo->level > pListInfoFrom->level) {
+                for (int i = 0; i < (pListInfoTo->level - pListInfoFrom->level); ++i) {
+                    f_increase_level_ul();
+                }
+            }
+            else {
+                f_decrease_level_to_current();
+                f_new_li();
+            }
+        }
+    }
+    *pListInfoFrom = *pListInfoTo;
+    return ret_forward_start;
 }
 
 // Process a Single HTML Slot
-Glib::ustring CtExport2Html::_html_process_slot(int start_offset, int end_offset, Glib::RefPtr<Gtk::TextBuffer> curr_buffer)
+/*static*/Glib::ustring CtExport2Html::html_process_slot(const CtConfig* const pCtConfig,
+                                                         CtMainWin* const pCtMainWin, // the unit tests may pass nullptr here!
+                                                         int start_offset,
+                                                         int end_offset,
+                                                         Glib::RefPtr<Gtk::TextBuffer> curr_buffer)
 {
-    Glib::ustring curr_html_text = "";
+    Glib::ustring curr_html_text;
+    CtListInfo curr_list_info;
+    std::list<CtListType> nested_list_types;
     CtTextIterUtil::SerializeFunc f_html_serialise = [&](Gtk::TextIter& start_iter,
                                                          Gtk::TextIter& curr_iter,
-                                                         CtCurrAttributesMap& curr_attributes)
+                                                         CtCurrAttributesMap& curr_attributes,
+                                                         CtListInfo* pCurrListInfo)
     {
-        curr_html_text += _html_text_serialize(start_iter, curr_iter, curr_attributes);
+        Glib::ustring list_html_tags;
+        const int forward_start = _html_process_list_info_change(list_html_tags, nested_list_types, &curr_list_info, pCurrListInfo);
+        //spdlog::debug("fw={} +'{}'", forward_start, list_html_tags.raw());
+        if (list_html_tags.size() > 0) curr_html_text += list_html_tags;
+        if (forward_start > 0) {
+            while ('\n' == start_iter.get_char()) {
+                if (not start_iter.forward_char()) break;
+            }
+            start_iter.forward_chars(forward_start);
+        }
+        const Glib::ustring html_slot = _html_text_serialize(pCtMainWin, start_iter, curr_iter, curr_attributes);
+        //spdlog::debug("slot({})='{}'", html_slot.size(), html_slot.raw());
+        curr_html_text += html_slot;
     };
-    CtTextIterUtil::generic_process_slot(start_offset, end_offset, curr_buffer, f_html_serialise);
+    CtTextIterUtil::generic_process_slot(pCtConfig, start_offset, end_offset, curr_buffer, f_html_serialise, true/*list_info*/);
 
     for (auto header : {CtConst::TAG_PROP_VAL_H1, CtConst::TAG_PROP_VAL_H2, CtConst::TAG_PROP_VAL_H3,
-                        CtConst::TAG_PROP_VAL_H4, CtConst::TAG_PROP_VAL_H5, CtConst::TAG_PROP_VAL_H6}) {
+                        CtConst::TAG_PROP_VAL_H4, CtConst::TAG_PROP_VAL_H5, CtConst::TAG_PROP_VAL_H6})
+    {
         curr_html_text = str::replace(curr_html_text, ("</" + Glib::ustring{header} + "><" + Glib::ustring{header} + " >").c_str(), "");
     }
+    CtListInfo list_info_none;
+    (void)_html_process_list_info_change(curr_html_text, nested_list_types, &curr_list_info, &list_info_none);
+
     return curr_html_text;
 }
 
 // Adds a slice to the HTML Text
-Glib::ustring CtExport2Html::_html_text_serialize(Gtk::TextIter start_iter,
-                                                  Gtk::TextIter end_iter,
-                                                  const CtCurrAttributesMap& curr_attributes)
+/*static*/Glib::ustring CtExport2Html::_html_text_serialize(CtMainWin* const pCtMainWin, // the unit tests may pass nullptr here!
+                                                            Gtk::TextIter start_iter,
+                                                            Gtk::TextIter end_iter,
+                                                            const CtCurrAttributesMap& curr_attributes)
 {
     Glib::ustring html_attrs;
     bool superscript_active{false};
@@ -658,7 +820,9 @@ Glib::ustring CtExport2Html::_html_text_serialize(Gtk::TextIter start_iter,
         }
         else if (tag_property == CtConst::TAG_LINK) {
             // <a href="http://www.example.com/">link-text goes here</a>
-            href = _get_href_from_link_prop_val(property_value);
+            if (pCtMainWin) {
+                href = _get_href_from_link_prop_val(pCtMainWin, property_value);
+            }
             continue;
         }
         html_attrs += Glib::ustring{tag_property.data()} + ":" + property_value + ";";
@@ -700,27 +864,30 @@ Glib::ustring CtExport2Html::_html_text_serialize(Gtk::TextIter start_iter,
     return html_text;
 }
 
-std::string CtExport2Html::_get_href_from_link_prop_val(Glib::ustring link_prop_val)
+/*static*/std::string CtExport2Html::_get_href_from_link_prop_val(CtMainWin* const pCtMainWin, Glib::ustring link_prop_val)
 {
     CtLinkEntry link_entry = CtMiscUtil::get_link_entry(link_prop_val);
-    if (link_entry.type == "")
+    if (link_entry.type.empty()) {
         return "";
+    }
 
-    std::string href = "";
-    if (link_entry.type == CtConst::LINK_TYPE_WEBS)
+    std::string href;
+    if (link_entry.type == CtConst::LINK_TYPE_WEBS) {
         href = link_entry.webs;
-    else if (link_entry.type == CtConst::LINK_TYPE_FILE)
-        href = "file://" + link_process_filepath(link_entry.file, _pCtMainWin->get_ct_storage()->get_file_path().parent_path().string(), true/*forHtml*/);
-    else if (link_entry.type == CtConst::LINK_TYPE_FOLD)
-        href = "file://" + link_process_folderpath(link_entry.fold, _pCtMainWin->get_ct_storage()->get_file_path().parent_path().string(), true/*forHtml*/);
-    else if (link_entry.type == CtConst::LINK_TYPE_NODE)
-    {
-        CtTreeIter node = _pCtMainWin->get_tree_store().get_node_from_node_id(link_entry.node_id);
-        if (node)
-        {
+    }
+    else if (link_entry.type == CtConst::LINK_TYPE_FILE) {
+        href = "file://" + link_process_filepath(link_entry.file, pCtMainWin->get_ct_storage()->get_file_path().parent_path().string(), true/*forHtml*/);
+    }
+    else if (link_entry.type == CtConst::LINK_TYPE_FOLD) {
+        href = "file://" + link_process_folderpath(link_entry.fold, pCtMainWin->get_ct_storage()->get_file_path().parent_path().string(), true/*forHtml*/);
+    }
+    else if (link_entry.type == CtConst::LINK_TYPE_NODE) {
+        CtTreeIter node = pCtMainWin->get_tree_store().get_node_from_node_id(link_entry.node_id);
+        if (node) {
             href = _get_html_filename(node);
-            if (!link_entry.anch.empty())
+            if (not link_entry.anch.empty()) {
                 href += "#" + link_entry.anch;
+            }
         }
     }
     return href;
@@ -753,61 +920,9 @@ Glib::ustring CtExport2Html::_get_object_alignment_string(Glib::ustring alignmen
 }
 
 // Get the HTML page filename given the tree iter
-Glib::ustring CtExport2Html::_get_html_filename(CtTreeIter tree_iter)
+/*static*/Glib::ustring CtExport2Html::_get_html_filename(CtTreeIter tree_iter)
 {
     Glib::ustring name = CtMiscUtil::get_node_hierarchical_name(tree_iter, "--"/*separator*/,
         true/*for_filename*/, true/*root_to_leaf*/, true/*trail_node_id*/, ".html"/*trailer*/);
     return str::replace(name, "#", "~");
 }
-
-std::unique_ptr<CtProcess> pandoc_process() {
-    auto p = std::make_unique<CtProcess>("pandoc");
-    p->append_arg("-t");
-    p->append_arg("html");
-    return p;
-}
-
-namespace CtPandoc {
-
-// Checks if the specified file is in the PATH environment variable
-bool in_path(const std::string& file)
-{
-    g_autofree gchar* prog_name = g_find_program_in_path(file.c_str());
-    return prog_name != nullptr;
-}
-
-
-bool has_pandoc()
-{
-    return in_path("pandoc");
-}
-
-
-void to_html(std::istream& input, std::ostream& output, std::string from_format)
-{
-    auto process = pandoc_process();
-    process->append_arg("--from");
-    process->append_arg(std::move(from_format));
-    try {
-        process->input(&input);
-        process->run(output);
-    } catch(std::exception& e) {
-        spdlog::error("Exception in to_html: {}", e.what());
-        throw;
-    }
-}
-
-
-void to_html(const fs::path& file, std::ostream& output) {
-    auto process = pandoc_process();
-    process->append_arg(file.string());
-
-    try {
-        process->run(output);
-    } catch(std::exception& e) {
-        spdlog::error("Exception in to_html with filepath: {}; message: {}", file, e.what());
-        throw;
-    }
-}
-
-}
