@@ -1,7 +1,7 @@
 /*
  * ct_export2pdf.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -170,10 +170,13 @@ void CtExport2Pango::_pango_process_slot(int start_offset,
 {
     CtTextIterUtil::SerializeFunc f_pango_serialize = [&](Gtk::TextIter& start_iter,
                                                           Gtk::TextIter& end_iter,
-                                                          CtCurrAttributesMap& curr_attributes) {
+                                                          CtCurrAttributesMap& curr_attributes,
+                                                          CtListInfo*/*pCurrListInfo*/)
+    {
         _pango_text_serialize(start_iter, end_iter, curr_attributes, out_slots);
     };
-    CtTextIterUtil::generic_process_slot(start_offset,
+    CtTextIterUtil::generic_process_slot(_pCtConfig,
+                                         start_offset,
                                          end_offset,
                                          curr_buffer,
                                          f_pango_serialize);
@@ -555,7 +558,7 @@ void CtPrint::_on_begin_print_text(const Glib::RefPtr<Gtk::PrintContext>& contex
             else if (auto codebox = dynamic_cast<const CtCodebox*>(pango_widget->widget)) {
                 _process_pango_codebox(print_data, codebox, pango_widget);
             }
-            else if (auto table = dynamic_cast<const CtTable*>(pango_widget->widget)) {
+            else if (auto table = dynamic_cast<const CtTableCommon*>(pango_widget->widget)) {
                 _process_pango_table(print_data, table, pango_widget);
             }
         }
@@ -563,7 +566,7 @@ void CtPrint::_on_begin_print_text(const Glib::RefPtr<Gtk::PrintContext>& contex
 
     print_data->operation->set_n_pages(print_data->pages.size());
     if (any_image_resized) {
-        print_data->warning = Glib::ustring(_("Warning: One or More Images Were Reduced to Enter the Page")) + " ("
+        print_data->warning = Glib::ustring(_("Warning: One or More Images Were Reduced to Enter the Page!")) + " ("
                                        + std::to_string(static_cast<int>(_page_width))+ "x" + std::to_string(static_cast<int>(_page_height)) + ")";
     }
 }
@@ -590,10 +593,14 @@ void CtPrint::_on_draw_page_text(const Glib::RefPtr<Gtk::PrintContext>& context,
 
     // draw page number
     cairo_context->set_source_rgb(0.5, 0.5, 0.5);
-    cairo_context->set_font_size(12);
     Glib::ustring page_num_str = std::to_string(page_nr+1) + "/" + std::to_string(operation->property_n_pages());
-    cairo_context->move_to(_page_width/2., _page_height+17);
-    cairo_context->show_text(page_num_str);
+    Glib::RefPtr<Pango::Layout> layout = context->create_pango_layout();
+    layout->set_font_description(_rich_font);
+    layout->set_markup(page_num_str);
+    auto layout_line = layout->get_line(0);
+    auto size = _get_width_height_from_layout_line(layout_line);
+    cairo_context->move_to(_page_width/2. - size.width/2, _page_height+17);
+    layout_line->show_in_cairo_context(cairo_context);
 
     //cairo_context->set_source_rgba(0.3, 0, 0, 0.3);
     //cairo_context->rectangle(0, 0, _page_width, _page_height);
@@ -694,7 +701,8 @@ void CtPrint::_process_pango_text(CtPrintData* print_data, CtPangoText* text_slo
 
     Glib::RefPtr<Pango::Layout> layout = context->create_pango_layout();
     layout->set_font_description(*font);
-    layout->set_width(int((_page_width - text_slot->indent) * Pango::SCALE));
+    const int max_layout_line_width = _page_width - text_slot->indent;
+    layout->set_width(max_layout_line_width * Pango::SCALE);
     layout->set_wrap(Pango::WRAP_WORD_CHAR);
     // the next line fixes the link issue, allowing to start paragraphs from where a link ends
     // don't apply paragraph indent because set_indent will work only for the first line
@@ -708,6 +716,9 @@ void CtPrint::_process_pango_text(CtPrintData* print_data, CtPangoText* text_slo
     for (int i = 0; i < layout_count; ++i) {
         auto layout_line = layout->get_line(i);
         auto size = _get_width_height_from_layout_line(layout_line);
+        if (size.width > max_layout_line_width) {
+            size.width = max_layout_line_width;
+        }
 
         if (not pages.last_line().test_element_height(size.height, _page_height)) {
             pages.line_on_new_page();
@@ -716,7 +727,7 @@ void CtPrint::_process_pango_text(CtPrintData* print_data, CtPangoText* text_slo
         if (-1 == pages.last_line().cur_x) {
             // initialise x for a new line (or after change of RTL in line)
             if (PANGO_DIRECTION_RTL == text_slot->pango_dir) {
-                pages.last_line().cur_x = _page_width - text_slot->indent;
+                pages.last_line().cur_x = max_layout_line_width;
             }
             else {
                 pages.last_line().cur_x = text_slot->indent;
@@ -1039,7 +1050,9 @@ void CtPrint::_codebox_split_content(const CtCodebox* codebox,
     }
 }
 
-void CtPrint::_process_pango_table(CtPrintData *print_data, const CtTable* table, const CtPangoWidget* pango_widget)
+void CtPrint::_process_pango_table(CtPrintData *print_data,
+                                   const CtTableCommon* table,
+                                   const CtPangoWidget* pango_widget)
 {
     auto context = print_data->context;
     CtPrintPages& pages = print_data->pages;
@@ -1085,7 +1098,7 @@ void CtPrint::_process_pango_table(CtPrintData *print_data, const CtTable* table
 
         if (0 == i) {
             double table_width{0.0};
-            for (size_t col = 0; col < table->get_table_matrix()[0].size(); ++col) {
+            for (size_t col = 0; col < table->get_num_columns(); ++col) {
                 table_width += table->get_col_width(col) * _page_dpi_scale;
             }
             if (table_width > available_width and available_width < (_page_width - pango_widget->indent)) {
@@ -1143,20 +1156,25 @@ void CtPrint::_process_pango_table(CtPrintData *print_data, const CtTable* table
     }
 }
 
-CtPageTable::TableLayouts CtPrint::_table_get_layouts(const CtTable* table, const int first_row, const int last_row, const Glib::RefPtr<Gtk::PrintContext>& context)
+CtPageTable::TableLayouts CtPrint::_table_get_layouts(const CtTableCommon* table,
+                                                      const int first_row,
+                                                      const int last_row,
+                                                      const Glib::RefPtr<Gtk::PrintContext>& context)
 {
+    std::vector<std::vector<Glib::ustring>> rows;
+    table->write_strings_matrix(rows);
     CtPageTable::TableLayouts table_layouts;
-    for (size_t row = 0; row < table->get_table_matrix().size(); ++row) {
-        if (first_row != -1 && row > 0 && (int)row < first_row) continue; // skip row out of range except header
-        if (last_row != -1 && (int)row > last_row) break;
+    for (size_t r = 0u; r < rows.size(); ++r) {
+        if (first_row != -1 && r > 0 && (int)r < first_row) continue; // skip row out of range except header
+        if (last_row != -1 && (int)r > last_row) break;
 
         std::vector<Glib::RefPtr<Pango::Layout>> layouts;
-        for (size_t col = 0; col < table->get_table_matrix()[0].size(); ++col) {
-            Glib::ustring text = str::xml_escape(table->get_table_matrix()[row][col]->get_text_content());
-            if (row == 0) text = "<b>" + text + "</b>";
+        for (size_t c = 0u; c < rows.at(r).size(); ++c) {
+            Glib::ustring text = str::xml_escape(rows.at(r).at(c));
+            if (r == 0) text = "<b>" + text + "</b>";
             Glib::RefPtr<Pango::Layout> cell_layout = context->create_pango_layout();
             cell_layout->set_font_description(_rich_font);
-            cell_layout->set_width(int((table->get_col_width(col) * _page_dpi_scale) * Pango::SCALE));
+            cell_layout->set_width(int((table->get_col_width(c) * _page_dpi_scale) * Pango::SCALE));
             cell_layout->set_wrap(Pango::WRAP_WORD_CHAR);
             cell_layout->set_markup(text);
             layouts.push_back(cell_layout);
@@ -1174,17 +1192,17 @@ void CtPrint::_table_get_grid(const CtPageTable::TableLayouts& table_layouts,
 {
     rows_h = std::vector<double>(table_layouts.size(), 0);
     cols_w = std::vector<double>(table_layouts[0].size(), 0);
-    for (size_t row = 0; row < table_layouts.size(); ++row) {
-        for (size_t col = 0; col < table_layouts[0].size(); ++col) {
-            auto cell_layout = table_layouts[row][col];
+    for (size_t r = 0; r < table_layouts.size(); ++r) {
+        for (size_t c = 0; c < table_layouts[0].size(); ++c) {
+            auto cell_layout = table_layouts[r][c];
             double cell_height = 0;
             for (int layout_line_idx = 0; layout_line_idx < cell_layout->get_line_count(); ++layout_line_idx) {
                 auto line_size = _get_width_height_from_layout_line(cell_layout->get_line(layout_line_idx));
                 cell_height += line_size.height;
-                if (line_size.width > cols_w[col]) cols_w[col] = line_size.width;
+                if (line_size.width > cols_w[c]) cols_w[c] = line_size.width;
             }
-            if (col_widths.at(col) > cols_w[col]) cols_w[col] = col_widths.at(col);
-            if (cell_height > rows_h[row]) rows_h[row] = cell_height;
+            if (col_widths.at(c) > cols_w[c]) cols_w[c] = col_widths.at(c);
+            if (cell_height > rows_h[r]) rows_h[r] = cell_height;
         }
     }
 }
@@ -1197,16 +1215,18 @@ double CtPrint::_table_get_width_height(std::vector<double>& data)
     return acc;
 }
 
-int CtPrint::_table_split_content(const CtTable* table, const int start_row, const int check_height, const Glib::RefPtr<Gtk::PrintContext>& context)
+int CtPrint::_table_split_content(const CtTableCommon* table,
+                                  const int start_row,
+                                  const int check_height,
+                                  const Glib::RefPtr<Gtk::PrintContext>& context)
 {
     int last_row = start_row;
-    for (; last_row < (int)table->get_table_matrix().size(); ++last_row) {
+    for (; last_row < (int)table->get_num_rows(); ++last_row) {
         std::vector<double> rows_h, cols_w;
         auto table_layouts = _table_get_layouts(table, start_row, last_row, context);
         _table_get_grid(table_layouts, table->get_col_widths(), rows_h, cols_w);
         double table_height = _table_get_width_height(rows_h);
-        if (table_height > check_height)
-        {
+        if (table_height > check_height) {
             if (start_row == last_row) // not enouth place for 1 row + a header, add a new page
                 return -1;
             return last_row -1;
