@@ -46,6 +46,10 @@ static fs::path _mingw64Dir;
 // replacement of Glib::canonicalize_filename for Glibmm < 2.64
 std::string legacy_canonicalize_filename(const std::string& filename, const std::string& relative_to/*= ""*/)
 {
+    // manage case where filename is an empty string -- just send it back unchanged
+    if (filename == "")
+      return "";
+
     std::string retFilepath;
     GFile* pGFile{nullptr};
     if (not Glib::path_is_absolute(filename) and not relative_to.empty()) {
@@ -571,18 +575,17 @@ path relative(const path& p, const path& base)
     return p;
 }
 
-path path::extension() const
+std::string path::extension() const
 {
     std::string name = filename().string();
     auto last_pos = name.find_last_of('.');
     if (last_pos == std::string::npos || last_pos == name.size() - 1 || last_pos == 0) {
-        return path("");
-    } else {
-        return path(name.begin() + last_pos, name.end());
+        return "";
     }
+    return name.substr(last_pos);
 }
 
-path path::stem() const
+std::string path::stem() const
 {
     if (empty()) return "";
     std::string name = filename().string();
@@ -606,4 +609,13 @@ std::string path::string_unix() const
     return str::replace(_path, CtConst::CHAR_BSLASH, CtConst::CHAR_SLASH);
 }
 
+#if !defined(_WIN32)
+void path::_replaceStartingTilde()
+{
+    if (str::startswith(_path, "~/")) {
+        _path = Glib::get_home_dir() + _path.substr(1);
+    }
+}
+#endif // !_WIN32
+
 } // namespace fs
