@@ -1,7 +1,7 @@
 ﻿/*
  * ct_main_win.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -71,6 +71,9 @@ CtMainWin::CtMainWin(bool                            no_gui,
         _hPaned.pack2(_vboxText, Gtk::EXPAND);
     }
     _hPaned.property_wide_handle() = true;
+    _vPaned.pack1(_hPaned, Gtk::EXPAND);
+    _vPaned.pack2(_hBoxVte, Gtk::FILL);
+    _vPaned.property_wide_handle() = true;
 
     _pMenuBar = _uCtMenu->build_menubar();
     _pScrolledWindowMenuBar = Gtk::manage(new Gtk::ScrolledWindow{});
@@ -108,7 +111,7 @@ CtMainWin::CtMainWin(bool                            no_gui,
     for (auto pToolbar : _pToolbars) {
         _vboxMain.pack_start(*pToolbar, false, false);
     }
-    _vboxMain.pack_start(_hPaned);
+    _vboxMain.pack_start(_vPaned);
     _vboxMain.pack_start(_init_status_bar(), false, false);
     _vboxMain.show_all();
     add(_vboxMain);
@@ -174,16 +177,20 @@ CtMainWin::CtMainWin(bool                            no_gui,
         else {
             present();
         }
-        while (gtk_events_pending()) gtk_main_iteration();
-        _hPaned.property_position() = _pCtConfig->hpanedPos; // must be after present() + process events pending (#1534, #1918)
-        _ctTextview.signal_size_allocate().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_size_allocate));
-        signal_configure_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_window_configure_event), false);
+        Glib::signal_idle().connect_once([this](){
+            _hPaned.property_position() = _pCtConfig->hpanedPos; // must be after present() + process events pending (#1534, #1918, #2126)
+            _vPaned.property_position() = _pCtConfig->vpanedPos;
+            _ctTextview.signal_size_allocate().connect(sigc::mem_fun(*this, &CtMainWin::_on_textview_size_allocate));
+            signal_configure_event().connect(sigc::mem_fun(*this, &CtMainWin::_on_window_configure_event), false);
 
-        // show status icon if needed
-        if (_pCtConfig->systrayOn) {
-            get_status_icon()->set_visible(true);
-        }
+            // show status icon if needed
+            if (_pCtConfig->systrayOn) {
+                get_status_icon()->set_visible(true);
+            }
+        });
     }
+
+    dispatcherErrorMsg.connect(sigc::mem_fun(*this, &CtMainWin::_on_dispatcher_error_msg));
 }
 
 CtMainWin::~CtMainWin()
@@ -193,6 +200,12 @@ CtMainWin::~CtMainWin()
     //std::cout << "~CtMainWin" << std::endl;
 }
 
+void CtMainWin::_on_dispatcher_error_msg()
+{
+    const std::string eroor_msg = errorsDEQueue.pop_front();
+    CtDialogs::error_dialog(eroor_msg, *this);
+}
+
 bool CtMainWin::start_on_systray_is_active() const
 {
     return _pCtConfig->systrayOn and _pCtConfig->startOnSystray;
@@ -289,6 +302,7 @@ void CtMainWin::config_apply()
     }
 
     show_hide_tree_view(_pCtConfig->treeVisible);
+    (void)show_hide_vte_cmd_passed_as_first_in_session(_pCtConfig->vteVisible, nullptr/*first_cmd_passed*/);
     show_hide_menubar(_pCtConfig->menubarVisible);
     show_hide_win_header(_pCtConfig->showNodeNameHeader);
     _ctWinHeader.lockIcon.hide();
@@ -352,6 +366,7 @@ void CtMainWin::update_theme()
     css_str += ".ct-codebox.ct-view-plain-text" + plFont;
     css_str += ".ct-codebox.ct-view-code" + codeFont;
     css_str += ".ct-tree-panel" + treeFont;
+    css_str += ".ct-table-light" + rtFont;
     css_str += " ";
     css_str += ".ct-tree-panel { color: " + _pCtConfig->ttDefFg + "; background-color: " + _pCtConfig->ttDefBg + "; } ";
     css_str += ".ct-tree-panel:selected { color: " + _pCtConfig->ttSelFg + "; background: " + _pCtConfig->ttSelBg + "; } ";
@@ -363,6 +378,7 @@ void CtMainWin::update_theme()
     css_str += ".ct-table grid { background: #cccccc; border-style:solid; border-width: 1px; border-color: gray; } ";
     css_str += "toolbar { padding: 2px 2px 2px 2px; } ";
     css_str += "toolbar button { padding: 0px; } ";
+    css_str += "textview border { background-color: transparent; } "; // Loss of transparency with PNGs (#1402, #2132)
     //printf("css_str_len=%zu\n", css_str.size());
 
     if (_css_provider_theme) {
@@ -717,13 +733,13 @@ void CtMainWin::update_selected_node_statusbar_info()
 void CtMainWin::tree_node_paste_from_other_window(CtMainWin* pWinToCopyFrom, gint64 nodeIdToCopyFrom)
 {
     if (not pWinToCopyFrom) {
-        CtDialogs::warning_dialog(_("No Previous Node Copy Was Performed During This Session or the Source Tree is No Longer Available"), *this);
+        CtDialogs::warning_dialog(_("No Previous Node Copy Was Performed During This Session or the Source Tree is No Longer Available."), *this);
         return;
     }
     CtTreeStore& other_ct_tree_store = pWinToCopyFrom->get_tree_store();
     CtTreeIter   other_ct_tree_iter = other_ct_tree_store.get_node_from_node_id(nodeIdToCopyFrom);
     if (not other_ct_tree_iter) {
-        CtDialogs::warning_dialog(_("The Source Tree Node is No Longer Available"), *this);
+        CtDialogs::warning_dialog(_("The Source Tree Node is No Longer Available."), *this);
         return;
     }
     _uCtActions->node_subnodes_paste2(other_ct_tree_iter, pWinToCopyFrom);
