@@ -1,7 +1,7 @@
 /*
  * ct_main_win_file.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -49,6 +49,7 @@ void CtMainWin::window_title_update(std::optional<bool> saveNeeded)
         title += Glib::ustring{" - "} + _("Show Menubar:") + " " + (pAction ? pAction->get_shortcut(_pCtConfig) : std::string{"?"});
     }
     if (_pHeaderBar) {
+        _pHeaderBar->set_title(title); // so it is shown on the taskbar button
         auto pCustomTitle = dynamic_cast<Gtk::Label*>(_pHeaderBar->get_custom_title());
         if (not pCustomTitle) {
             pCustomTitle = Gtk::manage(new Gtk::Label{});
@@ -157,17 +158,17 @@ bool CtMainWin::file_open(const fs::path& filepath, const std::string& node_to_f
     reset(); // cannot reset after load_from because load_from fill tree store
 
     Glib::ustring error;
-    auto new_storage = CtStorageControl::load_from(this, filepath, error, password);
-    if (!new_storage) {
+    CtStorageControl* new_storage = CtStorageControl::load_from(this, filepath, error, password);
+    if (not new_storage) {
         if (not error.empty()) {
             CtDialogs::error_dialog(str::format(_("Error Parsing the CherryTree File:\n\"%s\""), str::xml_escape(error)), *this);
         }
 
         // trying to recover prevous document
-        if (!prev_path.empty()) {
+        if (not prev_path.empty()) {
             file_open(prev_path, ""/*node*/, ""/*anchor*/); // it won't be in loop because storage is empty
         }
-        return false;                 // show the given document is not loaded
+        return false; // show the given document is not loaded
     }
 
     _uCtStorage.reset(new_storage);
@@ -205,7 +206,7 @@ bool CtMainWin::file_open(const fs::path& filepath, const std::string& node_to_f
             }
         }
         else {
-            CtDialogs::warning_dialog(str::format(_("No node named '%s' found"), str::xml_escape(node_to_focus)), *this);
+            CtDialogs::warning_dialog(str::format(_("No node named '%s' found."), str::xml_escape(node_to_focus)), *this);
         }
     }
 
@@ -340,10 +341,11 @@ void CtMainWin::file_autosave_restart()
     spdlog::debug("autosave is started");
     _autosave_timout_connection = Glib::signal_timeout().connect_seconds([this]() {
         if (get_file_save_needed()) {
-            spdlog::debug("autosave: time to save file");
-            file_save(false);
-        } else {
-            spdlog::debug("autosave: no needs to save file");
+            spdlog::debug("autosave needed");
+            _uCtActions->file_save();
+        }
+        else {
+            spdlog::debug("autosave no need");
         }
         return true;
     }, _pCtConfig->autosaveVal * 60);
@@ -366,7 +368,7 @@ void CtMainWin::mod_time_sentinel_restart()
                 spdlog::debug("mod time was {} now {}", _uCtStorage->get_mod_time(), currModTime);
                 fs::path file_path = _uCtStorage->get_file_path();
                 if (file_open(file_path, ""/*node*/, ""/*anchor*/)) {
-                    _ctStatusBar.update_status(_("The Document was Reloaded After External Update to CT* File"));
+                    _ctStatusBar.update_status(_("The Document was Reloaded After External Update to CT* File."));
                 }
             }
         }
