@@ -1,7 +1,7 @@
 /*
  * ct_menu.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -34,7 +34,10 @@ static xmlpp::Attribute* get_attribute(xmlpp::Node* pNode, char const* name)
 static void on_menu_activate(void* /*pObject*/, CtMenuAction* pAction)
 {
     if (pAction) {
-        pAction->run_action();
+        // this allows the menu to close before the action is executed
+        Glib::signal_idle().connect_once([pAction](){
+            pAction->run_action();
+        });
     }
 }
 
@@ -174,6 +177,7 @@ void CtMenu::build_popup_menu(Gtk::Menu* pMenu, POPUP_MENU_TYPE popupMenuType)
         case CtMenu::POPUP_MENU_TYPE::Latex: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_latex(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::Anchor: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_anchor(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::EmbFile: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_embfile(), nullptr); break;
+        case CtMenu::POPUP_MENU_TYPE::Terminal: _walk_menu_xml(pMenu, _get_popup_menu_ui_str_terminal(), nullptr); break;
         case CtMenu::POPUP_MENU_TYPE::Link: {
             _add_menu_separator(pMenu);
             _add_menu_item(pMenu, find_action("apply_tag_link"));
@@ -201,7 +205,8 @@ void CtMenu::build_popup_menu(Gtk::Menu* pMenu, POPUP_MENU_TYPE popupMenuType)
             _add_menu_item(pMenu, find_action("codebox_increase_height"));
             _add_menu_item(pMenu, find_action("codebox_decrease_height"));
             _add_menu_separator(pMenu);
-            _add_menu_item(pMenu, find_action("exec_code"));
+            _add_menu_item(pMenu, find_action("exec_code_all"));
+            _add_menu_item(pMenu, find_action("exec_code_los"));
             _add_menu_item(pMenu, find_action("strip_trail_spaces"));
             _add_menu_item(pMenu, find_action("repl_tabs_spaces"));
             _add_menu_separator(pMenu);
@@ -282,7 +287,7 @@ void CtMenu::_walk_menu_xml(Gtk::MenuShell* pMenuShell, xmlpp::Node* pNode)
         }
         else if (pNodeIter->get_name() == "menuitem") {
             CtMenuAction* pAction = find_action(get_attribute(pNodeIter, "action")->get_value());
-            _add_menu_item(pMenuShell, pAction);
+            if (pAction) _add_menu_item(pMenuShell, pAction);
         }
         else if (pNodeIter->get_name() == "separator") {
             _add_menu_separator(pMenuShell);
