@@ -1,7 +1,7 @@
 /*
  * ct_menu_ui.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -44,8 +44,26 @@ std::vector<std::string> CtMenu::_get_ui_str_toolbars()
                     str += "<property name='action-name'>win." + pAction->id + "</property>"; // 'win.' is a default action group in Window
                     str += "<property name='icon-name'>" + pAction->image + "</property>";
                     str += "<property name='label'>" + pAction->name + "</property>";
-                    const std::string kb_shortcut = pAction->get_shortcut(_pCtConfig);
-                    const std::string tooltip = kb_shortcut.empty() ? pAction->desc : (pAction->desc + " (" + str::xml_escape(kb_shortcut).c_str() + ")");
+                    std::string kb_shortcut = pAction->get_shortcut(_pCtConfig);
+                    std::string tooltip;
+                    if (kb_shortcut.empty()) {
+                        tooltip = pAction->desc;
+                    }
+                    else {
+                        if (kb_shortcut.find(CtMenu::KB_CONTROL) != std::string::npos) {
+                            kb_shortcut = str::replace(kb_shortcut, CtMenu::KB_CONTROL, "Ctrl+");
+                        }
+                        if (kb_shortcut.find(CtMenu::KB_SHIFT) != std::string::npos) {
+                            kb_shortcut = str::replace(kb_shortcut, CtMenu::KB_SHIFT, "Shift+");
+                        }
+                        if (kb_shortcut.find(CtMenu::KB_ALT) != std::string::npos) {
+                            kb_shortcut = str::replace(kb_shortcut, CtMenu::KB_ALT, "Alt+");
+                        }
+                        if (kb_shortcut.find(CtMenu::KB_META) != std::string::npos) {
+                            kb_shortcut = str::replace(kb_shortcut, CtMenu::KB_META, "Meta+");
+                        }
+                        tooltip = pAction->desc + " (" + str::xml_escape(kb_shortcut).c_str() + ")";
+                    }
                     str += "<property name='tooltip-text'>" + tooltip + "</property>";
                     str += "<property name='visible'>True</property>";
                     str += "<property name='use_underline'>True</property>";
@@ -108,7 +126,6 @@ const char* CtMenu::_get_ui_str_menu()
       <menuitem action='import_tomboy'/>
       <menuitem action='import_treepad'/>
       <menuitem action='import_zim'/>
-      <menuitem action='import_pandoc_file'/>
     </menu>
     <menu action='ExportSubMenu'>
       <menuitem action='export_pdf'/>
@@ -144,12 +161,50 @@ const char* CtMenu::_get_ui_str_menu()
     <menuitem action='copy_plain'/>
     <menuitem action='paste_plain'/>
     <separator/>
-    <menuitem action='cut_row'/>
-    <menuitem action='copy_row'/>
-    <menuitem action='dup_row'/>
-    <menuitem action='mv_up_row'/>
-    <menuitem action='mv_down_row'/>
-    <menuitem action='del_row'/>
+    <menu action='RowSubMenu'>
+      <menuitem action='cut_row'/>
+      <menuitem action='copy_row'/>
+      <menuitem action='dup_row'/>
+      <menuitem action='mv_up_row'/>
+      <menuitem action='mv_down_row'/>
+      <menuitem action='del_row'/>
+    </menu>
+    <separator/>
+    <menu action='TableSubMenu'>
+      <menuitem action='table_cut'/>
+      <menuitem action='table_copy'/>
+      <menuitem action='table_delete'/>
+      <menuitem action='table_column_add'/>
+      <menuitem action='table_column_delete'/>
+      <menuitem action='table_column_left'/>
+      <menuitem action='table_column_right'/>
+      <menuitem action='table_column_increase_width'/>
+      <menuitem action='table_column_decrease_width'/>
+      <menuitem action='table_row_add'/>
+      <menuitem action='table_row_cut'/>
+      <menuitem action='table_row_copy'/>
+      <menuitem action='table_row_paste'/>
+      <menuitem action='table_row_delete'/>
+      <menuitem action='table_row_up'/>
+      <menuitem action='table_row_down'/>
+      <menuitem action='table_rows_sort_descending'/>
+      <menuitem action='table_rows_sort_ascending'/>
+      <menuitem action='table_export'/>
+      <menuitem action='table_edit_properties'/>
+    </menu>
+    <menu action='CodeBoxSubMenu'>
+      <menuitem action='codebox_cut'/>
+      <menuitem action='codebox_copy'/>
+      <menuitem action='codebox_delete'/>
+      <menuitem action='codebox_delete_keeping_text'/>
+      <menuitem action='codebox_increase_width'/>
+      <menuitem action='codebox_decrease_width'/>
+      <menuitem action='codebox_increase_height'/>
+      <menuitem action='codebox_decrease_height'/>
+      <menuitem action='codebox_load_from_file'/>
+      <menuitem action='codebox_save_to_file'/>
+      <menuitem action='codebox_change_properties'/>
+    </menu>
   </menu>
 
   <menu action='InsertMenu'>
@@ -217,7 +272,8 @@ const char* CtMenu::_get_ui_str_menu()
   <menu action='ToolsMenu'>
     <menuitem action='spellcheck_toggle'/>
     <separator/>
-    <menuitem action='exec_code'/>
+    <menuitem action='exec_code_los'/>
+    <menuitem action='exec_code_all'/>
     <menuitem action='strip_trail_spaces'/>
     <menuitem action='repl_tabs_spaces'/>
     <separator/>
@@ -234,7 +290,8 @@ const char* CtMenu::_get_ui_str_menu()
     <menuitem action='tree_dup_node_subnodes'/>
     <menuitem action='tree_copy_node_subnodes'/>
     <menuitem action='tree_paste_node_subnodes'/>
-    <menuitem action='tree_node_date'/>
+    <menuitem action='tree_node_date_root'/>
+    <menuitem action='tree_node_date_sel'/>
     <separator/>
     <menuitem action='tree_node_prop'/>
     <menuitem action='tree_node_toggle_ro'/>
@@ -290,15 +347,17 @@ const char* CtMenu::_get_ui_str_menu()
     <menuitem action='toggle_show_toolbar'/>
     <menuitem action='toggle_show_statusbar'/>
     <menuitem action='toggle_show_node_name_head'/>
+    <menuitem action='toggle_show_vte'/>
     <separator/>
     <menuitem action='menubar_in_titlebar'/>
+    <menuitem action='toggle_fullscreen'/>
+    <menuitem action='toggle_always_on_top'/>
     <separator/>
     <menuitem action='toggle_focus_tree_text'/>
+    <menuitem action='toggle_focus_vte_text'/>
     <separator/>
     <menuitem action='toolbar_icons_size_p'/>
     <menuitem action='toolbar_icons_size_m'/>
-    <separator/>
-    <menuitem action='toggle_fullscreen'/>
   </menu>
 
   <menu action='BookmarksMenu'>
@@ -393,6 +452,8 @@ const char* CtMenu::_get_popup_menu_ui_str_text()
       <menuitem action='handle_todo_list'/>
     </menu>
   </menu>
+  <menuitem action='exec_code_los'/>
+  <menuitem action='exec_code_all'/>
   <menuitem action='strip_trail_spaces'/>
   <separator/>
   <menu action='FindSubMenu'>
@@ -419,7 +480,8 @@ const char* CtMenu::_get_popup_menu_ui_str_code()
   <menuitem action='cut_plain'/>
   <menuitem action='copy_plain'/>
   <separator/>
-  <menuitem action='exec_code'/>
+  <menuitem action='exec_code_los'/>
+  <menuitem action='exec_code_all'/>
   <menuitem action='strip_trail_spaces'/>
   <menuitem action='repl_tabs_spaces'/>
   <menu action='InsertSubMenu'>
@@ -517,3 +579,14 @@ const char* CtMenu::_get_popup_menu_ui_str_embfile()
 </popup>
     )MARKUP";
 }
+
+const char* CtMenu::_get_popup_menu_ui_str_terminal()
+{
+    return R"MARKUP(
+<popup>
+  <menuitem action='term_copy'/>
+  <menuitem action='term_paste'/>
+  <menuitem action='term_reset'/>
+</popup>
+    )MARKUP";
+}
