@@ -1,7 +1,7 @@
 /*
  * ct_misc_utils.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -28,6 +28,7 @@
 #include <cstring>
 #include "ct_const.h"
 #include "ct_logging.h"
+#include "ct_list.h"
 #include <ctime>
 #include <regex>
 #include <glib/gstdio.h> // to get stats
@@ -47,93 +48,70 @@
 #include <shellapi.h>
 #endif // _WIN32
 
-namespace CtCSV {
-
-CtStringTable table_from_csv(std::istream& input)
+CtCSV::CtStringTable CtCSV::table_from_csv(const std::string& filepath)
 {
-    // Disable exceptions
-    auto except_bit_before = input.exceptions();
-    input.exceptions(std::ios::goodbit);
-
     CtStringTable tbl_matrix;
-    std::string line;
 
-    std::array<char, 256> chunk_buff{};
-    input.read(chunk_buff.data(), chunk_buff.size());
-    std::streamsize pos;
     std::vector<std::string> tbl_row;
-    std::ostringstream cell_buff;
     constexpr char cell_tag = '"';
     constexpr char cell_sep = ',';
     constexpr char esc = '\\';
     bool in_string = false;
     bool escape_next = false;
-    while (input || (pos = input.gcount()) != 0) {
-        for (auto ch : chunk_buff) {
 
-            if (ch == '\0') break;
-            if (escape_next) {
-                escape_next = false;
-                cell_buff << ch;
+    std::string input = Glib::file_get_contents(filepath);
+    std::string cell_buff;
+
+    for (auto ch : input) {
+
+        if (ch == '\0') break;
+        if (escape_next) {
+            escape_next = false;
+            cell_buff += ch;
+            continue;
+        }
+        if (ch == esc ) {
+            // `\` escapes anything `"` escapes a quote
+            escape_next = true;
+            continue;
+        }
+        bool is_newline = ch == '\n';
+        if ((ch == cell_sep || is_newline) && !in_string) {
+            // Close the cell
+            tbl_row.emplace_back(cell_buff);
+            cell_buff.clear();
+
+            if (is_newline) {
+                tbl_matrix.emplace_back(tbl_row);
+                tbl_row.clear();
                 continue;
             }
-            if (ch == esc ) {
-                // `\` escapes anything `"` escapes a quote
-                escape_next = true;
-                continue;
-            }
-            bool is_newline = ch == '\n';
-            if ((ch == cell_sep || is_newline) && !in_string) {
-                // Close the cell
-                tbl_row.emplace_back(cell_buff.str());
-                std::ostringstream tmp_buff;
-                cell_buff.swap(tmp_buff);
-
-                if (is_newline) {
-                    tbl_matrix.emplace_back(tbl_row);
-                    tbl_row.clear();
-                    continue;
-                }
-
-            } else if (ch == cell_tag) {
-                in_string = !in_string;
-            } else {
-                cell_buff << ch;
-            }
         }
-
-        if (pos != 0) input.read(chunk_buff.data(), chunk_buff.size());
+        else if (ch == cell_tag) {
+            in_string = !in_string;
+        }
+        else {
+            cell_buff += ch;
+        }
     }
 
-    // Reset exception bit
-    input.exceptions(except_bit_before);
     return tbl_matrix;
 }
 
-std::string escape_to_csv(const std::string& input) {
-    std::ostringstream out;
-    for (const auto ch : input) {
-        if (ch == '"') {
-            out << '\\';
-        }
-        out << ch;
-    }
-    return out.str();
-}
-
-void table_to_csv(const CtStringTable& table, std::ostream& output) {
+std::string CtCSV::table_to_csv(const CtStringTable& table)
+{
+    std::string ret_str;
     for (const auto& row : table) {
         for (const auto& cell : row) {
-            output << fmt::format("\"{}\"", escape_to_csv(cell));
+            ret_str += fmt::format("\"{}\"", str::replace(cell, std::string{"\""}, std::string{"\\\""}));
 
-            if (cell != row.back()) output << ",";
+            if (cell != row.back()) ret_str += ",";
         }
-        output << "\n";
+        ret_str += "\n";
     }
+    return ret_str;
 }
 
-} // namespace CtCSV
-
 std::string CtMiscUtil::get_ct_language()
 {
     std::string retLang{CtConst::LANG_DEFAULT};
@@ -187,33 +165,6 @@ void CtMiscUtil::filepath_extension_fix(const CtDocType ctDocType, const CtDocEn
     }
 }
 
-bool CtMiscUtil::node_siblings_sort_iteration(Glib::RefPtr<Gtk::TreeStore> model, const Gtk::TreeNodeChildren& children,
-                                              std::function<bool(Gtk::TreeIter&, Gtk::TreeIter&)> need_swap)
-{
-    if (children.empty()) return false;
-    auto next_iter = [](Gtk::TreeIter iter) { return ++iter; };
-    auto sort_iteration = [&need_swap, &model, &next_iter](Gtk::TreeIter curr_sibling) -> bool {
-        bool swap_executed = false;
-        Gtk::TreeIter next_sibling = next_iter(curr_sibling);
-        while (next_sibling) {
-            if (need_swap(curr_sibling, next_sibling)) {
-                model->iter_swap(curr_sibling, next_sibling);
-                swap_executed = true;
-            } else {
-                curr_sibling = next_sibling;
-            }
-            next_sibling = next_iter(curr_sibling);
-        }
-        return swap_executed;
-    };
-
-    bool swap_executed = false;
-    while (sort_iteration(children.begin()))
-        swap_executed = true;
-    return swap_executed;
-}
-
-//"""Get the Node Hierarchical Name"""
 std::string CtMiscUtil::get_node_hierarchical_name(const CtTreeIter tree_iter, const char* separator/*="--"*/,
                                                    const bool for_filename/*=true*/, const bool root_to_leaf/*=true*/,
                                                    const bool trail_node_id/*=false*/, const char* trailer/*=""*/)
@@ -447,15 +398,17 @@ bool CtTextIterUtil::rich_text_attributes_update(const Gtk::TextIter& text_iter,
     return anyDelta;
 }
 
-void CtTextIterUtil::generic_process_slot(int start_offset,
-                                          int end_offset,
+void CtTextIterUtil::generic_process_slot(const CtConfig* const pCtConfig,
+                                          const int start_offset,
+                                          const int end_offset,
                                           const Glib::RefPtr<Gtk::TextBuffer>& rTextBuffer,
-                                          SerializeFunc serialize_func)
+                                          SerializeFunc serialize_func,
+                                          const bool list_info/*= false*/)
 {
     CtCurrAttributesMap curr_attributes;
     CtCurrAttributesMap delta_attributes;
     for (const auto& tag_property : CtConst::TAG_PROPERTIES) {
-        curr_attributes[tag_property] = "";
+        curr_attributes[tag_property].clear();
     }
     Gtk::TextIter curr_start_iter = rTextBuffer->get_iter_at_offset(start_offset);
     Gtk::TextIter curr_end_iter = curr_start_iter;
@@ -466,24 +419,48 @@ void CtTextIterUtil::generic_process_slot(int start_offset,
             curr_attributes[currDelta.first] = currDelta.second;
         }
     }
-    while (curr_end_iter.forward_to_tag_toggle(Glib::RefPtr<Gtk::TextTag>{nullptr}))
-    {
+
+    CtListInfo curr_list_info, prev_list_info;
+    bool list_info_changed{false};
+    bool last_was_newline{false};
+    if (not curr_end_iter.backward_char()) {
+        last_was_newline = true;
+    }
+    else {
+        last_was_newline = '\n' == curr_end_iter.get_char();
+        curr_end_iter.forward_char();
+    }
+
+    while (curr_end_iter.forward_char()) {
         if (curr_end_iter.compare(real_end_iter) >= 0) {
             break;
         }
-        if (CtTextIterUtil::rich_text_attributes_update(curr_end_iter, curr_attributes, delta_attributes)) {
-            serialize_func(curr_start_iter, curr_end_iter, curr_attributes);
+
+        if (list_info and last_was_newline) {
+            curr_list_info = CtList{pCtConfig, rTextBuffer}.get_paragraph_list_info(curr_end_iter);
+            if (curr_list_info != prev_list_info) {
+                list_info_changed = true;
+                prev_list_info = curr_list_info;
+            }
+        }
+
+        last_was_newline = '\n' == curr_end_iter.get_char();
+
+        if (CtTextIterUtil::rich_text_attributes_update(curr_end_iter, curr_attributes, delta_attributes) or
+            (list_info and last_was_newline and list_info_changed))
+        {
+            serialize_func(curr_start_iter, curr_end_iter, curr_attributes, &curr_list_info);
 
             for (auto& currDelta : delta_attributes) {
                 curr_attributes[currDelta.first] = currDelta.second;
             }
             curr_start_iter = curr_end_iter;
+            if (list_info_changed) list_info_changed = false;
         }
     }
 
-    if (curr_start_iter.compare(real_end_iter) < 0)
-    {
-        serialize_func(curr_start_iter, real_end_iter, curr_attributes);
+    if (curr_start_iter.compare(real_end_iter) < 0) {
+        serialize_func(curr_start_iter, real_end_iter, curr_attributes, &curr_list_info);
     }
 }
 
@@ -872,7 +849,9 @@ Glib::ustring CtFontUtil::get_font_str(const Glib::ustring& fontFamily, const in
 
 Glib::ustring CtFontUtil::get_font_str(const Pango::FontDescription& fontDesc)
 {
-    return fontDesc.get_family() + " " + std::to_string(get_font_size(fontDesc));
+    Glib::ustring font_family = fontDesc.get_family();
+    auto font_family_splitted = str::split(font_family, ",");
+    return font_family_splitted.back() + " " + std::to_string(get_font_size(fontDesc));
 }
 
 
