@@ -1,7 +1,7 @@
 /*
  * ct_misc_utils.h
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -33,10 +33,14 @@ class CtConfig;
 class CtTreeIter;
 
 namespace CtCSV {
-    using CtStringTable = std::vector<std::vector<std::string>>;
-    CtStringTable table_from_csv(std::istream& input);
-    void table_to_csv(const CtStringTable& table, std::ostream& output);
-}
+
+using CtStringTable = std::vector<std::vector<std::string>>;
+
+CtStringTable table_from_csv(const std::string& filepath);
+
+std::string table_to_csv(const CtStringTable& table);
+
+} // namespace CtCSV
 
 namespace CtMiscUtil {
 
@@ -46,8 +50,45 @@ std::string get_doc_extension(const CtDocType ctDocType, const CtDocEncrypt ctDo
 
 void filepath_extension_fix(const CtDocType ctDocType, const CtDocEncrypt ctDocEncrypt, std::string& filepath);
 
-bool node_siblings_sort_iteration(Glib::RefPtr<Gtk::TreeStore> model, const Gtk::TreeNodeChildren& children,
-                                  std::function<bool(Gtk::TreeIter&, Gtk::TreeIter&)> need_swap);
+template<class TreeOrListStore>
+bool node_siblings_sort(Glib::RefPtr<TreeOrListStore> model,
+                        const Gtk::TreeNodeChildren& children,
+                        std::function<bool(Gtk::TreeIter&, Gtk::TreeIter&)> f_need_swap,
+                        const size_t start_offset = 0u)
+{
+    if (children.size() <= start_offset) {
+        return false;
+    }
+    auto next_iter = [](Gtk::TreeIter iter) { return ++iter; };
+    auto sort_iteration = [&f_need_swap, &model, &next_iter](Gtk::TreeIter curr_sibling, size_t offset)->bool{
+        bool swap_executed{false};
+        while (offset-- > 0) {
+            ++curr_sibling;
+            if (not curr_sibling) {
+                return false;
+            }
+        }
+        Gtk::TreeIter next_sibling{next_iter(curr_sibling)};
+        while (next_sibling) {
+            if (f_need_swap(curr_sibling, next_sibling)) {
+                model->iter_swap(curr_sibling, next_sibling);
+                swap_executed = true;
+            }
+            else {
+                curr_sibling = next_sibling;
+            }
+            next_sibling = next_iter(curr_sibling);
+        }
+        return swap_executed;
+    };
+    bool swap_executed{false};
+    while (sort_iteration(children.begin(), start_offset)) {
+        if (not swap_executed) {
+            swap_executed = true;
+        }
+    }
+    return swap_executed;
+}
 
 std::string get_node_hierarchical_name(const CtTreeIter tree_iter, const char* separator="--",
                                        const bool for_filename=true, const bool root_to_leaf=true,
@@ -119,11 +160,14 @@ bool rich_text_attributes_update(const Gtk::TextIter& text_iter, const CtCurrAtt
 
 using SerializeFunc = std::function<void(Gtk::TextIter& start_iter,
                                          Gtk::TextIter& end_iter,
-                                         CtCurrAttributesMap& curr_attributes)>;
-void generic_process_slot(int start_offset,
-                          int end_offset,
+                                         CtCurrAttributesMap& curr_attributes,
+                                         CtListInfo* pCurrListInfo)>;
+void generic_process_slot(const CtConfig* const pCtConfig,
+                          const int start_offset,
+                          const int end_offset,
                           const Glib::RefPtr<Gtk::TextBuffer>& rTextBuffer,
-                          SerializeFunc serialize_func);
+                          SerializeFunc serialize_func,
+                          const bool list_info = false);
 
 const gchar* get_text_iter_alignment(const Gtk::TextIter& textIter, CtMainWin* pCtMainWin);
 
@@ -334,14 +378,14 @@ std::vector<STRING> split(const STRING& strToSplit, const char* delimiter)
 template<class STRLIST>
 std::string join(const STRLIST& cnt, const std::string& delim)
 {
+    std::string retStr;
     bool firstTime{true};
-    std::stringstream ss;
     for (auto& v : cnt) {
-        if (not firstTime) ss << delim;
+        if (not firstTime) retStr += delim;
         else firstTime = false;
-        ss << v;
+        retStr += v;
     }
-    return ss.str();
+    return retStr;
 }
 
 template<class Vector>
