@@ -1,7 +1,7 @@
 /*
  * ct_parser_html.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -30,14 +30,14 @@
 #include <cassert>
 
 namespace {
-std::vector<std::string> split_rednotebook_html_nodes(const std::string& input) 
+std::vector<std::string> split_rednotebook_html_nodes(const std::string& input)
 {
     static const auto reg = Glib::Regex::create("<p>[\\S\\s]<span id=\"(?:[12]\\d{3}-(?:0[1-9]|1[0-2])-(?:0[1-9]|[12]\\d|3[01]))\"></span>[\\S\\s]</p>");
     std::vector<std::string> out = reg->split(input);
     return out;
 }
 
-std::optional<std::string> match_rednotebook_title_h1(const Glib::ustring& input) 
+std::optional<std::string> match_rednotebook_title_h1(const Glib::ustring& input)
 {
     static const auto reg = Glib::Regex::create("<h1>([\\S\\s]*?)<\\/h1>");
 
@@ -53,7 +53,7 @@ std::optional<std::string> match_rednotebook_title_h1(const Glib::ustring& input
 
 
 
-std::shared_ptr<xmlpp::Document> html_to_xml_doc(const std::string& contents, CtConfig* config) 
+std::shared_ptr<xmlpp::Document> html_to_xml_doc(const std::string& contents, CtConfig* config)
 {
     CtHtml2Xml parser{config};
     parser.feed(contents);
@@ -140,7 +140,7 @@ const std::set<std::string> CtHtml2Xml::HTML_A_TAGS{
     CtConst::TAG_PROP_VAL_H4, CtConst::TAG_PROP_VAL_H5, CtConst::TAG_PROP_VAL_H6,
     "span", "font"};
 
-CtHtml2Xml::CtHtml2Xml(CtConfig* config) : _config(config)
+CtHtml2Xml::CtHtml2Xml(CtConfig* config) : _pCtConfig{config}
 {
 }
 
@@ -178,7 +178,7 @@ void CtHtml2Xml::feed(const std::string& html)
     }
 
 
-    _rich_text_save_pending(); 
+    _rich_text_save_pending();
 }
 
 void CtHtml2Xml::handle_starttag(std::string_view tag, const char** atts)
@@ -237,7 +237,7 @@ void CtHtml2Xml::handle_starttag(std::string_view tag, const char** atts)
         return;
     }
     _parsing_valid_tag = true;
-    
+
     if (HTML_A_TAGS.count(tag.begin())) _html_a_tag_counter += 1;
     if (_state == ParserState::WAIT_BODY)
     {
@@ -310,20 +310,20 @@ void CtHtml2Xml::handle_starttag(std::string_view tag, const char** atts)
         }
         else if (tag == "br") _rich_text_serialize(CtConst::CHAR_NEWLINE);
         else if (tag == "ol")  { _list_type = 'o'; _list_num = 1; }
-        else if (tag == "ul")  { 
-            _list_type = 'u'; 
+        else if (tag == "ul")  {
+            _list_type = 'u';
             _list_num = 0;
-            if (_list_level < static_cast<int>(_config->charsListbul.size()) - 1) _list_level++;
+            if (_list_level < static_cast<int>(_pCtConfig->charsListbul.size()) - 1) _list_level++;
         }
         else if (tag == "li") {
             if (_list_type == 'u') {
                 if (_list_level < 0) {
                     // A <ul> _should_ have appeared before this
                     // but ok, use just some default list
-                    _rich_text_serialize(_config->charsListbul[0] + CtConst::CHAR_SPACE);
+                    _rich_text_serialize(_pCtConfig->charsListbul[0] + CtConst::CHAR_SPACE);
                 }
                 else {
-                    _rich_text_serialize(_config->charsListbul[_list_level] + CtConst::CHAR_SPACE);
+                    _rich_text_serialize(_pCtConfig->charsListbul[_list_level] + CtConst::CHAR_SPACE);
                 }
             }
             else {
@@ -377,7 +377,7 @@ void CtHtml2Xml::handle_starttag(std::string_view tag, const char** atts)
         else if (tag == "ol" && _html_td_tag_open) { _list_type = 'o'; _list_num = 1; }
         else if (tag == "ul" && _html_td_tag_open) { _list_type = 'u'; _list_num = 0; }
         else if (tag == "li" && _html_td_tag_open) {
-            if (_list_type == 'u') _table.back().back().text += _config->charsListbul[0] + CtConst::CHAR_SPACE;
+            if (_list_type == 'u') _table.back().back().text += _pCtConfig->charsListbul[0] + CtConst::CHAR_SPACE;
             else {
                 _table.back().back().text += std::to_string(_list_num) + ". ";
                 _list_num += 1;
@@ -660,36 +660,35 @@ void CtHtml2Xml::_insert_table()
     if (_table.empty()) return;
 
     // add more cells for rowspan > 1
-    for (auto& row: _table)
+    for (auto& row : _table)
         for (auto iter = row.begin(); iter != row.end(); ++ iter)
             if (iter->rowspan > 1)
                 row.insert(std::next(iter), iter->rowspan - 1, {1, ""});
     // find bigger row size
     size_t row_len = 0;
-    for (auto& row: _table)
+    for (auto& row : _table)
         row_len  = std::max(row_len, row.size());
     // add more cell for rowspan = 0
-    for (auto& row: _table)
+    for (auto& row : _table)
         for (auto iter = row.begin(); iter != row.end(); ++ iter)
             if (iter->rowspan == 0 && row.size() < row_len)
                 row.insert(std::next(iter), row_len - row.size(), {1, ""});
     // add more cell just in case
-    for (auto& row: _table)
+    for (auto& row : _table)
         if (row.size() < row_len)
             row.insert(row.end(), row_len - row.size(), {1, ""});
 
-
-    std::vector<std::vector<Glib::ustring>> rows;
-    rows.reserve(_table.size());
-    for (const auto& row: _table)
-    {
-        rows.push_back(std::vector<Glib::ustring>{});
-        rows.back().reserve(row.size());
-        for (const auto& cell: row)
-            rows.back().push_back(str::trim(cell.text));
+    std::vector<std::vector<Glib::ustring>> table_matrix;
+    table_matrix.reserve(_table.size());
+    for (const auto& row : _table) {
+        table_matrix.push_back(std::vector<Glib::ustring>{});
+        table_matrix.back().reserve(row.size());
+        for (const auto& cell : row)
+            table_matrix.back().push_back(str::trim(cell.text));
     }
 
-    CtXmlHelper::table_to_xml(_slot_root, rows, _char_offset, CtConst::TAG_PROP_VAL_LEFT, _config->tableColWidthDefault, "");
+    const bool is_light = table_matrix.size() > 0 and table_matrix.size() * table_matrix.front().size() > static_cast<unsigned>(_pCtConfig->tableCellsGoLight);
+    CtXmlHelper::table_to_xml(_slot_root, table_matrix, _char_offset, CtConst::TAG_PROP_VAL_LEFT, _pCtConfig->tableColWidthDefault, "", is_light);
 
     _char_offset += 1;
 }
@@ -759,15 +758,12 @@ void CtHtml2Xml::_rich_text_save_pending()
 }
 
 
-void CtRedNotebookParser::feed(std::istream& in) 
+void CtRedNotebookParser::feed(const std::string& in)
 {
-    std::ostringstream ss;
-    ss << in.rdbuf();
-    _feed_str(ss.str());
+    _feed_str(in);
 }
 
-
-void CtRedNotebookParser::_feed_str(const std::string& in) 
+void CtRedNotebookParser::_feed_str(const std::string& in)
 {
     auto pages = split_rednotebook_html_nodes(in);
     pages.erase(pages.cbegin(), pages.cbegin() + 1);
@@ -779,7 +775,7 @@ void CtRedNotebookParser::_feed_str(const std::string& in)
 }
 
 
-void CtRedNotebookParser::_add_node(std::string&& name, const std::string& contents) 
+void CtRedNotebookParser::_add_node(std::string&& name, const std::string& contents)
 {
     node new_node{
         std::move(name), html_to_xml_doc(contents, _ct_config)
@@ -788,27 +784,23 @@ void CtRedNotebookParser::_add_node(std::string&& name, const std::string& conte
     _nodes.emplace_back(std::move(new_node));
 }
 
-
-void CtNoteCaseHTMLParser::feed(std::istream& input) {
-    std::stringstream ss;
-    ss << input.rdbuf();
-    _feed_str(ss.str());
+void CtNoteCaseHTMLParser::feed(const std::string& input)
+{
+    _feed_str(input);
 }
 
-
 void CtNoteCaseHTMLParser::_feed_str(const std::string& str) {
     auto split_nodes = _split_notecase_html_nodes(str);
-    std::vector<node> new_nodes = _generate_notecase_nodes(std::make_move_iterator(split_nodes.begin()), 
+    std::vector<node> new_nodes = _generate_notecase_nodes(std::make_move_iterator(split_nodes.begin()),
                                                         std::make_move_iterator(split_nodes.end()), _ct_config);
     _nodes.insert(_nodes.cend(), std::make_move_iterator(new_nodes.cbegin()), std::make_move_iterator(new_nodes.cend()));
 }
 
-
-std::vector<CtNoteCaseHTMLParser::notecase_split_output> CtNoteCaseHTMLParser::_split_notecase_html_nodes(const std::string& input) 
-{   
+std::vector<CtNoteCaseHTMLParser::notecase_split_output> CtNoteCaseHTMLParser::_split_notecase_html_nodes(const std::string& input)
+{
     /*
      * Each "note" is seperated by a DT with font weight as bold and then a link tag with a 22 length string
-     * (I assume the string is some sort of hash since it is just a jumble of characters, I guess it is used 
+     * (I assume the string is some sort of hash since it is just a jumble of characters, I guess it is used
      * as anchor points for links). Links are not handled because they require a pro version.
      */
     static const auto reg = Glib::Regex::create("<DT style=\"font-weight: bold;\"><A name=\"[a-z|A-Z|0-9]{22}\"></A>([\\S\\s]*?)</DT>");
