@@ -1,7 +1,7 @@
 /*
  * ct_storage_control.h
  *
- * Copyright 2009-2020
+ * Copyright 2009-2022
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -25,6 +25,7 @@
 
 #include "ct_types.h"
 #include <glibmm/miscutils.h>
+#include <thread>
 
 class CtMainWin;
 class CtStorageControl
@@ -42,18 +43,20 @@ public:
                                      const CtExporting exporting = CtExporting::NONE,
                                      const int start_offset = 0,
                                      const int end_offset = -1);
+    static bool document_integrity_check_pass(CtMainWin* pCtMainWin,
+                                              const fs::path& file_path,
+                                              Glib::ustring& error);
+    virtual ~CtStorageControl();
 
 public:
     bool save(bool need_vacuum, Glib::ustring& error);
 
 private:
-    CtStorageControl() = default;
+    CtStorageControl(CtMainWin* pCtMainWin);
 
     static fs::path _extract_file(CtMainWin* pCtMainWin, const fs::path& file_path, Glib::ustring& password);
     static bool     _package_file(const fs::path& file_from, const fs::path& file_to, const Glib::ustring& password);
 
-    void _put_in_backup(const fs::path& main_backup);
-
 public:
     Glib::RefPtr<Gsv::Buffer> get_delayed_text_buffer(const gint64& node_id,
                                                       const std::string& syntax,
@@ -82,13 +85,29 @@ public:
     void add_nodes_from_storage(const fs::path& path, Gtk::TreeIter parent_iter);
 
 private:
-    CtMainWin*                       _pCtMainWin{nullptr};
+    CtMainWin*                 const _pCtMainWin;
+    CtConfig*                  const _pCtConfig;
     fs::path                         _file_path;
     time_t                           _mod_time{0};
     Glib::ustring                    _password;
     fs::path                         _extracted_file_path;
     std::unique_ptr<CtStorageEntity> _storage;
     CtStorageSyncPending             _syncPending;
+
+private:
+    struct CtBackupEncryptData {
+        bool needBackup;
+        bool needEncrypt;
+        std::string main_backup;
+        std::string file_path;
+        std::string password;
+        std::string extracted_copy;
+    };
+    std::unique_ptr<std::thread> _pThreadBackupEncrypt;
+    ThreadSafeDEQueue<std::shared_ptr<CtBackupEncryptData>,9> _backupEncryptDEQueue;
+    static void _staticBackupEncryptThread(CtStorageControl* pStorageCtrl) { pStorageCtrl->_backupEncryptThread(); }
+    void _backupEncryptThread();
+    bool _backupEncryptKeepGoing{true};
 };
 
 class CtImagePng;
