@@ -1,7 +1,7 @@
 /*
  * ct_storage_xml.cc
  *
- * Copyright 2009-2022
+ * Copyright 2009-2023
  * Giuseppe Penone <giuspen@gmail.com>
  * Evgenii Gurianov <https://github.com/txe>
  *
@@ -51,45 +51,47 @@ void CtStorageXml::test_connection()
 
 bool CtStorageXml::populate_treestore(const fs::path& file_path, Glib::ustring& error)
 {
-    try
-    {
+    try {
         // open file
         auto parser = _get_parser(file_path);
 
         // read bookmarks
-        for (xmlpp::Node* xml_node :  parser->get_document()->get_root_node()->get_children("bookmarks"))
-        {
+        for (xmlpp::Node* xml_node :  parser->get_document()->get_root_node()->get_children("bookmarks")) {
             Glib::ustring bookmarks_csv = static_cast<xmlpp::Element*>(xml_node)->get_attribute_value("list");
-            for (gint64& nodeId : CtStrUtil::gstring_split_to_int64(bookmarks_csv.c_str(), ","))
-                _pCtMainWin->get_tree_store().bookmarks_add(nodeId);
+            for (auto nodeId : CtStrUtil::gstring_split_to_int64(bookmarks_csv.c_str(), ",")) {
+                if (not _isDryRun) {
+                    _pCtMainWin->get_tree_store().bookmarks_add(nodeId);
+                }
+            }
         }
 
         // read nodes
         std::list<CtTreeIter> nodes_with_duplicated_id;
         std::function<void(xmlpp::Element*, const gint64, Gtk::TreeIter)> nodes_from_xml;
         nodes_from_xml = [&](xmlpp::Element* xml_element, const gint64 sequence, Gtk::TreeIter parent_iter) {
-            bool has_duplicated_id = false;
+            bool has_duplicated_id{false};
             Gtk::TreeIter new_iter = _node_from_xml(xml_element, sequence, parent_iter, -1, &has_duplicated_id);
-            if (has_duplicated_id) {
+            if (has_duplicated_id and not _isDryRun) {
                 nodes_with_duplicated_id.push_back(_pCtMainWin->get_tree_store().to_ct_tree_iter(new_iter));
             }
             gint64 child_sequence = 0;
-            for (xmlpp::Node* xml_node : xml_element->get_children("node"))
+            for (xmlpp::Node* xml_node : xml_element->get_children("node")) {
                 nodes_from_xml(static_cast<xmlpp::Element*>(xml_node), ++child_sequence, new_iter);
+            }
         };
         gint64 sequence = 0;
-        for (xmlpp::Node* xml_node: parser->get_document()->get_root_node()->get_children("node"))
+        for (xmlpp::Node* xml_node : parser->get_document()->get_root_node()->get_children("node")) {
             nodes_from_xml(static_cast<xmlpp::Element*>(xml_node), ++sequence, Gtk::TreeIter());
+        }
 
         // fixes duplicated ids by setting new ids
-        for (auto& node: nodes_with_duplicated_id) {
+        for (auto& node : nodes_with_duplicated_id) {
             node.set_node_id(_pCtMainWin->get_tree_store().node_id_get());
         }
 
         return true;
     }
-    catch (std::exception& e)
-    {
+    catch (std::exception& e) {
         error = std::string("CtDocXmlStorage got exception: ") + e.what();
         return false;
     }
@@ -102,8 +104,7 @@ bool CtStorageXml::save_treestore(const fs::path& file_path,
                                   const int start_offset/*= 0*/,
                                   const int end_offset/*=-1*/)
 {
-    try
-    {
+    try {
         xmlpp::Document xml_doc;
         xml_doc.create_root_node(CtConst::APP_NAME);
 
@@ -121,8 +122,7 @@ bool CtStorageXml::save_treestore(const fs::path& file_path,
         if ( CtExporting::NONE == exporting or
              CtExporting::ALL_TREE == exporting ) {
             auto ct_tree_iter = _pCtMainWin->get_tree_store().get_ct_iter_first();
-            while (ct_tree_iter)
-            {
+            while (ct_tree_iter) {
                 _nodes_to_xml(&ct_tree_iter, xml_doc.get_root_node(), &storage_cache, exporting, start_offset, end_offset);
                 ct_tree_iter++;
             }
@@ -137,8 +137,7 @@ bool CtStorageXml::save_treestore(const fs::path& file_path,
 
         return true;
     }
-    catch (std::exception& e)
-    {
+    catch (std::exception& e) {
         error = e.what();
         return false;
     }
@@ -186,13 +185,16 @@ Glib::RefPtr<Gsv::Buffer> CtStorageXml::get_delayed_text_buffer(const gint64& no
 
 Gtk::TreeIter CtStorageXml::_node_from_xml(xmlpp::Element* xml_element, gint64 sequence, Gtk::TreeIter parent_iter, gint64 new_id, bool* has_duplicated_id)
 {
-    if (has_duplicated_id) *has_duplicated_id = false;
-
+    if (has_duplicated_id) {
+        *has_duplicated_id = false;
+    }
     CtNodeData node_data;
-    if (new_id == -1)
+    if (new_id == -1) {
         node_data.nodeId = CtStrUtil::gint64_from_gstring(xml_element->get_attribute_value("unique_id").c_str());
-    else
+    }
+    else {
         node_data.nodeId = new_id;
+    }
     node_data.name = xml_element->get_attribute_value("name");
     node_data.syntax = xml_element->get_attribute_value("prog_lang");
     node_data.tags = xml_element->get_attribute_value("tags");
@@ -205,17 +207,20 @@ Gtk::TreeIter CtStorageXml::_node_from_xml(xmlpp::Element* xml_element, gint64 s
     node_data.tsCreation = CtStrUtil::gint64_from_gstring(xml_element->get_attribute_value("ts_creation").c_str());
     node_data.tsLastSave = CtStrUtil::gint64_from_gstring(xml_element->get_attribute_value("ts_lastsave").c_str());
     node_data.sequence = sequence;
-    if (new_id == -1)
-    {
+
+    if (_isDryRun) {
+        return Gtk::TreeIter{};
+    }
+
+    if (new_id == -1) {
         if (_delayed_text_buffers.count(node_data.nodeId) != 0) {
             spdlog::debug("node has duplicated id {}, will be fixed", node_data.nodeId);
             if (has_duplicated_id) *has_duplicated_id = true;
             // create buffer now because we cannot put a duplicate id in _delayed_text_buffers
             // the id will be fixed on top level code
-            node_data.rTextBuffer = CtStorageXmlHelper(_pCtMainWin).create_buffer_and_widgets_from_xml(xml_element, node_data.syntax, node_data.anchoredWidgets, nullptr, -1);
+            node_data.rTextBuffer = CtStorageXmlHelper{_pCtMainWin}.create_buffer_and_widgets_from_xml(xml_element, node_data.syntax, node_data.anchoredWidgets, nullptr, -1);
         }
-        else
-        {
+        else {
             // because of widgets which are slow to insert for now, delay creating buffers
             // save node data in a separate document
             auto node_buffer = std::make_shared<xmlpp::Document>();
@@ -335,8 +340,11 @@ Glib::RefPtr<Gsv::Buffer> CtStorageXmlHelper::create_buffer_and_widgets_from_xml
     return buffer;
 }
 
-void CtStorageXmlHelper::get_text_buffer_one_slot_from_xml(Glib::RefPtr<Gsv::Buffer> buffer, xmlpp::Node* slot_node,
-                                                 std::list<CtAnchoredWidget*>& widgets, Gtk::TextIter* text_insert_pos, int force_offset)
+void CtStorageXmlHelper::get_text_buffer_one_slot_from_xml(Glib::RefPtr<Gsv::Buffer> buffer,
+                                                           xmlpp::Node* slot_node,
+                                                           std::list<CtAnchoredWidget*>& widgets,
+                                                           Gtk::TextIter* text_insert_pos,
+                                                           int force_offset)
 {
     xmlpp::Element* slot_element = static_cast<xmlpp::Element*>(slot_node);
     Glib::ustring slot_element_name = slot_element->get_name();
@@ -348,22 +356,25 @@ void CtStorageXmlHelper::get_text_buffer_one_slot_from_xml(Glib::RefPtr<Gsv::Buf
     else if (slot_element_name == "table")       slot_type = SlotType::Table;
     else if (slot_element_name == "codebox")     slot_type = SlotType::Codebox;
 
-    if (slot_type == SlotType::RichText)
-    {
+    if (slot_type == SlotType::RichText) {
         _add_rich_text_from_xml(buffer, slot_element, text_insert_pos);
     }
-    else if (slot_type != SlotType::None)
-    {
+    else if (slot_type != SlotType::None) {
         const int char_offset = force_offset != -1 ? force_offset : std::stoi(slot_element->get_attribute_value("char_offset"));
         Glib::ustring justification = slot_element->get_attribute_value(CtConst::TAG_JUSTIFICATION);
         if (justification.empty()) justification = CtConst::TAG_PROP_VAL_LEFT;
 
         CtAnchoredWidget* widget{nullptr};
-        if (slot_type == SlotType::Image)        widget = _create_image_from_xml(slot_element, char_offset, justification);
-        else if (slot_type == SlotType::Table)   widget = _create_table_from_xml(slot_element, char_offset, justification);
-        else if (slot_type == SlotType::Codebox) widget = _create_codebox_from_xml(slot_element, char_offset, justification);
-        if (widget)
-        {
+        if (slot_type == SlotType::Image) {
+            widget = _create_image_from_xml(slot_element, char_offset, justification);
+        }
+        else if (slot_type == SlotType::Table) {
+            widget = _create_table_from_xml(slot_element, char_offset, justification);
+        }
+        else if (slot_type == SlotType::Codebox) {
+            widget = _create_codebox_from_xml(slot_element, char_offset, justification);
+        }
+        if (widget) {
             widget->insertInTextBuffer(buffer);
             widgets.push_back(widget);
         }
@@ -382,45 +393,61 @@ Glib::RefPtr<Gsv::Buffer> CtStorageXmlHelper::create_buffer_no_widgets(const Gli
 
 bool CtStorageXmlHelper::populate_table_matrix(CtTableMatrix& tableMatrix,
                                                const char* xml_content,
-                                               CtTableColWidths& tableColWidths)
+                                               CtTableColWidths& tableColWidths,
+                                               bool& is_light)
 {
     xmlpp::DomParser parser;
     if (CtXmlHelper::safe_parse_memory(parser, xml_content)) {
-        populate_table_matrix(tableMatrix, parser.get_document()->get_root_node(), tableColWidths);
+        populate_table_matrix(tableMatrix,
+                              parser.get_document()->get_root_node(),
+                              tableColWidths,
+                              is_light);
         return true;
     }
     return false;
 }
 
-void CtStorageXmlHelper::populate_table_matrix(CtTableMatrix& tableMatrix, xmlpp::Element* xml_element, CtTableColWidths& tableColWidths)
+void CtStorageXmlHelper::populate_table_matrix(CtTableMatrix& tableMatrix,
+                                               xmlpp::Element* xml_element,
+                                               CtTableColWidths& tableColWidths,
+                                               bool& is_light)
 {
-    for (xmlpp::Node* pNodeRow : xml_element->get_children("row"))
-    {
+    const Glib::ustring isLightStr = xml_element->get_attribute_value("is_light");
+    if (not isLightStr.empty()) {
+        is_light = CtStrUtil::is_str_true(isLightStr);
+    }
+    for (xmlpp::Node* pNodeRow : xml_element->get_children("row")) {
         tableMatrix.push_back(CtTableRow{});
-        for (xmlpp::Node* pNodeCell : pNodeRow->get_children("cell"))
-        {
+        for (xmlpp::Node* pNodeCell : pNodeRow->get_children("cell")) {
             xmlpp::TextNode* pTextNode = static_cast<xmlpp::Element*>(pNodeCell)->get_child_text();
             const Glib::ustring textContent = pTextNode ? pTextNode->get_content() : "";
-            tableMatrix.back().push_back(new CtTextCell{_pCtMainWin, textContent, CtConst::TABLE_CELL_TEXT_ID});
+            if (is_light) {
+                tableMatrix.back().push_back(new Glib::ustring{textContent});
+            }
+            else {
+                tableMatrix.back().push_back(new CtTextCell{_pCtMainWin, textContent, CtConst::TABLE_CELL_TEXT_ID});
+            }
         }
     }
     tableMatrix.insert(tableMatrix.begin(), tableMatrix.back());
     tableMatrix.pop_back();
-    auto colWidthsStr = xml_element->get_attribute_value("col_widths");
+    const Glib::ustring colWidthsStr = xml_element->get_attribute_value("col_widths");
     if (not colWidthsStr.empty()) {
         tableColWidths = CtStrUtil::gstring_split_to_int(colWidthsStr.c_str(), ",");
     }
 }
 
-/*static*/ void CtStorageXmlHelper::save_buffer_no_widgets_to_xml(xmlpp::Element* p_node_parent,
-                                                                  Glib::RefPtr<Gtk::TextBuffer> rBuffer,
-                                                                  int start_offset,
-                                                                  int end_offset,
-                                                                  const gchar change_case)
+void CtStorageXmlHelper::save_buffer_no_widgets_to_xml(xmlpp::Element* p_node_parent,
+                                                       Glib::RefPtr<Gtk::TextBuffer> rBuffer,
+                                                       int start_offset,
+                                                       int end_offset,
+                                                       const gchar change_case)
 {
     CtTextIterUtil::SerializeFunc rich_txt_serialize = [&](Gtk::TextIter& start_iter,
                                                            Gtk::TextIter& end_iter,
-                                                           CtCurrAttributesMap& curr_attributes) {
+                                                           CtCurrAttributesMap& curr_attributes,
+                                                           CtListInfo*/*pCurrListInfo*/)
+    {
         xmlpp::Element* p_rich_text_node = p_node_parent->add_child("rich_text");
         for (const auto& map_iter : curr_attributes)
         {
@@ -437,7 +464,7 @@ void CtStorageXmlHelper::populate_table_matrix(CtTableMatrix& tableMatrix, xmlpp
         p_rich_text_node->add_child_text(slot_text);
     };
 
-    CtTextIterUtil::generic_process_slot(start_offset, end_offset, rBuffer, rich_txt_serialize);
+    CtTextIterUtil::generic_process_slot(_pCtMainWin->get_ct_config(), start_offset, end_offset, rBuffer, rich_txt_serialize);
 }
 
 void CtStorageXmlHelper::_add_rich_text_from_xml(Glib::RefPtr<Gsv::Buffer> buffer, xmlpp::Element* xml_element, Gtk::TextIter* text_insert_pos)
@@ -485,7 +512,9 @@ CtAnchoredWidget* CtStorageXmlHelper::_create_image_from_xml(xmlpp::Element* xml
     return new CtImagePng{_pCtMainWin, rawBlob, link, charOffset, justification};
 }
 
-CtAnchoredWidget* CtStorageXmlHelper::_create_codebox_from_xml(xmlpp::Element* xml_element, int charOffset, const Glib::ustring& justification)
+CtAnchoredWidget* CtStorageXmlHelper::_create_codebox_from_xml(xmlpp::Element* xml_element,
+                                                               int charOffset,
+                                                               const Glib::ustring& justification)
 {
     xmlpp::TextNode* pTextNode = xml_element->get_child_text();
     const Glib::ustring textContent = pTextNode ? pTextNode->get_content() : "";
@@ -496,35 +525,41 @@ CtAnchoredWidget* CtStorageXmlHelper::_create_codebox_from_xml(xmlpp::Element* x
     const bool highlightBrackets = CtStrUtil::is_str_true(xml_element->get_attribute_value("highlight_brackets"));
     const bool showLineNumbers = CtStrUtil::is_str_true(xml_element->get_attribute_value("show_line_numbers"));
 
-    return new CtCodebox(_pCtMainWin,
-                        textContent,
-                        syntaxHighlighting,
-                        frameWidth,
-                        frameHeight,
-                        charOffset,
-                        justification,
-                        widthInPixels,
-                        highlightBrackets,
-                        showLineNumbers);
+    return new CtCodebox{_pCtMainWin,
+                         textContent,
+                         syntaxHighlighting,
+                         frameWidth,
+                         frameHeight,
+                         charOffset,
+                         justification,
+                         widthInPixels,
+                         highlightBrackets,
+                         showLineNumbers};
 }
 
-CtAnchoredWidget* CtStorageXmlHelper::_create_table_from_xml(xmlpp::Element* xml_element, int charOffset, const Glib::ustring& justification)
+CtAnchoredWidget* CtStorageXmlHelper::_create_table_from_xml(xmlpp::Element* xml_element,
+                                                             int charOffset,
+                                                             const Glib::ustring& justification)
 {
     const int colWidthDefault = std::stoi(xml_element->get_attribute_value("col_max"));
 
     CtTableMatrix tableMatrix;
     CtTableColWidths tableColWidths;
-    populate_table_matrix(tableMatrix, xml_element, tableColWidths);
-
-    return new CtTable(_pCtMainWin, tableMatrix, colWidthDefault, charOffset, justification, tableColWidths);
+    bool is_light{false};
+    populate_table_matrix(tableMatrix, xml_element, tableColWidths, is_light);
+    if (is_light) {
+        return new CtTableLight{_pCtMainWin, tableMatrix, colWidthDefault, charOffset, justification, tableColWidths};
+    }
+    return new CtTableHeavy{_pCtMainWin, tableMatrix, colWidthDefault, charOffset, justification, tableColWidths};
 }
 
 void CtXmlHelper::table_to_xml(xmlpp::Element* p_parent,
                                const std::vector<std::vector<Glib::ustring>>& rows,
-                               int char_offset,
-                               Glib::ustring justification,
-                               int defaultWidth,
-                               Glib::ustring colWidths)
+                               const int char_offset,
+                               const Glib::ustring justification,
+                               const int defaultWidth,
+                               const Glib::ustring colWidths,
+                               const bool is_light)
 {
     xmlpp::Element* p_table_node = p_parent->add_child("table");
     p_table_node->set_attribute("char_offset", std::to_string(char_offset));
@@ -532,6 +567,9 @@ void CtXmlHelper::table_to_xml(xmlpp::Element* p_parent,
     p_table_node->set_attribute("col_min", std::to_string(defaultWidth)); // todo get rid of column min
     p_table_node->set_attribute("col_max", std::to_string(defaultWidth));
     p_table_node->set_attribute("col_widths", colWidths);
+    if (is_light) {
+        p_table_node->set_attribute("is_light", "1");
+    }
 
     auto row_to_xml = [&](const std::vector<Glib::ustring>& tableRow) {
         xmlpp::Element* row_element = p_table_node->add_child("row");
@@ -543,8 +581,7 @@ void CtXmlHelper::table_to_xml(xmlpp::Element* p_parent,
 
     // put header at the end
     bool is_header = true;
-    for (auto& row: rows)
-    {
+    for (auto& row : rows) {
         if (is_header) { is_header = false; }
         else row_to_xml(row);
     }
